Package: iAR
Type: Package
Title: Irregularly Observed Autoregressive Models
Version: 1.3.4
Date: 2026-05-14
Authors@R: c(
    person("Elorrieta", "Felipe", email = "felipe.elorrieta@usach.cl", role = c("aut","cre")),
    person("Ojeda", "Cesar", email = "cesar.ojeda@correounivalle.edu.co", role = "aut"),
    person("Eyheramendy", "Susana", email = "susana.eyheramendy@gmail.com", role = "aut"),
    person("Palma", "Wilfredo", email = "wilfredo.palma@gmail.com", role = "aut"))
Maintainer: Elorrieta Felipe <felipe.elorrieta@usach.cl>
Description: Data sets, functions and scripts with examples to implement autoregressive models for irregularly observed time series. The models available in this package are the irregular autoregressive model (Eyheramendy et al.(2018) <doi:10.1093/mnras/sty2487>), the complex irregular autoregressive model (Elorrieta et al.(2019) <doi:10.1051/0004-6361/201935560>) and the bivariate irregular autoregressive model (Elorrieta et al.(2021) <doi:10.1093/mnras/stab1216>).
License: GPL-2
Depends: R (>= 3.5.0)
Imports: Rcpp (>= 1.0.7),ggplot2,stats, Rdpack, S7, zoo
RdMacros: Rdpack
Suggests: arfima, knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
URL: https://github.com/felipeelorrieta
LinkingTo: Rcpp,RcppArmadillo
LazyData: true
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-05-16 15:50:25 UTC; felocarril
Author: Elorrieta Felipe [aut, cre],
  Ojeda Cesar [aut],
  Eyheramendy Susana [aut],
  Palma Wilfredo [aut]
Repository: CRAN
Date/Publication: 2026-05-16 17:00:03 UTC
Config/roxygen2/version: 7.3.3
Config/testthat/edition: 3
RoxygenNote: 7.3.3
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-16 21:30:35 UTC; unix
Archs: iAR.so.dSYM

Package: iAR
Type: Package
Title: Irregularly Observed Autoregressive Models
Version: 1.3.4
Date: 2026-05-14
Authors@R: c(
    person("Elorrieta", "Felipe", email = "felipe.elorrieta@usach.cl", role = c("aut","cre")),
    person("Ojeda", "Cesar", email = "cesar.ojeda@correounivalle.edu.co", role = "aut"),
    person("Eyheramendy", "Susana", email = "susana.eyheramendy@gmail.com", role = "aut"),
    person("Palma", "Wilfredo", email = "wilfredo.palma@gmail.com", role = "aut"))
Maintainer: Elorrieta Felipe <felipe.elorrieta@usach.cl>
Description: Data sets, functions and scripts with examples to implement autoregressive models for irregularly observed time series. The models available in this package are the irregular autoregressive model (Eyheramendy et al.(2018) <doi:10.1093/mnras/sty2487>), the complex irregular autoregressive model (Elorrieta et al.(2019) <doi:10.1051/0004-6361/201935560>) and the bivariate irregular autoregressive model (Elorrieta et al.(2021) <doi:10.1093/mnras/stab1216>).
License: GPL-2
Depends: R (>= 3.5.0)
Imports: Rcpp (>= 1.0.7),ggplot2,stats, Rdpack, S7, zoo
RdMacros: Rdpack
Suggests: arfima, knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
URL: https://github.com/felipeelorrieta
LinkingTo: Rcpp,RcppArmadillo
LazyData: true
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-05-16 15:50:25 UTC; felocarril
Author: Elorrieta Felipe [aut, cre],
  Ojeda Cesar [aut],
  Eyheramendy Susana [aut],
  Palma Wilfredo [aut]
Repository: CRAN
Date/Publication: 2026-05-16 17:00:03 UTC
Config/roxygen2/version: 7.3.3
Config/testthat/edition: 3
RoxygenNote: 7.3.3
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-16 17:15:53 UTC; unix
Archs: iAR.so.dSYM

# Generated by roxygen2: do not edit by hand

export(BiAR)
export(CiAR)
export(fit)
export(forecast)
export(gentime)
export(harmonicfit)
export(iAR)
export(interpolation)
export(kalman)
export(loglik)
export(multidata)
export(pairingits)
export(phase)
export(plot)
export(plot_fit)
export(plot_forecast)
export(sim)
export(summary)
export(unidata)
export(utilities)
import(Rcpp)
import(S7)
import(ggplot2)
import(zoo)
importFrom(Rdpack,reprompt)
importFrom(stats,anova)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(stats,nlminb)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,residuals)
importFrom(stats,rexp)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,rt)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(zoo,plot.zoo)
importFrom(zoo,zoo)
useDynLib(iAR, .registration = TRUE)

# iAR <img src="https://pacha.dev/LSTS/hexicon.svg" width=150 align="right" alt="sticker"/>

## Description

Data sets, functions and scripts with examples to implement autoregressive models for irregularly observed time series. The models available in this package are the irregular autoregressive (iAR) model [Eyheramendy et al.(2018)](https://doi.org/10.1093/mnras/sty2487), the complex irregular autoregressive (CiAR) model [Elorrieta et al.(2019)](https://doi.org/10.1051/0004-6361/201935560) and the bivariate irregular autoregressive (BiAR) model [Elorrieta et al.(2021)](https://doi.org/10.1093/mnras/stab1216).

## Installation

You can install the released version of iAR from Github with:

``` r
remotes::install_github("felipeelorrieta/iAR-package")
```

## Code of Conduct
  
Please note that the iAR project is released with a [Contributor Code of Conduct](https://contributor-covenant.org/version/2/0/CODE_OF_CONDUCT.html). By contributing to this project, you agree to abide by its terms.

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  fig.width = 7,
  fig.height = 4
)

## ----load-pkg-----------------------------------------------------------------
library(iAR)

## ----gentime------------------------------------------------------------------
set.seed(2847)

# x defaults to NULL — no setup needed
times <- gentime(n = 100)@times

cat("Number of observations:", length(times), "\n")
cat("Time span: [", round(min(times), 1), ",", round(max(times), 1), "]\n")
cat("Mean gap between observations:", round(mean(diff(times)), 2), "\n")

## ----gentime-plot, fig.cap = "Distribution of inter-observation gaps (log scale) for 100 irregular times drawn from an exponential mixture."----
gaps <- diff(times)
hist(log1p(gaps),
     main = "Inter-observation gaps (log1p scale)",
     xlab = "log(1 + gap)", col = "steelblue", border = "white",
     breaks = 20)

## ----iar-sim------------------------------------------------------------------
set.seed(2847)
times <- gentime(n = 100)@times

# Build the model: family = "norm", phi = 0.9
m_iar <- iAR(family = "norm", times = times, coef = 0.9)
m_iar <- sim(m_iar)

head(m_iar@series)

## ----iar-plot, fig.cap = "Simulated iAR series (phi = 0.9). The irregular spacing is visible in the unequal distances along the x-axis."----
plot(m_iar, type = "o",main = "Simulated iAR series (phi = 0.9)",ylab="Values",xlab="Time",pch=20)

## ----iar-prep-----------------------------------------------------------------
y       <- m_iar@series
y_std   <- y / sd(y)           # unit variance
m_iar@series     <- y_std
m_iar@series_esd <- rep(0, length(times))

## ----iar-loglik---------------------------------------------------------------
m_loglik <- loglik(m_iar)
cat("loglik estimate: phi =", round(m_loglik@coef, 4), "\n")
cat("Log-likelihood: ", round(m_loglik@loglik, 4), "\n")

## ----iar-kalman---------------------------------------------------------------
m_kalman <- kalman(m_iar)
cat("kalman estimate: phi =", round(m_kalman@coef, 4), "\n")
cat("Kalman log-lik: ", round(m_kalman@kalmanlik, 4), "\n")

## ----iar-hessian--------------------------------------------------------------
m_h <- iAR(family = "norm", times = times, coef = 0.9, hessian = TRUE)
m_h@series     <- y_std
m_h@series_esd <- rep(0, length(times))
m_h <- loglik(m_h)

# Summary contains the coefficient, SE, p-value, information criteria, and residual diagnostics
summary(m_h)

## ----iar-forecast-------------------------------------------------------------
m_fc <- forecast(m_kalman, tAhead = 10)
cat("Forecast (10 time units ahead):", round(m_fc@forecast, 4), "\n")

## ----iar-plot-forecast, fig.cap = "iAR series with one-step-ahead forecast (blue dot)."----
plot_forecast(m_fc,ylab="Values",xlab="Time",type="o",pch=20)

## ----iar-interp---------------------------------------------------------------
# Introduce a missing value at position 10
m_interp           <- m_kalman
m_interp@series[10] <- NA

m_interp <- interpolation(m_interp)
cat("Interpolated value at position 10:",
    round(m_interp@interpolated_values, 4), "\n")
cat("Original value at position 10    :",
    round(y_std[10], 4), "\n")

## ----ciar-sim-----------------------------------------------------------------
set.seed(2847)
times <- gentime(n = 100)@times

# phi = (0.9, 0): purely real CiAR — same as iAR but more general
m_ciar <- CiAR(times = times, coef = c(0.9, 0))
m_ciar <- sim(m_ciar)

# Standardise
y_c           <- m_ciar@series
y_c_std       <- y_c / sd(y_c)
m_ciar@series     <- y_c_std
m_ciar@series_esd <- rep(0, length(times))

## ----ciar-kalman--------------------------------------------------------------
m_ciar <- kalman(m_ciar)
cat("CiAR estimate: phiR =", round(m_ciar@coef[1], 4),
    " phiI =", round(m_ciar@coef[2], 4), "\n")
cat("Modulus |phi| =", round(sqrt(sum(m_ciar@coef^2)), 4), "\n")

## ----ciar-forecast------------------------------------------------------------
m_ciar <- forecast(m_ciar, tAhead = 10)
cat("CiAR forecast:", round(m_ciar@forecast, 4), "\n")

## ----biar-sim-----------------------------------------------------------------
set.seed(2847)
times <- gentime(n = 100)@times

# coef = c(phiR, phiI), rho = cross-series correlation
m_biar <- BiAR(times = times, coef = c(0.9, 0.3), rho = 0.9)
m_biar <- sim(m_biar)

head(m_biar@series)   # matrix: column 1 = series 1, column 2 = series 2

## ----biar-kalman--------------------------------------------------------------
n       <- length(times)
y_b     <- m_biar@series
y_b_std <- y_b / matrix(apply(y_b, 2, sd), nrow = n, ncol = 2, byrow = TRUE)

m_biar@series     <- y_b_std
m_biar@series_esd <- matrix(0, n, 2)

m_biar <- kalman(m_biar)
cat("BiAR estimate: phiR =", round(m_biar@coef[1], 4),
    " phiI =", round(m_biar@coef[2], 4), "\n")

## ----biar-forecast------------------------------------------------------------
m_biar <- forecast(m_biar, tAhead = 10)
cat("BiAR forecast (series 1):", round(m_biar@forecast[1], 4), "\n")
cat("BiAR forecast (series 2):", round(m_biar@forecast[2], 4), "\n")

## ----agn-load-----------------------------------------------------------------
data(agn)
head(agn)

## ----agn-plot, fig.cap = "AGN MCG-6-30-15 light curve. Gaps in the observation schedule produce the irregular spacing typical of ground-based astronomical surveys."----
plot(agn$t, agn$m, xlab = "Heliocentric JD - 2450000",
     ylab = "Flux [10^-15 erg/s/cm^2/A]",
     main = "AGN MCG-6-30-15 (K band)",type="o",pch=20)

## ----agn-fit------------------------------------------------------------------
# Standardise: centre and scale by SD (measurement errors available)
y_agn   <- agn$m
y_agn_c <- (y_agn - mean(y_agn)) / sd(y_agn)
esd_agn <- agn$merr / sd(y_agn)

m_agn <- iAR(
  family     = "norm",
  times      = agn$t,
  series     = y_agn_c,
  series_esd = esd_agn
)

m_agn <- kalman(m_agn)
cat("Estimated phi:", round(m_agn@coef, 4), "\n")
cat("Kalman log-lik:", round(m_agn@kalmanlik, 4), "\n")

## ----agn-fit-plot, fig.cap = "AGN light curve (standardised) with fitted iAR values overlaid."----
m_agn <- fit(m_agn)
plot_fit(m_agn, xlab = "Heliocentric JD - 2450000",
     ylab = "Flux [10^-15 erg/s/cm^2/A]",type="o",pch=20)

## ----agn-forecast-------------------------------------------------------------
m_agn <- forecast(m_agn, tAhead = 1)
cat("One-step-ahead forecast:", round(m_agn@forecast, 4), "\n")

## ----pair---------------------------------------------------------------------
data(cvnovag)
data(cvnovar)

# Pair the G-band and R-band CV Nova light curves
paired <- pairingits(data1 = cvnovag, data2 = cvnovar, tol = 0.01)

cat("Paired observations:", nrow(paired@paired), "\n")
head(paired@paired)

#' Maximum Likelihood Estimation of the iAR-T model
#'
#' Maximum Likelihood Estimation of the iAR-T model.
#'
#' @param series Array with the time series observations
#' @param times Array with the irregular observational times
#' @param df degrees of freedom
#' @param hessian logical. Indicates whether to compute the Hessian matrix.; if TRUE, the Hessian matrix will be calculated; if FALSE, the Hessian will not be computed.
#'
#' @return A list with the following components:
#' \itemize{
#' \item{coef}{ MLE of the phi parameter of the iAR-T model.}
#' \item{sigma}{ MLE of the sigma parameter of the iAR-T model.}
#' \item{ll}{ Value of the negative log likelihood evaluated in phi and sigma.}
#' \item{summary}{ If hessian=T, returns the standard error and p-value of the estimated parameter phi.}
#' }
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{iARtsample}}, \code{\link{iARphit}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' n=300
#' set.seed(6714)
#' st<-gentime(n)
#' y<-iARtsample(coef=0.9,times=st,sigma=1,df=3)
#' model<-iARt(series=y$series, times=st)
#' phi=model$coef
#' sigmaest=model$sigma
#' }
#' @noRd
iARt<-function (series, times,df=3, hessian = FALSE)
{
  aux<-1e10
  value<-1e10
  br<-0
  summary=list()
  for(i in 1:20)
  {
    phi=runif(1)
    sigma=var(series)*runif(1)
    if(hessian==F)
    {
      optim<-nlminb(start=c(phi,sigma),objective=iARphit,series=series,times=times,df=df,yest=0,lower=c(0,0.0001),upper=c(0.9999,2*var(series)))
      value<-optim$objective
      if(aux>value)
      {
        par<-optim$par
        aux<-value
        br<-br+1
      }
    }
    if(hessian==T)
    {
      out<-optim(par=c(phi,sigma),fn=iARphit,series=series,times=times,df=df,yest=0,method="L-BFGS-B",lower=c(0,0.0001),upper=c(0.9999,2*var(series)),hessian=hessian)
      value<-out$value
      if(aux>value)
      {
        par<-out$par
        conv <- out$convergence
        aux<-value
        hess=out$hessian
        stderr=as.numeric(sqrt(diag(solve(hess))))
        tvalue=par/stderr
        pvalue=2*pnorm(abs(tvalue),lower.tail=F)
        summary=list(hess=hess,stderr=stderr,tvalue=tvalue,pvalue=pvalue,convergence=conv)
        br<-br+1
      }
    }
    if(aux<=value & br>10 & i>15)
      break;
    }
    if(aux==1e10)
      par<-c(0,0)
    return(list(coef=par[1],sigma=par[2],ll=aux,summary=summary))
}

#' Forecast from iAR package model's
#'
#' Forecast with any of the models available in the iAR package
#'
#' @param coef Autocorrelation coefficient estimated by the method specified.
#' @param series Array with the time series observations.
#' @param times Array with the observational times.
#' @param tAhead The time ahead for which the forecast is required.
#' @param model model to be used for the forecast. The default is to use the iAR model. Other models available are "iAR-T", "iAR-Gamma", "CiAR" and "BiAR".
#' @param mu Level parameter of the iAR-Gamma process. A positive value.
#' @param phiI Imaginary parameter of CiAR model or Cross-correlation parameter of BiAR model.
#' @param df degrees of freedom parameter of iAR-T model.
#' @param level significance level for the confidence interval. The default value is 95.
#'
#' @return A dataframe with the following columns:
#' \itemize{
#' \item{tAhead}{ The time ahead used for the forecast.}
#' \item{forecast}{ Point forecast in the time ahead required.}
#' \item{stderror}{ Standard error of the forecast.}
#' \item{lowerCI}{ Lower limit of the confidence interval.}
#' \item{upperCI}{ Upper limit of the confidence interval.}
#' }
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{iARforecast}}, \code{\link{iARgforecast}}, \code{\link{iARforecast}}, \code{\link{BiARforecast}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' st <- gentime(n=200,lambda1=15,lambda2=2)
#' y  <- iARsample(coef=0.9,times=st)
#' model<-iARloglik(series=y$series,times=st)
#' phi=model$coef
#' foriAR<-iARforecast(coef=phi,series=y$series,tAhead=c(1.3),standardized=FALSE,zero_mean=FALSE)
#' foriAR
#' foriAR<-Forecast_iARModels(coef=phi,series=y$series,times=st,tAhead=c(1.3,2.6))
#' foriAR
#' }
#' @noRd
Forecast_iARModels<-function(coef,series,times,tAhead,model="iAR",mu=NULL,phiI=NULL,df=NULL,level=95)
{
  if(model == "iAR")
  {
    y=as.numeric(series)
    sigma = sqrt(var(series))
    mu = mean(series)
    y1=(series-mu)/sigma
    fore=iARforecast(coef=coef,series=y1,tAhead=tAhead)
    fore=fore*sigma+mu
    error=sigma*(1-coef**(2*tAhead))
  }
  if(model == "iAR-T")
  {
    fore=iARforecast(coef=coef,series=series,tAhead=tAhead,standardized=FALSE,zero_mean=FALSE)
    error=((df-2)/df)*sigma*(1-coef**(2*tAhead))
  }
  if(model == "iAR-Gamma")
  {
    y1=series
    if(min(series)<0)
      y1=series-min(series)
    fore=iARgforecast(coef=coef,mean=mu,series=series,tAhead=tAhead)
    if(min(series)<0)
      fore=fore+min(series)
    error=sigma*(1-coef**(2*tAhead))
  }
  if(model == "CiAR")
  {
    if(is.null(phiI))
      phiI=0
    sigma = sqrt(var(as.numeric(series)))
    mu = mean(as.numeric(series))
    y1=series
    fore=CiARforecast(coef=c(coef,phiI),series=y1,times=times,tAhead=tAhead)$forecast
    Mod=sqrt(coef^2+phiI^2)
    error=sigma*(1-Mod**(2*tAhead))
  }
  if(model == "BiAR")
  {
    mu = apply(y,1,mean)
    y1=series
    y2=y1[2,]
    y1=y1[1,]
    fore=BiARforecast(coef=c(coef,phiI),series1=y1,series2=y2,times=times,tAhead=tAhead)$forecast
    sigma=matrix(apply(series,1,sd),nrow=2)
    Mod=sqrt(coef^2+phiI^2)
    error=sigma*(1-Mod**(2*tAhead))
  }
  upper=fore+qnorm(level/100)*error
  lower=fore-qnorm(level/100)*error
  fin=data.frame(tAhead=tAhead,forecast=fore,stderror=error,lowerCI=lower,upperCI=upper)
  return(fin)
}

#' Simulate from an iAR-T Model
#'
#' Simulates an iAR-T Time Series Model.
#'
#' @param coef A coefficient of iAR-T model. A value between 0 and 1.
#' @param times Array with observational times.
#' @param sigma Scale parameter of the iAR-T process. A positive value.
#' @param df degrees of freedom.
#'
#' @return A list with the following components:
#' \itemize{
#' \item{series}{ Array with simulated iAR-t process.}
#' }
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' n=300
#' set.seed(6714)
#' st<-gentime(n)
#' y<-iARtsample(coef=0.9,times=st,sigma=1,df=3)
#' plot(st,y$series,type='l')
#' hist(y$series,breaks=20)
#' }
#' @noRd
iARtsample<-function(coef,times,sigma=1,df=3)
{
  n<-length(times)
  delta<-diff(times)
  y <- vector("numeric", n)
  y[1] <- rnorm(1)
  for (i in 2:n)
  {
    phid=coef**(delta[i-1]) #Expected Value Conditional Distribution
    yhat = phid * y[i-1]  #Mean of conditional distribution
    gL=sigma*(1-phid**(2)) #Variance Value Conditional Distribution
    y[i] <- rt(1, df=df)*sqrt(gL * (df-2)/df) + yhat #Conditional-t IAR
  }
  return(list(series=y))
}

#' Plot Forecast Method for unidata Objects
#'
#' This method visualizes the `unidata` and `multidata` objects by plotting both the 
#' original time series and the forecasted values. The `unidata` (`multidata`) object 
#' is converted into a `zoo` object for plotting, and the forecasted 
#' values are added as points with a different color.
#'
#' @usage
#' plot_forecast(x,...)
#' 
#' @param x An object of class `unidata` or `multidata`. This object must contain the 
#'   time series in the `series` slot, the associated time points in the 
#'   `times` slot, the forecasted values in the `forecast` slot, and the 
#'   forecast horizon in the `tAhead` slot.
#' @param ... Additional arguments passed to the `plot.zoo` function for 
#'   customizing the plot (e.g., titles, colors, etc.).
#'
#' @importFrom zoo plot.zoo zoo
#' @export
plot_forecast <- S7::new_generic("plot_forecast", "x")
S7::method(plot_forecast, unidata) <- function(x, ...) {
  zoo::plot.zoo(zoo::zoo(x@series,x@times), xlim=c(min(x@times),max(x@times)+x@tAhead), ...)
  points(zoo::zoo(x@forecast,max(x@times)+x@tAhead), col = 2)
}
S7::method(plot_forecast, multidata) <- function(x, ...) {
  zoo::plot.zoo(zoo::zoo(x@series[,1],x@times), xlim=c(min(x@times),max(x@times)+x@tAhead), ...)
  points(zoo::zoo(x@forecast[,1],max(x@times)+x@tAhead), col = 2)
  readline("Press [Enter] to show the next plot...")
  zoo::plot.zoo(zoo::zoo(x@series[,1],x@times), xlim=c(min(x@times),max(x@times)+x@tAhead), ...)
  points(zoo::zoo(x@forecast[,1],max(x@times)+x@tAhead), col = 2)
}

#' Test for the significance of the autocorrelation estimated by the iAR package models in periodic irregularly observed time series
#'
#' This function perform a test for the significance of the autocorrelation estimated by the iAR package models. This test is based on the residuals of the periodical time series fitted with an harmonic model using an incorrect period.
#'
#' @param series Array with the time series observations.
#' @param times Array with the irregular observational times.
#' @param series_esd Array with the variance of the measurement errors.
#' @param f Frequency (1/Period) of the raw time series.
#' @param coef autocorrelation estimated by one of the iAR package models.
#' @param model model used to estimate the autocorrelation parameter ("iAR", "iAR-Gamma", "iAR-T", "CiAR" or "BiAR").
#' @param plot logical; if true, the function return a density plot of the distribution of the bad fitted examples; if false, this function does not return a plot.
#' @param xlim The x-axis limits (x1, x2) of the plot. Only works if plot='TRUE'. See \code{\link{plot.default}} for more details.
#' @param df degrees of freedom parameter of the iAR-T model.
#'
#' @details The null hypothesis of the test is: The autocorrelation estimated in the time series belongs to the distribution of the coefficients estimated for the residuals of the data fitted using wrong periods. Therefore, if the hypothesis is rejected, it can be concluded that the residuals of the harmonic model do not remain a time dependency structure.The statistic of the test is log(phi) which was contrasted with a normal distribution with parameters corresponding to the log of the mean and the variance of the phi computed for the residuals of the bad fitted light curves.
#' @return A list with the following components:
#' \item{phi}{ MLE of the autocorrelation parameter of the iAR/CiAR model.}
#' \item{bad}{ A matrix with two columns. The first column contains the incorrect frequencies used to fit each harmonic model. The second column has the MLEs of the autocorrelation parameters of the iAR/CiAR model that has been fitted to the residuals of the harmonic model fitted using the frequencies of the first column.}
#' \item{norm}{ Mean and variance of the normal distribution of the bad fitted examples.}
#' \item{z0}{ Statistic of the test (log(abs(phi))).}
#' \item{pvalue}{ P-value computed for the test.}
#'
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{clcep}}, \code{\link{harmonicfit}},\code{\link{iARPermutation}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' data(clcep)
#' f1=0.060033386
#' #results=harmonicfit(file=clcep,f1=f1)
#' #y=results$res/sqrt(var(results$res))
#' #st=results$t
#' #res3=iARloglik(y,st,standardized=TRUE)[1]
#' #res3$coef
#' #require(ggplot2)
#' #test<-iARTest(series=clcep[,2],times=clcep[,1],f=f1,coef=res3$coef,
#' #model="iAR",plot=TRUE,xlim=c(-10,0.5))
#' #test
#' }
#' @noRd
iARTest<-function (series, times, series_esd=0,f, coef,model="iAR",plot = TRUE, xlim = c(-1, 0),df=3)
{
  aux = seq(2.5, 47.5, by = 2.5)
  aux = c(-aux, aux)
  aux = sort(aux)
  f0 = f * (1 + aux/100)
  f0 <- sort(f0)
  l1 <- length(f0)
  bad <- rep(0, l1)
  data <- cbind(times, series)
  if(sum(series_esd)==0)
    series_esd=rep(0,length(series))
  for (j in 1:l1) {
    results = harmonicfit(file = data, f1 = f0[j])
    y = results$res/sqrt(var(results$res))
    st = results$t
    if(model=="iAR")
    {
      res3 = iARloglik(y, st,series_esd)[1]
      bad[j] = res3$coef
    }
    if(model=="iAR-T")
    {
      bad[j] = iARt(y, times=st,df=df)$coef
    }
    if(model=="iAR-Gamma")
    {
      y1=y
      if(min(y1)<0)
        y1=y1-min(y1)
      bad[j] = iARgamma(y1, times=st)$coef
    }
    if(model=="CiAR")
    {
      res3 = CiARkalman(series=y, times=st,series_esd=series_esd)[1]
      bad[j] = res3$phiR
    }
  }
  mubf <- mean(log(abs(bad)))
  sdbf <- sd(log(abs(bad)))
  z0 <- log(abs(coef))
  pvalue <- pnorm(z0, mubf, sdbf)
  out <- NULL
  BAD <-data.frame(f0,bad)
  if (plot == TRUE) {
    phi2 = bad
    phi2 <- as.data.frame(phi2)
    coef <- as.data.frame(coef)
    out <- ggplot(phi2, aes(log(abs(phi2)))) + geom_density(adjust = 2) +
      xlab("") + ylab("") + theme_bw() + ggtitle("") +
      geom_point(data = coef, aes(log(abs(coef))), y = 0, size = 4,
                 col = "red", shape = 17) + xlim(xlim[1], xlim[2]) +
      theme(plot.title = element_text(face = "bold", size = 20),
            panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
            panel.background = element_blank())
    out
  }
  return(list(phi = coef, bad=BAD, norm = c(mubf, sdbf), z0 = z0, pvalue = pvalue, out = out))
}

#' Utilities Class
#'
#' The `utilities` class is an S7 class designed to group utility functions 
#' that are not directly tied to other specific objects in the package. 
#' These include the functions `gentime`, `paringits`, `harmonicfit`, and `foldlc`.
#'
#' @param times A numeric vector storing the time points.
#' @param series A numeric vector representing the values of the time series.
#' @param series_esd A numeric vector representing the error standard deviations of the time series.
#' @param times_phased A numeric vector of phased times.
#' @param series_phased A numeric vector containing the time series values ordered by phase.
#' @param series_esd_phased A numeric vector containing the error standard deviations of the time series ordered by phase. 
#' @param fitted_values A numeric vector containing the fitted values from the harmonic model.
#' @param residuals A numeric vector containing the residuals from the harmonic model.
#' @param coef A numeric vector containing the estimated coefficients of the harmonic model.
#' @param summary A summary object containing the harmonic model information.
#' @param paired Data Frame with the paired datasets.
#'
#' @section Description:
#' This class acts as a container for standalone methods that perform independent 
#' operations within the package. By grouping them under a single class, the package 
#' achieves better modularity and organization, facilitating maintenance and extensibility.
#'
#' @section Available Methods:
#' - `gentime`: Generates time points based on a specified statistical distribution.
#' - `paringits`: A method for pairing irregular time series.
#' - `harmonicfit`: A method for fitting harmonic models to data.
#' - `foldlc`: A method for folding light curves in time series analysis.
#'
#' @examples
#' # Create a utilities object
#'
#' @export
utilities <- S7::new_class(
  "utilities",
  package = "iAR",
  properties = list(
    times = S7::class_numeric,
    series = S7::class_numeric,
    series_esd = S7::class_numeric,
    times_phased = S7::class_numeric,
    series_phased = S7::class_numeric,
    series_esd_phased = S7::class_numeric,
    fitted_values = S7::class_numeric,
    residuals = S7::class_numeric,
    coef = S7::class_numeric,
    summary = S7::class_list,
    paired = S7::class_numeric
  )
)

#' Interpolation from iAR-T model
#'
#' Interpolation of missing values from models fitted by \code{\link{iARt}}
#'
#' @param coef A given array with the parameters of the iAR-T model. The first element of the array corresponding to the phi parameter and the second element to the scale parameter sigma
#' @param series Array with the time series observations.
#' @param times Array with the irregular observational times.
#' @param df degrees of freedom
#' @param yini a single value, initial value for the estimation of the missing value of the time series.
#'
#'
#' @return A list with the following components:
#' \itemize{
#' \item{fitted}{ Estimation of a missing value of the iAR-T process.}
#' \item{ll}{ Value of the negative log likelihood evaluated in the fitted missing values.}
#' }
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{iARtsample}}, \code{\link{iARt}}
#'
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' set.seed(6714)
#' n<-100
#' st<-gentime(n)
#' y<-iARtsample(coef=0.9,times=st,sigma=1,df=3)
#' model<-iARt(series=y$series, times=st)
#' napos=10
#' y0=y$series
#' y=y$series
#' y[napos]=NA
#' xest=c(model$coef,model$sigma)
#' yest=iARtinterpolation(coef=xest,series=y,times=st)
#' yest$fitted
#' mse=(y0[napos]-yest$fitted)^2
#' print(mse)
#' plot(st,y,type='l',xlim=c(st[napos-5],st[napos+5]))
#' points(st,y,pch=20)
#' points(st[napos],yest$fitted,col="red",pch=20)
#' }
#' @noRd
iARtinterpolation<-function(coef,series,times,df=3,yini=0)
{
  aux<-1e10
  value<-1e10
  br<-0
  countNA = length(which(is.na(series)))
  if(countNA==0){
    stop("This function requires a NA value at the point to be interpolated.")}
  if(countNA > 1){
    stop("This function requires only one NA value.")}
  if (yini==0)
    yini = rnorm(1)
  out = optim(yini, iARphit, lower = -Inf, upper = Inf,
              coef=coef,series=series,times=times,df=df, method="L-BFGS-B")
  par = out$par
  aux = out$value
  return(list(fitted = par, ll = aux))
}

#' Maximum Likelihood Estimation of the iAR Model
#'
#' Maximum Likelihood Estimation of the iAR Model.
#'
#' @param series Array with the time series observations.
#' @param times Array with the irregular observational times.
#' @param series_esd Array with the measurements error standard deviations.
#' @param zero_mean logical; if TRUE, the array series has zero mean; if FALSE, series has a mean different from zero.
#' @param standardized logical; if TRUE, the array series is standardized; if FALSE, series contains the raw time series.
#' @param hessian logical. Indicates whether to compute the Hessian matrix.; if TRUE, the Hessian matrix will be calculated; if FALSE, the Hessian will not be computed.
#'
#' @return A list with the following components:
#' \itemize{
#' \item{coef}{ MLE of the phi parameter of the iAR model.}
#' \item{ll}{ Value of the negative log likelihood evaluated in phi.}
#' \item{summary}{ If hessian=T, returns the standard error and p-value of the estimated parameter phi.}
#' }
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{iARsample}}, \code{\link{arima}}, \code{\link{iARphiloglik}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' #Generating iAR sample
#' set.seed(6714)
#' st<-gentime(n=100)
#' y<-iARsample(coef=0.99,times=st)
#' y<-y$series
#' #Compute Phi
#' phi=iARloglik(series=y,times=st)$coef
#' print(phi)
#' #Compute the standard deviation of innovations
#' n=length(y)
#' d=c(0,diff(st))
#' phi1=phi**d
#' yhat=phi1*as.vector(c(0,y[1:(n-1)]))
#' plot(st,y,type='l')
#' lines(st,yhat,col='red')
#' sigma=var(y)
#' nu=c(sigma,sigma*(1-phi1**(2))[-1])
#' tau<-nu/sigma
#' sigmahat<-mean(c((y-yhat)**2/tau))
#' nuhat<-sigmahat*(1-phi1**(2))
#' nuhat2<-sqrt(nuhat)
#' #Equally spaced models
#' require(arfima)
#' fit2<-arfima(y,order=c(1,0,0))
#' fit<-arima(y,order=c(1,0,0),include.mean=FALSE)
#' syarf<-tacvfARFIMA(phi=fit2$modes[[1]]$phi,dfrac=fit2$modes[[1]]$dfrac,
#' sigma2=fit2$modes[[1]]$sigma,maxlag=20)[1]
#' syar<-fit$sigma/(1-fit$coef[1]**2)
#' print(sigmahat)
#' print(syar)
#' print(syarf)
#' carf<-fit2$modes[[1]]$sigma/syarf
#' car<-(1-fit$coef[1]**2)
#' ciar<-(1-phi1**(2))
#' #Compute the standard deviation of innovations (regular case)
#' sigma=var(y)
#' nuhat3=sqrt(sigma*ciar)
#' searf<-sqrt(sigma*carf)
#' sear<-sqrt(sigma*car)
#' #Plot the standard deviation of innovations
#' plot(st[-1], nuhat3[-1], t="n", axes=FALSE,xlab='Time',ylab='Standard Deviation of Innovations')
#' axis(1)
#' axis(2)
#' segments(x0=st[-1], y0=nuhat3[-1], y1=0, col=8)
#' points(st, nuhat3, pch=20, col=1, bg=1)
#' abline(h=sd(y),col='red',lwd=2)
#' abline(h=sear,col='blue',lwd=2)
#' abline(h=searf,col='green',lwd=2)
#' abline(h=mean(nuhat3[-1]),col='black',lwd=2)
#' }
#' @noRd
iARloglik=function(series,times,series_esd=0,zero_mean=TRUE,standardized=TRUE,hessian=FALSE){
  if(sum(series_esd)==0){
    series_esd=rep(0,length(series))}
  summary=list()
  if(hessian==F)
  {
    out=optimize(iARphiloglik, interval=c(0,1), series=series, times=times, series_esd=series_esd, zero_mean = zero_mean, standardized = standardized)
    phi=out$minimum
    ll=out$objective
    summary=list()
  }
  if(hessian==T)
  {
    out=optim(par=c(x=0),fn=iARphiloglik, series=series, times=times, series_esd=series_esd, zero_mean = zero_mean, standardized = standardized,method="L-BFGS-B",lower=0,upper=0.9999999,hessian=hessian)
    phi=as.numeric(out$par)
    conv <- out$convergence
    ll=out$value
    hess=as.numeric(out$hessian)
    stderr=sqrt(diag(solve(hess)))
    tvalue=phi/stderr
    pvalue=2*pnorm(abs(tvalue),lower.tail=F)
    summary=list(hess=hess,stderr=stderr,tvalue=tvalue,pvalue=pvalue,convergence=conv)
  }
  return(list(coef=phi,loglik=ll,summary=summary))
}

.onLoad <- function(...) {
  S7::methods_register()
}

# Format one coefficient table value; returns "  NA" when SE is unavailable.
.fmt_se <- function(x) if (is.null(x) || !is.finite(x)) "  NA" else format(round(x, 2), nsmall = 2)

# Clamp an iAR estimated coefficient to (0, 1) and issue warnings if it
# hits the boundary or is suspiciously close to it.
.clamp_iar_coef <- function(phi, eps = 1e-4, near_boundary = 0.995) {
  if (!is.finite(phi) || phi <= 0 || phi >= 1) {
    phi_clamped <- max(eps, min(1 - eps, phi))
    warning(paste0(
      "Estimated coefficient (", round(phi, 6), ") is at or beyond the boundary of (0, 1). ",
      "It has been clamped to ", phi_clamped, ". ",
      "Results may be unreliable. If negative autocorrelation is expected, consider using CiAR."
    ), call. = FALSE)
    return(phi_clamped)
  }
  if (phi > near_boundary) {
    warning(paste0(
      "Estimated coefficient (", round(phi, 4), ") is near the upper boundary of (0, 1). ",
      "The model may be near non-stationary; results should be interpreted with caution."
    ), call. = FALSE)
  } else if (phi < (1 - near_boundary)) {
    warning(paste0(
      "Estimated coefficient (", round(phi, 4), ") is near zero. ",
      "This may indicate weak autocorrelation or convergence issues."
    ), call. = FALSE)
  }
  phi
}

#' Maximum Likelihood Estimation of the BiAR Model via Kalman Recursions
#'
#' Maximum Likelihood Estimation of the BiAR model parameters phiR and phiI. The estimation procedure uses the Kalman Filter to find the maximum of the likelihood.
#'
#'
#' @param series1 Array with the observations of the first time series of the BiAR process.
#' @param series2 Array with the observations of the second time series of the BiAR process.
#' @param times Array with the irregular observational times.
#' @param series_esd1 Array with the measurements error standard deviations of the first time series of the BiAR process.
#' @param series_esd2 Array with the measurements error standard deviations of the second time series of the BiAR process.
#' @param zero_mean logical; if true, the array y has zero mean; if false, y has a mean different from zero.
#' @param hessian logical. Indicates whether to compute the Hessian matrix.; if TRUE, the Hessian matrix will be calculated; if FALSE, the Hessian will not be computed.
#' @param niter Number of iterations in which the function nlminb will be repeated.
#' @param seed a single value, interpreted as the seed of the random process.
#'
#' @return A list with the following components:
#' \itemize{
#' \item{phiR}{ MLE of the autocorrelation coefficient of BiAR model (phiR).}
#' \item{phiI}{ MLE of the cross-correlation coefficient of the BiAR model (phiI).}
#' \item{ll}{ Value of the negative log likelihood evaluated in phiR and phiI.}
#' \item{summary}{ If hessian=T, returns the standard error and p-value of the estimated parameter phi.}
#' }
#' @references
#' \insertRef{Elorrieta_2021}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{BiARsample}}, \code{\link{BiARphikalman}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' n=80
#' set.seed(6714)
#' st<-gentime(n)
#' x=BiARsample(phiR=0.9,phiI=0,times=st,rho=0)
#' y=x$series
#' y1=y/apply(y,1,sd)
#' biar=BiARkalman(series1=y1[1,],series2=y1[2,],times=st,series_esd1 = rep(0,length(y[1,])),
#' series_esd2=rep(0,length(y[1,])))
#' biar
#' }
#' @noRd
BiARkalman<-function (series1, series2, times, series_esd1 = 0, series_esd2 = 0, zero_mean = TRUE, hessian=FALSE,niter = 10, seed = 1234)
{
  set.seed(seed)
  aux <- 1e+10
  value <- 1e+10
  br <- 0
  if (sum(series_esd1) == 0) {
    series_esd1 = rep(0, length(series1))
  }
  if (sum(series_esd2) == 0) {
    series_esd2 = rep(0, length(series2))
  }
  summary=list()
  for (i in 1:niter) {
    phiR = 2 * runif(1) - 1
    phiI = 2 * runif(1) - 1
    if (Mod(complex(1, real = phiR, imaginary = phiI)) < 1) {
      if(hessian==F)
      {
        optim <- nlminb(start = c(phiR, phiI), objective = BiARphikalman,
                      series1 = series1,series2 = series2, times = times, series_esd1 = series_esd1, series_esd2=series_esd2,zero_mean = zero_mean, yest=c(0,0),lower	= c(-1, -1), upper = c(1, 1))
        value <- optim$objective
        par0<-optim$par
      }
      if(hessian==T)
      {
        out<-optim(par=c(phiR,phiI),fn=BiARphikalman,series1 = series1,series2 = series2, times = times, series_esd1 = series_esd1, series_esd2=series_esd2,zero_mean = zero_mean, yest=c(0,0),method="L-BFGS-B",lower=c(-1,-1),upper=c(1,1),hessian=hessian)
        value<-out$value
        par0<-out$par
      }
    }
    if (aux > value) {
      par <- par0
      aux <- value
      if(hessian==T)
      {
        conv <- out$convergence
        hess=out$hessian
        stderr=as.numeric(sqrt(diag(solve(hess))))
        tvalue=par/stderr
        pvalue=2*pnorm(abs(tvalue),lower.tail=F)
        summary=list(hess=hess,stderr=stderr,tvalue=tvalue,pvalue=pvalue,convergence=conv)
      }
      br <- br + 1
    }
    if (aux <= value & br > 1 & i > trunc(niter/2))
      break
  }
  if (aux == 1e+10)
    par <- c(0, 0)
  return(list(phiR = par[1], phiI = par[2], ll = aux,summary=summary))
}

#' Plot Fit Method for unidata Objects
#'
#' This method visualizes the `unidata` and `multidata` objects by plotting both the 
#' original time series and the fitted values. The `unidata` (`multidata`) object is 
#' converted into a `zoo` object for plotting, and the fitted values are 
#' added as a secondary line with a different style and color.
#'
#' @usage
#' plot_fit(x,...)
#' 
#' @param x An object of class `unidata` or `multidata`. This object must contain the 
#'   time series in the `series` slot, the associated time points in the 
#'   `times` slot, and the fitted values in the `fitted_values` slot.
#' @param ... Additional arguments passed to the `plot.zoo` function for 
#'   customizing the plot (e.g., titles, colors, etc.).
#'
#' @importFrom zoo plot.zoo zoo
#' @export
plot_fit<- S7::new_generic("plot_fit", "x")
S7::method(plot_fit, unidata) <- function(x, ...) {
  zoo::plot.zoo(zoo::zoo(x@series, x@times), ...)
  lines(zoo::zoo(x@fitted_values, x@times), col = 2, lty = 2)
}
S7::method(plot_fit, multidata) <- function(x, ...) {
  zoo::plot.zoo(zoo::zoo(x@series[,1], x@times), ...)
  lines(zoo::zoo(x@fitted_values[,1], x@times), col = 2, lty = 2)
  readline("Press [Enter] to show the next plot...")
  zoo::plot.zoo(zoo::zoo(x@series[,2], x@times), ...)
  lines(zoo::zoo(x@fitted_values[,2], x@times), col = 2, lty = 2)
}

#' @name agn
#' @title Active Galactic Nuclei
#' @description Time series of the AGN MCG-6-30-15 measured in the K-band between 2006 August and 2011 July with the ANDICAM camera mounted on the 1.3 m telescope at Cerro Tololo Inter-American Observatory (CTIO)
#' @docType data
#' @usage agn
#' @format A data frame with 237 observations on the following 3 variables:
#' \describe{
#'   \item{t}{heliocentric Julian Day - 2450000}
#'   \item{m}{Flux $(10^(-15) ergs/s/cm^2 /A)$}
#'   \item{merr}{measurement error standard deviations.}
#' }
#' @references
#' \insertRef{Lira_2015}{iAR}
#' @examples
#' data(agn)
#' plot(agn$t,agn$m,type="l",ylab="",xlab="")
"agn"

#' Interpolation from iAR model
#'
#' Interpolation of missing values from models fitted by \code{\link{iARkalman}}
#'
#' @param coef A given phi coefficient of the iAR model.
#' @param series Array with the time series observations.
#' @param times Array with the irregular observational times.
#' @param series_esd Array with the measurements error standard deviations.
#' @param yini a single value, initial value for the estimation of the missing value of the time series.
#' @param zero_mean logical; if TRUE, the array y has zero mean; if FALSE, y has a mean different from zero.
#' @param standardized logical; if TRUE, the array y is standardized; if FALSE, y contains the raw time series.
#'
#'
#' @return A list with the following components:
#' \itemize{
#' \item{fitted}{ Estimation of a missing value of the iAR process.}
#' \item{ll}{ Value of the negative log likelihood evaluated in the fitted missing values.}
#' }
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{iARsample}}, \code{\link{iARkalman}}
#'
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' set.seed(6714)
#' st<-gentime(n=100)
#' y<-iARsample(coef=0.99,times=st)
#' y<-y$series
#' phi=iARkalman(series=y,times=st)$coef
#' print(phi)
#' napos=10
#' y0=y
#' y[napos]=NA
#' xest=phi
#' yest=iARinterpolation(coef=xest,series=y,times=st)
#' yest$fitted
#' mse=(y0[napos]-yest$fitted)^2
#' print(mse)
#' plot(st,y,type='l',xlim=c(st[napos-5],st[napos+5]))
#' points(st,y,pch=20)
#' points(st[napos],yest$fitted,col="red",pch=20)
#' }
#' @noRd
iARinterpolation<-function(coef,series,times,series_esd=0,yini=0,zero_mean=TRUE,standardized=TRUE)
{
  aux<-1e10
  value<-1e10
  br<-0
  countNA = length(which(is.na(series)))
  if(countNA==0){
    stop("This function requires a NA value at the point to be interpolated.")}
  if(countNA > 1){
    stop("This function requires only one NA value.")}
  if(sum(series_esd)==0){
    series_esd=rep(0,length(series))}
  if (yini==0)
    yini = rnorm(1)
  out = optim(c(yini), iARphikalman, lower = -Inf, upper = Inf,
              coef=coef,series = series, times = times, series_esd=series_esd,zero_mean = zero_mean,standardized=standardized, method="BFGS")
  par = out$par
  aux = out$value
  return(list(fitted = par, ll = aux))
}

#' Fitted Values of iAR-Gamma model
#'
#' Fit an iAR-Gamma model to an irregularly observed time series.
#'
#' @param coef Estimated phi parameter by the iAR-Gamma model.
#' @param mean Estimated mean parameter by the iAR-Gamma model.
#' @param series Array with the time series observations.
#' @param times Array with the irregular observational times.
#'
#' @return Fitted values of the iAR-Gamma model
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#' \code{\link{gentime}}, \code{\link{iARgsample}}, \code{\link{iARgamma}}
#' @keywords internal
#' @examples
#' \dontshow{
#' n=300
#' set.seed(6714)
#' st<-gentime(n)
#' y<-iARgsample(coef=0.9,times=st,sigma=1,mean=1)
#' model<-iARgamma(series=y$series, times=st)
#' phi=model$coef
#' muest=model$mean
#' sigmaest=model$sigma
#' fit=iARgfit(coef=phi,mean=muest,series=y$series,times=st)
#' }
#' @noRd
iARgfit<-function(coef,mean,series,times)
{
  delta=diff(times)
  y1=series[-c(length(series))]
  xd=coef**delta
  yhat = mean + xd * y1
  fit=c(mean,yhat)
  return(fitted=fit)
}

#' Computing phased time series
#'
#' This function computes a time series folded on its period.
#'
#' @name phase
#' 
#' @usage phase(x, ...)
#'  
#' @param x An object of class `utilities`, or `NULL` (the default). When `x` is
#'   `NULL` or omitted, the `utilities` object is created internally, so
#'   `phase(data = clcep, f1 = f1)` is the recommended usage.
#' @param ... Additional arguments for pairing time series:
#'   \describe{
#'     \item{data}{A data frame with three columns corresponding to the time, values, and standard errors of the irregularly observed time series.}
#'     \item{f1}{frequency (1 / period) of the time series.}
#'     \item{twop}{logical; if TRUE, the phased series will be duplicated over two cycles (0–2).}
#'    }
#' @return An object of class `utilities` with the slots:
#' \item{series_phased}{A numeric vector containing the time series values ordered by phase.}
#' \item{series_esd_phased}{A numeric vector containing the error standard deviations of the time series ordered by phase.}
#' \item{times_phased}{A numeric vector of phased times (values between 0 and 1, or 0 and 2 if 'two.cycles = TRUE').}
#' 
#' @details 
#' The phase \eqn{\phi} of an observation is computed as
#' \deqn{\phi = \frac{t - t_0}{p} - \mathrm{E}(t),}
#' where \eqn{t_0} is the reference time (by default the first observation),
#' \eqn{p = 1/f_1} is the period, and \eqn{\mathrm{E}(t)} is the integer part of
#' \eqn{(t - t_0)/p}.
#' 
#' @export
#' @references
#' \insertRef{Elorrieta_2021}{iAR}
#'
#' @examples
#' data(clcep)
#' f1=0.060033386
#' o1=phase(data=clcep,f1=f1,twop=TRUE)
#' plot(o1@times_phased,o1@series_phased,pch=20)
phase <- S7::new_generic("phase", "x", function(x = NULL, ...) S7::S7_dispatch())
S7::method(phase, NULL) <- function(x, ...) {
  phase(utilities(), ...)
}

S7::method(phase, utilities) <- function(x, data, f1, twop=TRUE)
{
  mycurve = data
  t = mycurve[, 1]
  m = mycurve[, 2]
  merr=rep(0,length(t))
  if(dim(data)[2]>2)
    merr = mycurve[, 3]
  P <- 1/f1
  fold <- (t - t[1])%%(P)/P
  if(twop==TRUE)
  {
    fold <- c(fold, fold + 1)
    m <- rep(m, 2)
    merr <- rep(merr, 2)
  }
  dat1 <- cbind(fold, m, merr)
  dat1 <- as.data.frame(dat1)
  dat1<-dat1[order(dat1[,1],decreasing=TRUE),]
  x@series_phased <- dat1[,2]
  x@series_esd_phased <- dat1[,3]
  x@times_phased <- dat1[,1]
  return(x)
}

#' Plot Method for unidata Objects
#'
#' This method allows visualizing `unidata` and `multidata` objects using the `plot.zoo` 
#' function from the `zoo` package. It converts the `unidata` (`multidata`) object into 
#' a `zoo` object and then applies the plotting function to the time series 
#' contained in the object.
#' @name plot
#' @usage plot(x, y, ...)
#' 
#' @param x An object of class `unidata` or `multidata`. This object must contain the time 
#'   series in the `series` slot and the associated time points in the 
#'   `times` slot.
#' @param y Ignored. Present for compatibility with the base \code{plot} generic.
#' @param ... Additional arguments passed to the `plot.zoo` function for 
#'   customizing the plot (e.g., titles, colors, etc.).
#'
#' @importFrom zoo plot.zoo zoo
#' @export
S7::method(plot, unidata) <- function(x, y = NULL, ...) {
  zoo::plot.zoo(zoo::zoo(x@series, x@times), ...)
}
S7::method(plot, multidata) <- function(x, y = NULL, ...) {
  zoo::plot.zoo(zoo::zoo(x@series[,1], x@times), ...)
  readline("Press [Enter] to show the next plot...")
  zoo::plot.zoo(zoo::zoo(x@series[,2], x@times), ...)
}
S7::method(plot, utilities) <- function(x, y = NULL, ...) {
  zoo::plot.zoo(zoo::zoo(x@series_phased, x@times_phased), ...)
}

#' Maximum Likelihood Estimation for iAR Models
#'
#' Maximum Likelihood Estimation for irregular autoregressive (iAR) models, supporting different distribution families:
#' normal (`iAR`), t (`iAR-T`), and gamma (`iAR-Gamma`).
#'
#' @name loglik
#'
#' @param x An object of class \code{iAR}, containing the model specification, parameters, and the time series to be evaluated:
#'   \itemize{
#'     \item \code{series}: The observed time series.
#'     \item \code{times}: A numeric vector specifying the time points of the series.
#'     \item \code{series_esd}: (Optional) A standardized version of the series.
#'     \item \code{zero_mean}: Logical. Indicates whether the series should be mean-centered.
#'     \item \code{standardized}: Logical. Indicates whether the series is standardized.
#'     \item \code{hessian}: Logical. If \code{TRUE}, the function computes the Hessian matrix for parameter estimation.
#'     \item \code{family}: The distribution family of the iAR model (one of "norm", "t", or "gamma").
#'     \item \code{df}: Degrees of freedom for the t-distribution (only for \code{family = "t"}).
#'     \item \code{sigma}: The scale parameter for the t-distribution (only for \code{family = "t"}).
#'     \item \code{mean}: The mean parameter for the gamma distribution (only for \code{family = "gamma"}).
#'     \item \code{variance}: The variance parameter for the gamma distribution (only for \code{family = "gamma"}).
#'   }
#'
#' @return An updated \code{iAR} object with the following additional attributes:
#'   \itemize{
#'     \item \code{coef}: Estimated model coefficients.
#'     \item \code{loglik}: Log-likelihood value of the model.
#'     \item \code{summary}: A summary table containing parameter estimates, standard errors, and p-values.
#'     \item \code{sigma}: For t and gamma families, the estimated scale parameter.
#'     \item \code{mean}: For the gamma family, the estimated mean parameter.
#'     \item \code{variance}: For the gamma family, the estimated variance parameter.
#'   }
#'
#' @param ... Additional arguments (unused).
#'
#' @details
#' This method estimates the parameters of an iAR model using the Maximum Likelihood Estimation (MLE) approach.
#' Depending on the chosen distribution family, the corresponding likelihood function is maximized:
#' \itemize{
#'   \item "norm" maximizes the likelihood for a normally-distributed series.
#'   \item "t" maximizes the likelihood for a t-distributed series.
#'   \item "gamma" maximizes the likelihood for a gamma-distributed series.
#' }
#' The function updates the \code{iAR} object with the estimated parameters, the log-likelihood value, and a summary
#' table that includes standard errors and p-values.
#'
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @examples
#' # Example: Estimating parameters for a normal iAR model
#' library(iAR)
#' times <- 1:100
#' model <- iAR(family = "norm", times = times, coef = 0.9, hessian = TRUE)
#' model <- sim(model)  # Simulate the series
#' model <- loglik(model)  # Estimate parameters using MLE
#' print(model@coef)  # Access the estimated coefficients
#' print(model@loglik)  # Access the computed log-likelihood
#'
#' @export
loglik <- S7::new_generic("loglik","x")
S7::method(generic=loglik,signature=iAR) <- function(x) {
  if(length(x@series) == 0) stop("The loglik method needs a time series")
  if(any(is.na(x@series))) {
    warning("NA values were removed for parameter estimation.",call. = FALSE)
  }
  keep <- is.finite(x@series)
  series_aux <- x@series[keep]
  times_aux  <- x@times[keep]
  if(length(series_aux) < 3) stop("The Kalman method requires at least 3 observations")
  if(sd(series_aux, na.rm = TRUE) == 0)
    stop("The series is constant (zero variance). Likelihood estimation requires variability in the data.")
  if(length(x@series_esd) != 0) {
    series_esd_aux <- x@series_esd[keep]
  }
  else {
    series_esd_aux <- x@series_esd
  }
  if(x@family == "norm"){
    res <- iARloglik(series = series_aux,
                     times = times_aux,
                     series_esd = series_esd_aux,
                     zero_mean = x@zero_mean,
                     standardized = x@standardized,
                     hessian = x@hessian)
    x@coef <- .clamp_iar_coef(res$coef)
    x@loglik <- res$loglik
    x@summary <- res$summary
    if(x@hessian == TRUE)
    {
      if (!isTRUE(tryCatch({ chol(res$summary$hess); TRUE }, error = function(e) FALSE))) {
        warning("Hessian matrix is not positive definite or not invertible. ",call. = FALSE)
      }
      if (any(!is.finite(res$summary$stderr))) {
        warning("Some standard errors are non-finite.",call. = FALSE)
      }
      if (res$summary$convergence != 0) {
        warning(paste0("Optimization may not have converged (code = ", res$summary$convergence, ")."), call. = FALSE)
      }
    }
    return(x)
  }

  if(x@family == "t"){
    res <- iARt(series = series_aux,
                times = times_aux,
                df = x@df,
                hessian = x@hessian)
    x@coef <- .clamp_iar_coef(res$coef)
    x@sigma <- res$sigma
    x@loglik <- res$ll
    x@summary <- res$summary
    if(x@hessian == TRUE)
    {
      if (!isTRUE(tryCatch({ chol(res$summary$hess); TRUE }, error = function(e) FALSE))) {
        warning("Hessian matrix is not positive definite or not invertible. ",call. = FALSE)
      }
      if (any(!is.finite(res$summary$stderr))) {
        warning("Some standard errors are non-finite.",call. = FALSE)
      }
      if (res$summary$convergence != 0) {
        warning(paste0("Optimization may not have converged (code = ", res$summary$convergence, ")."), call. = FALSE)
      }
    }
    return(x)
  }

  if(x@family == "gamma"){
    res <- iARgamma(series = series_aux,
                    times = times_aux,
                    hessian = x@hessian)
    x@coef <- .clamp_iar_coef(res$coef)
    x@mean <- res$mean
    x@variance <- res$sigma
    x@loglik <- res$ll
    x@summary <- res$summary
    if(x@hessian == TRUE)
    {
      if (!isTRUE(tryCatch({ chol(res$summary$hess); TRUE }, error = function(e) FALSE))) {
        warning("Hessian matrix is not positive definite or not invertible. ",call. = FALSE)
      }
      if (any(!is.finite(res$summary$stderr))) {
        warning("Some standard errors are non-finite.",call. = FALSE)
      }
      if (res$summary$convergence != 0) {
        warning(paste0("Optimization may not have converged (code = ", res$summary$convergence, ")."), call. = FALSE)
      }
    }
    return(x)
  }
}

#' Forecast from iAR model
#'
#' Forecast from models fitted by \code{\link{iARloglik}}
#'
#' @param coef Estimated phi parameter by the iAR model.
#' @param series Array with the time series observations.
#' @param standardized logical; if TRUE, the array series is standardized; if FALSE, series contains the raw time series
#' @param zero_mean logical; if TRUE, the array series has zero mean; if FALSE, series has a mean different from zero.
#' @param tAhead The time ahead for forecast is required.
#'
#' @return Forecasted value from the iAR model
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#' \code{\link{gentime}}, \code{\link{iARsample}}, \code{\link{iARloglik}}, \code{\link{iARkalman}}, \code{\link{iARfit}}
#' @keywords internal
#' @examples
#' \dontshow{
#' set.seed(6714)
#' st<-gentime(n=100)
#' y<-iARsample(coef=0.99,times=st)
#' y<-y$series
#' n=length(y)
#' p=trunc(n*0.99)
#' ytr=y[1:p]
#' yte=y[(p+1):n]
#' str=st[1:p]
#' ste=st[(p+1):n]
#' tahead=ste-str[p]
#' phi=iARloglik(times=ytr,series=str)$coef
#' forIAR=iARforecast(coef=phi,series=ytr,tAhead=tahead)
#' }
#' @noRd
iARforecast<-function(coef,series,standardized=TRUE,zero_mean=TRUE,tAhead)
{
  sigma = 1
  mu = 0
  if (zero_mean == FALSE)
  {
    mu = mean(series)
    series = series-mu
  }
  if (standardized == FALSE)
  {
    sigma = var(series)
    series=series/sqrt(sigma)
  }
  y1=series[length(series)]
  fit=(coef**(tAhead)*y1)
  fit=(fit*sqrt(sigma)+mu)
  return(fitted=fit)
}

#' iAR: Irregularly Observed Autoregressive Models
#'
#' Description: Data sets, functions and scripts with examples to implement autoregressive
#' models for irregularly observed time series. The models available in this package
#' are the irregular autoregressive model (Eyheramendy et al.(2018) <doi:10.1093/mnras/sty2487>),
#' the complex irregular autoregressive model (Elorrieta et al.(2019) <doi:10.1051/0004-6361/201935560>)
#' and the bivariate irregular autoregressive model (Elorrieta et al.(2021) <doi:10.1093/mnras/stab1216>)
#'
#'
#' "_PACKAGE"
#' @name iAR-package
#' @import S7 Rcpp ggplot2 zoo
#' @importFrom Rdpack reprompt
#' @importFrom stats nlminb rexp
#' @importFrom stats rnorm runif qnorm
#' @importFrom stats rt anova
#' @importFrom stats lm residuals
#' @importFrom stats optimize optim var
#' @importFrom stats pnorm rgamma
#' @importFrom stats sd na.omit
#' @useDynLib iAR, .registration = TRUE
NULL

#' Fitted Values for iAR, CiAR, and BiAR Classes
#'
#' Fitted Values for the provided data. This method is implemented for:
#' 1. Irregular Autoregressive models (`iAR`)
#' 2. Complex Irregular Autoregressive models (`CiAR`)
#' 3. Bivariate Autoregressive models (`BiAR`)
#' 
#' @name fit
#'
#' @param x An object of class \code{iAR}, \code{CiAR}, or \code{BiAR}, containing the model specification and parameters:
#'   \itemize{
#'     \item For \code{iAR}:
#'       \itemize{
#'         \item \code{family}: The distribution family of the iAR model (one of "norm", "t", or "gamma").
#'         \item \code{series}: A numeric vector representing the time series to be fitted.
#'         \item \code{coef}: The coefficient(s) of the iAR model.
#'         \item \code{times}: A numeric vector specifying the time points of the series.
#'         \item \code{zero_mean}: Logical, whether to fit a zero-mean model.
#'         \item \code{standardized}: Logical, whether the model output should be standardized (for "norm" family).
#'         \item \code{mean}: The mean parameter (only for "gamma" family).
#'       }
#'     \item For \code{CiAR}:
#'       \itemize{
#'         \item \code{coef}: The real and imaginary parts of the CiAR model's coefficients.
#'         \item \code{series}: A numeric vector representing the time series to be fitted.
#'         \item \code{times}: A numeric vector specifying the time points of the series.
#'         \item \code{zero_mean}: Logical, whether to fit a zero-mean model.
#'         \item \code{standardized}: Logical, whether the model output should be standardized.
#'         \item \code{c}: A scaling parameter for the CiAR model.
#'       }
#'     \item For \code{BiAR}:
#'       \itemize{
#'         \item \code{coef}: The coefficients of the BiAR model (real and imaginary parts).
#'         \item \code{series}: A numeric matrix with two columns representing the bivariate time series to be fitted.
#'         \item \code{times}: A numeric vector specifying the time points of the series.
#'         \item \code{series_esd}: A numeric matrix for the error structure (optional, used internally).
#'         \item \code{zero_mean}: Logical, whether to fit a zero-mean model.
#'       }
#'   }
#'
#' @param ... Additional arguments (unused).
#'
#' @return An updated object of class \code{iAR}, \code{CiAR}, or \code{BiAR}, where the \code{fitted_values} property contains the fitted time series values.
#'
#' @details
#' This method fits the specified time series model to the data contained in the object. Depending on the class of the input object:
#' \itemize{
#'   \item For \code{iAR}, the function supports three distribution families:
#'     \item "norm" for normal distribution.
#'     \item "t" for t-distribution.
#'     \item "gamma" for gamma distribution.
#'   \item For \code{CiAR}, the function uses complex autoregressive processes.
#'   \item For \code{BiAR}, the function fits a bivariate autoregressive process.
#' }
#' All required parameters (e.g., coefficients, time points) must be set before calling this method.
#'
#' @references
#' \insertRef{Eyheramendy_2018}{iAR},\insertRef{Elorrieta_2019}{iAR},\insertRef{Elorrieta_2021}{iAR}
#'
#' @examples
#' # Example 1: Fitting a normal iAR model
#' library(iAR)
#' n=100
#' set.seed(6714)
#' times=gentime(n=n)@times
#' model_norm <- iAR(family = "norm", times = times, coef = 0.9)  
#' model_norm <- sim(model_norm)
#' model_norm <- kalman(model_norm) 
#' model_norm <- fit(model_norm)
#' plot(model_norm@times, model_norm@series, type = "l", main = "Original Series")
#' lines(model_norm@times, model_norm@fitted_values, col = "red", lwd = 2)
#' plot_fit(model_norm)
#'
#' # Example 2: Fitting a CiAR model
#' set.seed(6714)
#' model_CiAR <- CiAR(times = times,coef = c(0.9, 0))
#' model_CiAR <- sim(model_CiAR)
#' y=model_CiAR@series
#' y1=y/sd(y)
#' model_CiAR@series=y1
#' model_CiAR@series_esd=rep(0,n)
#' model_CiAR <- kalman(model_CiAR)
#' print(model_CiAR@coef)
#' model_CiAR <- fit(model_CiAR)
#' yhat=model_CiAR@fitted_values
#'
#' # Example 3: Fitting a BiAR model
#' n=80
#' set.seed(6714)
#' times=gentime(n=n)@times
#' model_BiAR <- BiAR(times = times,coef = c(0.9, 0.3), rho = 0.9)
#' model_BiAR <- sim(model_BiAR)
#' y=model_BiAR@series
#' y1=y/apply(y,2,sd)
#' model_BiAR@series=y1
#' model_BiAR@series_esd=matrix(0,n,2)
#' model_BiAR <- kalman(model_BiAR)
#' print(model_BiAR@coef) 
#' model_BiAR <- fit(model_BiAR)
#' print(model_BiAR@rho)
#' yhat=model_BiAR@fitted_values
#'
#' @export
fit <- S7::new_generic("fit", "x")
S7::method(generic = fit, signature = iAR) <- function(x) {
  if(length(x@series)==0) stop("The fit method needs a time series")
  if(x@family == "norm"){
    if(length(x@coef)==0) stop("The fit method needs the coefficient of the iAR model")
    res <- iARfit(coef = x@coef,
                  series = x@series,
                  times = x@times,
                  zero_mean = x@zero_mean,
                  standardized = x@standardized)
    x@fitted_values <- res
    return(x)
  }
  if(x@family == "t"){
    if(length(x@coef)==0) stop("The fit method needs the coefficient of the iAR-T model")
    res <- iARfit(coef = x@coef,
                  series = x@series,
                  times = x@times,
                  zero_mean = x@zero_mean,
                  standardized = FALSE) # preguntar a felipe
    x@fitted_values <- res
    x@standardized <- FALSE
    return(x)
  }
  if(x@family == "gamma"){
    if(length(x@coef)==0) stop("The fit method needs the coefficients of the iAR-Gamma model")
    res <- iARgfit(coef = x@coef,
                   series = x@series,
                   times = x@times,
                   mean = x@mean)
    x@fitted_values <- res
    return(x)
  }
}
S7::method(generic = fit, signature = CiAR)  <- function(x, c = 1) {
  if(length(x@series)==0) stop("The fit method needs a time series")
  if(length(x@coef)==0) stop("The fit method needs the coefficients of the CiAR model")
  res <- CiARfit(coef = x@coef,
                 series = x@series,
                 times = x@times,
                 zero_mean = x@zero_mean,
                 standardized = x@standardized,
                 c = c)$fitted
  x@fitted_values <- res
  return(x)
}
S7::method(generic = fit, signature = BiAR) <- function(x) {
  if(length(x@series)==0) stop("The fit method needs a bivariate time series")
  if(length(x@coef)==0) stop("The fit method needs the coefficients of the BiAR model")
  no_series_esd <- length(x@series_esd) == 0
  esd1 <- if(no_series_esd) rep(0, nrow(x@series)) else x@series_esd[, 1]
  esd2 <- if(no_series_esd) rep(0, nrow(x@series)) else x@series_esd[, 2]
  res <- BiARfit(coef = x@coef,
                   series1 = x@series[, 1],
                   series2 = x@series[, 2],
                   times = x@times,
                   series_esd1 = esd1,
                   series_esd2 = esd2,
                   zero_mean = x@zero_mean)
  x@fitted_values <- t(res$fitted)
  x@rho <- res$rho
  return(x)
}

#' Delta Scuti
#'
#' Time series of a Delta Scuti variable star obtained from HIPPARCOS.
#'
#' @format A data frame with 116 observations on the following 3 variables:
#' \describe{
#'   \item{t}{heliocentric Julian Day}
#'   \item{m}{magnitude}
#'   \item{merr}{measurement error of the magnitude (in mag).}
#' }
#' @details The frequency computed by GLS for this light curve is 14.88558646.
#' Catalogs and designations of this star:
#' HD 1989: HD 199757
#' TYCHO-2 2000: TYC 7973-401-1
#' USNO-A2.0: USNO-A2 0450-39390397
#' HIP: HIP 103684
#' @examples
#' data(dscut)
#' f1=14.88558646
#' o1=phase(data=dscut,f1=f1,twop=TRUE)
#' plot(o1@times_phased,o1@series_phased,pch=20)
"dscut"

#' Forecast from iAR-Gamma model
#'
#' Forecast from models fitted by \code{\link{iARgamma}}
#'
#' @param coef Estimated phi parameter by the iAR-Gamma model.
#' @param mean Estimated mean parameter by the iAR-Gamma model.
#' @param series Array with the time series observations.
#' @param tAhead The time ahead for forecast is required.
#'
#' @return Forecasted value from the iAR-Gamma model
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#' \code{\link{gentime}}, \code{\link{iARgsample}}, \code{\link{iARgamma}}, \code{\link{iARgfit}}
#' @keywords internal
#' @examples
#' \dontshow{
#' n=100
#' set.seed(6714)
#' st<-gentime(n)
#' y<-iARgsample(coef=0.9,times=st,sigma=1,mean=1)
#' y<-y$series
#' n=length(y)
#' p=trunc(n*0.99)
#' ytr=y[1:p]
#' yte=y[(p+1):n]
#' str=st[1:p]
#' ste=st[(p+1):n]
#' tahead=ste-str[p]
#' model<-iARgamma(series=ytr, times=str)
#' phi=model$coef
#' muest=model$mean
#' sigmaest=model$sigma
#' fit=iARgforecast(coef=phi,mean=muest,series=ytr,tAhead=tahead)
#' }
#' @noRd
iARgforecast<-function(coef,mean,series,tAhead)
{
  y1=series[c(length(series))]
  xd=coef**(tAhead)
  yhat = mean + xd * y1
  fit=c(yhat)
  return(fitted=fit)
}

#' `BiAR` Class
#'
#' Represents a bivariate irregular autoregressive (BiAR) time series model.
#' This class extends the `multidata` class and provides additional properties
#' for modeling, forecasting, and interpolation of bivariate time series data.
#'
#' @param times A numeric vector representing the time points.
#' @param series A numeric matrix or vector representing the values of the time series.
#' @param series_esd A numeric matrix or vector representing the error standard deviations of the time series.
#' @param series_names An optional character vector representing the name of the series. 
#' @param fitted_values A numeric vector containing the fitted values from the model.
#' @param loglik A numeric value representing the log-likelihood of the model.
#' @param kalmanlik A numeric value representing the Kalman likelihood of the model.
#' @param coef A numeric vector of length 2 containing the autoregressive (phiR) and
#'   cross-correlation (phiI) coefficients of the model. Each value must be in [-1, 1]
#'   and the modulus `sqrt(phiR^2 + phiI^2)` must be less than 1. Defaults to `c(0.8, 0)`.
#'   Unlike [iAR], which uses a scalar, both [CiAR] and `BiAR` use a length-2 vector.
#' @param rho A numeric vector containing the estimated coefficients of the model.
#' @param tAhead A numeric value specifying the forecast horizon (default: 1).
#' @param forecast A numeric vector containing the forecasted values.
#' @param interpolated_values A numeric vector containing the interpolated values.
#' @param interpolated_times A numeric vector containing the times of the interpolated data points.
#' @param interpolated_series A numeric vector containing the interpolated series.
#' @param zero_mean A logical value indicating if the model assumes a zero-mean process (default: TRUE).
#' @param standardized A logical value indicating if the model assumes a standardized process (default: TRUE).
#' @param hessian A logical value indicating whether the Hessian matrix is computed during estimation (default: FALSE).
#' @param summary A list containing the summary of the model fit, including diagnostics and statistical results.
#' 
#'
#' @section Validation Rules:
#' - `@times` must be a numeric vector without dimensions and strictly increasing.
#' - `@series` must be a numeric matrix with two columns (bivariate) or be empty.
#' - The number of rows in `@series` must match the length of `@times`.
#' - `@series_esd`, if provided, must be a numeric matrix. Its dimensions must match those of `@series`, or it must have one row and the same number of columns.
#' - If `@series_esd` contains NA values, they must correspond positionally to NA values in `@series`.
#' - `@series_names`, if provided, must be a character vector with length equal to the number of columns in `@series`, and all names must be unique.
#' - `@coef` must be a numeric vector of length 2, with each element strictly between -1 and 1.
#' - The modulus `sqrt(phiR^2 + phiI^2)` must be strictly less than 1 (stationarity condition).
#' - `@tAhead` must be a strictly positive numeric scalar.
#'
#' @details
#' The `BiAR` class is designed to handle bivariate irregularly observed time series data
#' using an autoregressive approach. It extends the `multidata` class to include 
#' additional properties for modeling bivariate time series.
#'
#' Key features of the `BiAR` class include:
#' - Support for bivariate time series data.
#' - Forecasting and interpolation functionalities for irregular time points.
#' - Assumptions of zero-mean and standardized processes, configurable by the user.
#' - Estimation of model parameters and likelihoods, including Kalman likelihood.
#'
#' @references
#' \insertRef{Elorrieta_2021}{iAR}
#'
#'
#' @examples
#' times=gentime(n=200, distribution = "expmixture",
#'              lambda1 = 130, lambda2 = 6.5, p1 = 0.15, p2 = 0.85)@times
#' my_BiAR <- BiAR(times = times,coef = c(0.9, 0.3), rho = 0.9)
#'
#' # Access properties
#' my_BiAR@coef
#'
#' @export
BiAR <- S7::new_class(
  "BiAR",
  parent = multidata,
  package = "iAR",
  properties = list(fitted_values = S7::class_numeric,
                    loglik = S7::class_numeric,
                    kalmanlik = S7::class_numeric,
                    coef = S7::new_property(
                      S7::class_numeric,
                      default = c(0.8, 0), # igual que el ciar
                      validator = function(value) {
                        if (!(is.numeric(value) &&
                              length(value) == 2 &&
                              is.null(dim(value)))) {
                          return("'coef' must be a numeric vector of length 2")
                        }
                        if (any(value < -1 | value > 1)) {
                          return("Each value in 'coef' must be between -1 and 1")
                        }
                        if (sqrt(sum(value^2)) >= 1) {
                          return(paste0(
                            "The modulus sqrt(phiR^2 + phiI^2) must be strictly less than 1 ",
                            "(got ", round(sqrt(sum(value^2)), 4), "). ",
                            "See the stationarity condition for BiAR models."
                          ))
                        }
                        NULL
                      }
                    ),
                    tAhead = S7::new_property(
                      S7::class_numeric,
                      default = 1,
                      validator = function(value) {
                        if (!(length(value) == 1 &&
                              is.null(dim(value)) &&
                              value > 0)) {
                          return("'tAhead' must be a strictly positive numeric scalar")
                        }
                        NULL
                      }
                    ),
                    rho = S7::new_property(
                      class_numeric,
                      default = 0,
                      validator = function(value) {
                        if (!(length(value) == 1 &&
                              is.null(dim(value)) &&
                              value >= -1 && value <= 1)) {
                          return("'rho' must be a single numeric value in [-1, 1]")
                        }
                        NULL
                      }
                    ),
                    forecast = S7::class_numeric,
                    interpolated_values = S7::class_numeric,
                    interpolated_times = S7::class_numeric,
                    interpolated_series = S7::class_numeric,
                    zero_mean = S7::new_property(class_logical, default = TRUE),
                    standardized = S7::new_property(class_logical, default = TRUE),
                    hessian = S7::new_property(class_logical, default = FALSE),
                    summary = S7::class_list))

#' Eclipsing Binaries (Beta Lyrae)
#'
#' Time series of a Beta Lyrae variable star obtained from OGLE.
#'
#' @format A data frame with 470 observations on the following 3 variables:
#' \describe{
#'   \item{t}{heliocentric Julian Day}
#'   \item{m}{magnitude}
#'   \item{merr}{measurement error of the magnitude (in mag).}
#' }
#' @details The frequency computed by GLS for this light curve is 1.510571586.
#' Catalogs and designations of this star:OGLE051951.22-694002.7
#' @examples
#' data(eb)
#' f1=1.510571586
#' o1=phase(data=eb,f1=f1,twop=TRUE)
#' plot(o1@times_phased,o1@series_phased,pch=20)
"eb"

#' Maximum Likelihood Estimation of Parameters for iAR, CiAR, and BiAR Models using the Kalman Filter
#'
#' Performs Maximum Likelihood Estimation (MLE) of the parameters of the iAR, CiAR, and BiAR models by maximizing the likelihood function using the Kalman filter.
#' This method applies the Kalman filter to compute the likelihood and estimate the model parameters that maximize the likelihood for each model type.
#'
#' @name kalman
#'
#' @param x An object of class \code{iAR}, \code{CiAR}, or \code{BiAR} containing the model specification and parameters:
#'   \itemize{
#'     \item For \code{iAR} models, the object should contain:
#'       \itemize{
#'         \item \code{series}: A numeric vector of the time series data.
#'         \item \code{times}: A numeric vector specifying the time points of the series.
#'         \item \code{series_esd}: A numeric vector specifying the error structure.
#'         \item \code{zero_mean}: A logical value indicating if the series should be zero-centered.
#'         \item \code{standardized}: A logical value indicating if the series should be standardized.
#'       }
#'     \item For \code{CiAR} models, the object should contain:
#'       \itemize{
#'         \item \code{series}: A numeric vector of the time series data.
#'         \item \code{times}: A numeric vector specifying the time points of the series.
#'         \item \code{series_esd}: A numeric vector specifying the error structure.
#'         \item \code{zero_mean}: A logical value indicating if the series should be zero-centered.
#'         \item \code{standardized}: A logical value indicating if the series should be standardized.
#'         \item \code{c}: A numeric value specifying the scale parameter for the CiAR model.
#'         \item \code{niter}: An integer specifying the number of iterations for the Kalman filter.
#'         \item \code{seed}: An integer seed for random number generation (optional).
#'       }
#'     \item For \code{BiAR} models, the object should contain:
#'       \itemize{
#'         \item \code{series}: A numeric matrix containing two columns for bivariate time series data.
#'         \item \code{times}: A numeric vector specifying the time points of the series.
#'         \item \code{series_esd}: A numeric matrix specifying the error structure for both series.
#'         \item \code{zero_mean}: A logical value indicating if the series should be zero-centered.
#'         \item \code{niter}: An integer specifying the number of iterations for the Kalman filter.
#'         \item \code{seed}: An integer seed for random number generation (optional).
#'       }
#'   }
#'
#' @param ... Additional arguments (unused).
#'
#' @return An updated object of class \code{iAR}, \code{CiAR}, or \code{BiAR}, where the \code{coef} property is updated with the estimated model parameters (using MLE) and the \code{kalmanlik} property contains the log-likelihood value of the model.
#'
#' @details
#' This function applies the Kalman filter to perform Maximum Likelihood Estimation (MLE) of the parameters of the autoregressive models (iAR, CiAR, BiAR). The Kalman filter is used to maximize the likelihood function based on the given time series data, and the parameters that maximize the likelihood are estimated.
#'
#' - For \code{iAR}, the Kalman filter is applied to estimate the model parameters by maximizing the likelihood.
#' - For \code{CiAR}, the Kalman filter is applied to estimate the parameters of the complex autoregressive model by maximizing the likelihood.
#' - For \code{BiAR}, the Kalman filter is applied to estimate the parameters of the bivariate autoregressive model by maximizing the likelihood.
#'
#' The method returns the updated model object, including the estimated parameters and the log-likelihood value.
#'
#' @references
#' \insertRef{Eyheramendy_2018}{iAR},\insertRef{Elorrieta_2019}{iAR},\insertRef{Elorrieta_2021}{iAR}
#'
#' @examples
#' # Example 1: Applying Kalman filter for MLE of iAR model parameters
#' library(iAR)
#' n=100
#' set.seed(6714)
#' times=gentime(n=n)@times
#' model_norm <- iAR(family = "norm", times = times, coef = 0.9,hessian=TRUE)
#' model_norm <- sim(model_norm)
#' model_norm <- kalman(model_norm)
#' print(model_norm@coef)  # Access the estimated coefficients
#' print(model_norm@kalmanlik)  # Access the Kalman likelihood value
#'
#' # Example 2: Applying Kalman filter for MLE of CiAR model parameters
#' set.seed(6714)
#' model_CiAR <- CiAR(times = times,coef = c(0.9, 0))
#' model_CiAR <- sim(model_CiAR)
#' y=model_CiAR@series
#' y1=y/sd(y)
#' model_CiAR@series=y1
#' model_CiAR@series_esd=rep(0,n)
#' model_CiAR <- kalman(model_CiAR)
#' print(model_CiAR@coef)
#' print(model_CiAR@kalmanlik)
#'
#' # Example 3: Applying Kalman filter for MLE of BiAR model parameters
#' set.seed(6714)
#' model_BiAR <- BiAR(times = times,coef = c(0.9, 0.3), rho = 0.9)
#' model_BiAR <- sim(model_BiAR)
#' y=model_BiAR@series
#' y1=y/apply(y,2,sd)
#' model_BiAR@series=y1
#' model_BiAR@series_esd=matrix(0,n,2)
#' model_BiAR <- kalman(model_BiAR)
#' print(model_BiAR@coef)
#' print(model_BiAR@kalmanlik)
#'
#' @export
kalman <- S7::new_generic("kalman", "x")
S7::method(kalman, iAR) <- function(x) {
  if(length(x@series) == 0) stop("The Kalman method needs a time series")
  if(any(is.na(x@series))) {
    warning("NA values were removed for parameter estimation.",call. = FALSE)
  }
  keep <- is.finite(x@series)
  series_aux <- x@series[keep]
  times_aux  <- x@times[keep]
  if(length(series_aux) < 3) stop("The Kalman method requires at least 3 observations")
  if(sd(series_aux, na.rm = TRUE) == 0)
    stop("The series is constant (zero variance). Likelihood estimation requires variability in the data.")
  if(length(x@series_esd) != 0) {
    series_esd_aux <- x@series_esd[keep]
  }
  else {
    series_esd_aux <- x@series_esd
  }
  if(x@family == "norm") {
    res <- iARkalman(series = series_aux,
                     times = times_aux,
                     series_esd = series_esd_aux,
                     zero_mean = x@zero_mean,
                     standardized = x@standardized,
                     hessian = x@hessian)
    x@coef <- .clamp_iar_coef(res$coef)
    x@kalmanlik <- res$kalman
    x@summary <- res$summary
    if(x@hessian == TRUE)
    {
      if (!isTRUE(tryCatch({ chol(res$summary$hess); TRUE }, error = function(e) FALSE))) {
        warning("Hessian matrix is not positive definite or not invertible. ",call. = FALSE)
      }
      if (any(!is.finite(res$summary$stderr))) {
        warning("Some standard errors are non-finite.",call. = FALSE)
      }
      if (res$summary$convergence != 0) {
        warning(paste0("Optimization may not have converged (code = ", res$summary$convergence, ")."), call. = FALSE)
      }
    }
    return(x)
  }
}

S7::method(kalman, CiAR) <- function(x, c = 1, niter = 10, seed = NULL) {
  if(length(x@series) == 0) stop("The Kalman method needs a time series")
  if(any(is.na(x@series))) {
    warning("NA values were removed for parameter estimation.",call. = FALSE)
  }
  keep <- is.finite(x@series)
  series_aux <- x@series[keep]
  times_aux  <- x@times[keep]
  if(length(series_aux) < 3) stop("The Kalman method requires at least 3 observations")
  if(sd(series_aux, na.rm = TRUE) == 0)
    stop("The series is constant (zero variance). Likelihood estimation requires variability in the data.")
  if(length(x@series_esd) != 0) {
    series_esd_aux <- x@series_esd[keep]
  }
  else {
    series_esd_aux <- x@series_esd
  }
  res <- CiARkalman(series = series_aux,
                    times = times_aux,
                    series_esd = series_esd_aux,
                    zero_mean = x@zero_mean,
                    standardized = x@standardized,
                    hessian = x@hessian,
                    c = c,
                    niter = niter,
                    seed = seed)
  x@coef <- c(res$phiR, res$phiI)
  if (sqrt(sum(x@coef^2)) > 0.995)
    warning(paste0(
      "Estimated modulus sqrt(phiR^2 + phiI^2) = ", round(sqrt(sum(x@coef^2)), 4),
      " is near 1. The model may be near non-stationary; results should be interpreted with caution."
    ), call. = FALSE)
  x@kalmanlik <- res$ll
  x@summary <- res$summary
  if(x@hessian == TRUE)
  {
    if (!isTRUE(tryCatch({ chol(res$summary$hess); TRUE }, error = function(e) FALSE))) {
    warning("Hessian matrix is not positive definite or not invertible. ",call. = FALSE)
    }
    if (any(!is.finite(res$summary$stderr))) {
    warning("Some standard errors are non-finite.",call. = FALSE)
    }
    if (res$summary$convergence != 0) {
      warning(paste0("Optimization may not have converged (code = ", res$summary$convergence, ")."), call. = FALSE)
    }
  }
  return(x)
}

S7::method(kalman, BiAR) <- function(x, niter = 10, seed = NULL) {
  if(length(x@series) == 0) stop("The Kalman method needs a bivariate time series")
  if(any(!is.finite(x@series))) {
    warning("Rows with NA values were removed for parameter estimation.", call. = FALSE)
  }
  keep <- apply(is.finite(x@series), 1, all)
  series_aux <- x@series[keep, , drop = FALSE]
  times_aux  <- x@times[keep]
  if(nrow(series_aux) < 3) stop("The Kalman method requires at least 3 observations")
  if(any(apply(series_aux, 2, sd, na.rm = TRUE) == 0))
    stop("One or both series are constant (zero variance). Likelihood estimation requires variability in the data.")

  no_series_esd <- length(x@series_esd) == 0
  if(no_series_esd) {
    esd1 <- 0
    esd2 <- 0
  } else {
    series_esd_aux <- x@series_esd[keep, , drop = FALSE]
    esd1 <- series_esd_aux[, 1]
    esd2 <- series_esd_aux[, 2]
  }
  res <- BiARkalman(series1 = series_aux[, 1],
                    series2 = series_aux[, 2],
                    times = times_aux,
                    series_esd1 = esd1,
                    series_esd2 = esd2,
                    zero_mean = x@zero_mean,
                    hessian = x@hessian,
                    niter = niter,
                    seed = seed)
  x@coef <- c(res$phiR, res$phiI)
  if (sqrt(sum(x@coef^2)) > 0.995)
    warning(paste0(
      "Estimated modulus sqrt(phiR^2 + phiI^2) = ", round(sqrt(sum(x@coef^2)), 4),
      " is near 1. The model may be near non-stationary; results should be interpreted with caution."
    ), call. = FALSE)
  x@kalmanlik <- res$ll
  x@summary <- res$summary
  if(x@hessian == TRUE)
  {
    if (!isTRUE(tryCatch({ chol(res$summary$hess); TRUE }, error = function(e) FALSE))) {
      warning("Hessian matrix is not positive definite or not invertible. ",call. = FALSE)
    }
    if (any(!is.finite(res$summary$stderr))) {
      warning("Some standard errors are non-finite.",call. = FALSE)
    }
    if (res$summary$convergence != 0) {
      warning(paste0("Optimization may not have converged (code = ", res$summary$convergence, ")."), call. = FALSE)
    }
  }
  return(x)
}

#' Test for the significance of the autocorrelation estimated by the iAR package models
#'
#' This function perform a test for the significance of the autocorrelation estimated by the iAR package models. This test is based in to take N disordered samples of the original data.
#'
#' @param series Array with the time series observations.
#' @param times Array with the irregular observational times.
#' @param series_esd Array with the variance of the measurement errors.
#' @param iter Number of disordered samples of the original data (N).
#' @param coef autocorrelation estimated by one of the iAR package models.
#' @param model model used to estimate the autocorrelation parameter ("iAR", "iAR-Gamma", "iAR-T", "CiAR" or "BiAR").
#' @param plot logical; if true, the function return a density plot of the distribution of the bad fitted examples; if false, this function does not return a plot.
#' @param xlim The x-axis limits (x1, x2) of the plot. Only works if plot='TRUE'. See \code{\link{plot.default}} for more details.
#' @param df degrees of freedom parameter of the iAR-T model.
#'
#'
#' @details The null hypothesis of the test is: The autocorrelation coefficient estimated for the time series belongs to the distribution of the coefficients estimated on the disordered data, which are assumed to be uncorrelated. Therefore, if the hypothesis is accepted, it can be concluded that the observations of the time series are uncorrelated.The statistic of the test is log(phi) which was contrasted with a normal distribution with parameters corresponding to the log of the mean and the variance of the phi computed for the N samples of the disordered data. This test differs for \code{\link{iARTest}} in that to perform this test it is not necessary to know the period of the time series.
#' @return A list with the following components:
#' \item{coef}{ MLE of the autocorrelation parameter of the model.}
#' \item{bad}{ MLEs of the autocorrelation parameters of the models that has been fitted to the disordered samples.}
#' \item{norm}{ Mean and variance of the normal distribution of the disordered data.}
#' \item{z0}{ Statistic of the test (log(abs(phi))).}
#' \item{pvalue}{ P-value computed for the test.}
#'
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{Planets}},\code{\link{iARTest}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' data(Planets)
#' t<-Planets[,1]
#' res<-Planets[,2]
#' y=res/sqrt(var(res))
#' #res3=iARloglik(y,t,standardized=TRUE)[1]
#' #res3$coef
#' #set.seed(6713)
#' #require(ggplot2)
#' #test<-iARPermutation(series=y,times=t,coef=res3$coef,model="iAR",plot=TRUE,xlim=c(-9.6,-9.45))
#' } 
#' @noRd
iARPermutation<-function (series, times, series_esd=0, iter = 100, coef,model="iAR", plot = TRUE, xlim = c(-1,0),df=3)
{
  phi2 = rep(0, iter)
  if(sum(series_esd)==0)
    series_esd=rep(0,length(series))
  for (i in 1:iter) {
    ord <- sample(1:length(series))
    y1 <- series[ord]
    merr1 <- series_esd[ord]
    if(model=="iAR")
    {
      phi2[i] = iARloglik(series = y1, times = times,merr1)$coef
    }
    if(model=="iAR-T")
    {
      phi2[i] = iARt(y1, times=times,df=df)$coef
    }
    if(model=="iAR-Gamma")
    {
      if(min(y1)<0)
        y1=y1-min(y1)
      phi2[i] = iARgamma(y1, times=times)$coef
    }
    if(model=="CiAR")
    {
      phi2[i] = CiARkalman(series=y1, times=times,series_esd=merr1)[1]$phiR
    }
    if(model=="BiAR")
    {
      y1=series
      y2=y1[2,]
      y1=y1[1,]
      if(sum(series_esd)==0)
        series_esd=matrix(0,nrow=2,ncol=length(y1))
      merr1=series_esd
      merr2=merr1[2,]
      merr1=merr1[1,]
      ord <- sample(1:length(y1))
      y1 <- y1[ord,]
      y2 <- y2[ord,]
      merr1 <- merr1[ord,]
      merr2 <- merr2[ord,]
      phi2[i] = BiARkalman(series1=y1,series2=y2,times=times,series_esd1 = merr1,series_esd2=merr2)$phiR
    }
  }
  mubf <- mean(log(abs(phi2)))
  sdbf <- sd(log(abs(phi2)))
  z0 <- log(abs(coef))
  pvalue <- pnorm(z0, mubf, sdbf)
  out <- NULL
  BAD =phi2
  if (plot == TRUE) {
    phi2 <- as.data.frame(phi2)
    coef <- as.data.frame(coef)
    out <- ggplot(phi2, aes(log(abs(phi2)))) + geom_density(adjust = 2) +
      xlab("") + ylab("") + theme_bw() + ggtitle("") +
      geom_point(data = coef, aes(log(abs(coef))), y = 0, size = 4,
                 col = "red", shape = 17) + xlim(xlim[1], xlim[2]) +
      theme(plot.title = element_text(face = "bold", size = 20),
            panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
            panel.background = element_blank())
    out
  }
  return(list(coef = coef, bad=BAD,norm = c(mubf, sdbf), z0 = z0, pvalue = pvalue,
              out = out))
}

#' @name cvnovag
#' @title ZTF g-band Cataclysmic Variable/Nova
#' @description Time series of a cataclysmic variable/nova object observed in the g-band of the ZTF survey and processed by the ALeRCE broker.ZTF Object code: ZTF18aayzpbr
#' @docType data
#' @usage cvnovag
#' @format A data frame with 67 observations on the following 3 variables:
#' \describe{
#'   \item{t}{heliocentric Julian Day - 2400000}
#'   \item{m}{magnitude}
#'   \item{merr}{measurement error standard deviations.}
#' }
#' @references
#' \insertRef{Forster_2021}{iAR}
#' @examples
#' data(cvnovag)
#' plot(cvnovag$t,cvnovag$m,type="l",ylab="",xlab="",col="green")
"cvnovag"

#' Interpolation from CiAR model
#'
#' Interpolation of missing values from models fitted by \code{\link{CiARkalman}}
#'
#' @param coef An array with the parameters of the CiAR model. The elements of the array are, in order, the real (phiR) and the imaginary (phiI) part of the coefficient of CiAR model.
#' @param series Array with the time series observations.
#' @param times Array with the irregular observational times.
#' @param series_esd Array with the measurements error standard deviations.
#' @param yini a single value, initial value for the estimation of the missing value of the time series.
#' @param zero_mean logical; if TRUE, the array series has zero mean; if FALSE, series has a mean different from zero.
#' @param standardized logical; if TRUE, the array series is standardized; if FALSE, series contains the raw time series.
#' @param c Nuisance parameter corresponding to the variance of the imaginary part.
#' @param seed a single value, interpreted as the seed of the random process.
#'
#' @return A list with the following components:
#' \itemize{
#' \item{fitted}{ Estimation of a missing value of the CiAR process.}
#' \item{ll}{ Value of the negative log likelihood evaluated in the fitted missing values.}
#' }
#' @references
#' \insertRef{Elorrieta_2019}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{CiARsample}}, \code{\link{CiARkalman}}
#'
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' n=100
#' set.seed(6714)
#' st<-gentime(n)
#' x=CiARsample(phiR=0.9,phiI=0,times=st,c=1)
#' y=x$series
#' y1=y/sd(y)
#' ciar=CiARkalman(series=y1,times=st)
#' ciar
#' napos=10
#' y0=y1
#' y1[napos]=NA
#' xest=c(ciar$phiR,ciar$phiI)
#' yest=CiARinterpolation(coef=xest,series=y1,times=st)
#' yest$fitted
#' mse=(y0[napos]-yest$fitted)^2
#' print(mse)
#' plot(st,y,type='l',xlim=c(st[napos-5],st[napos+5]))
#' points(st,y,pch=20)
#' points(st[napos],yest$fitted*sd(y),col="red",pch=20)
#' }
#' @noRd
CiARinterpolation<-function(coef,series,times,series_esd=0,yini=0,zero_mean=TRUE,standardized=TRUE,c=1,seed=NULL)
{
  set.seed(seed)
  aux<-1e10
  value<-1e10
  br<-0
  countNA = length(which(is.na(series)))
  if(countNA==0){
    stop("This function requires a NA value at the point to be interpolated.")}
  if(countNA > 1){
    stop("This function requires only one NA value.")}
  if(sum(series_esd)==0){
    series_esd=rep(0,length(series))}
  if (yini==0)
    yini = rnorm(1)
  out = optim(c(yini), CiARphikalman, lower = -Inf, upper = Inf,
              coef=coef, series=series, times=times,series_esd=series_esd, zero_mean=zero_mean, standardized=standardized,
              c=c, method="BFGS")
  par = out$par
  aux = out$value
  return(list(fitted = par, ll = aux))
}

#' Harmonic Fit to Time Series
#'
#' This function fit an k-harmonic function to time series data.
#'
#' @name harmonicfit
#' 
#' @usage harmonicfit(x, ...)
#'  
#' @param x An object of class `utilities`, or `NULL` (the default). When `x` is
#'   `NULL` or omitted, the `utilities` object is created internally, so
#'   `harmonicfit(data = clcep, f1 = f1)` is the recommended usage.
#' @param ... Additional arguments for pairing time series:
#'   \describe{
#'     \item{data}{A data frame with three columns corresponding to the time, values, and standard errors of the irregularly observed time series.}
#'     \item{f1}{frequency (1 / period) of the time series.}
#'     \item{nham}{Number of harmonic components in the model.}
#'     \item{weighted}{logical; if true, a weighted least squares (WLS) estimation is performed using weights based on the standard deviations of the errors. Default is 'FALSE'.}
#'     \item{remove_trend}{logical; if true, the linear trend of time series will be removed before the the harmonic model is fitted.}
#'    }
#' @return An object of class `utilities` with the slots:
#' \item{fitted_values}{Fitted values from the harmonic model.}
#' \item{residuals}{Residuals from the harmonic model.} 
#' \item{coef}{Estimated coefficients of the harmonic model.}
#' \item{summary}{A summary object containing detailed model information.}
#' 
#' @details The function fits a harmonic regression model with 'nham' components to the input time series, optionally removing a linear trend and allowing for weighted estimation when standard errors are available. 
#' @export
#' @references
#' \insertRef{Elorrieta_2021}{iAR}
#'
#' @examples
#' data(clcep)
#' f1=0.060033386
#' o1=harmonicfit(data=clcep,f1=f1)
#' head(o1@fitted_values)
#' head(o1@residuals)
harmonicfit <- S7::new_generic("harmonicfit", "x", function(x = NULL, ...) S7::S7_dispatch())
S7::method(harmonicfit, NULL) <- function(x, ...) {
  harmonicfit(utilities(), ...)
}

S7::method(harmonicfit, utilities) <- function (x,data, f1, nham = 4, weighted=FALSE, remove_trend=TRUE)
{
  mycurve = data
  t = mycurve[, 1]
  y = mycurve[, 2]
  merr=rep(0,length(t))
  if(dim(data)[2]>2)
    merr = mycurve[, 3]
  if(remove_trend==TRUE & weighted==FALSE)
    y = residuals(lm(mycurve[, 2] ~ t))
  if(remove_trend==TRUE & weighted==TRUE)
    y = residuals(lm(mycurve[, 2] ~ t,weights=merr))
  nam = NULL
  for (i in 1:nham) {
    y = cbind(y, sin(2 * i * pi * f1 * t), cos(2 * i * pi *
                                                 f1 * t))
    nam=c(nam,paste(c("sin","cos"),"comp",i,sep=""))
  }
  df = data.frame(y)
  names(df)[-1]=nam
  model = lm(y ~ ., data = df)
  if (weighted==TRUE)
    model = lm(y ~ ., data = df, weights = merr)
  #res = residuals(model)
  #R2 <- summary(model)$adj.r.squared
  #MSE <- anova(model)$"Mean Sq"[nham * 2 + 1]
  x@fitted_values <- model$fitted.values
  x@residuals <- residuals(model)
  x@coef <- summary(model)$coefficients
  x@summary <-summary(model)
  return(x)
}

#' Maximum Likelihood Estimation of the CiAR Model via Kalman Recursions
#'
#' Maximum Likelihood Estimation of the CiAR model parameters phiR and phiI. The estimation procedure uses the Kalman Filter to find the maximum of the likelihood.
#'
#' @param series Array with the time series observations.
#' @param times Array with the irregular observational times.
#' @param series_esd Array with the measurements error standard deviations.
#' @param zero_mean logical; if TRUE, the array series has zero mean; if FALSE, series has a mean different from zero.
#' @param standardized logical; if TRUE, the array series is standardized; if FALSE, series contains the raw time series.
#' @param hessian logical. Indicates whether to compute the Hessian matrix.; if TRUE, the Hessian matrix will be calculated; if FALSE, the Hessian will not be computed.
#' @param c Nuisance parameter corresponding to the variance of the imaginary part.
#' @param niter Number of iterations in which the function nlminb will be repeated.

#' @param seed a single value, interpreted as the seed of the random process.
#'
#' @return A list with the following components:
#' \itemize{
#' \item{phiR}{ MLE of the Real part of the coefficient of CiAR model (phiR).}
#' \item{phiI}{ MLE of the Imaginary part of the coefficient of the CiAR model (phiI).}
#' \item{ll}{ Value of the negative log likelihood evaluated in phiR and phiI.}
#' \item{summary}{ If hessian=T, returns the standard error and p-value of the estimated parameter phi.}
#' }
#' @references
#' \insertRef{Elorrieta_2019}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{CiARsample}}, \code{\link{CiARphikalman}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' n=100
#' set.seed(6714)
#' st<-gentime(n)
#' x=CiARsample(phiR=0.9,phiI=0,times=st,c=1)
#' y=x$series
#' y1=y/sd(y)
#' ciar=CiARkalman(series=y1,times=st)
#' ciar
#' Mod(complex(real=ciar$phiR,imaginary=ciar$phiI))
#' }
#' @noRd
CiARkalman<-function(series,times,series_esd=0,zero_mean=TRUE,standardized=TRUE,hessian=FALSE,c=1,niter=10,seed=1234)
{
  set.seed(seed)
  aux<-1e10
  value<-1e10
  br<-0
  if(sum(series_esd)==0){
    series_esd=rep(0,length(series))}
  summary=list()
  for(i in 1:niter)
  {
    phiR=2*runif(1)-1
    phiI=2*runif(1)-1
    if(Mod(complex(1,real=phiR,imaginary=phiI))<1)
    {
      if(hessian==F)
      {
        optim <- nlminb(start=c(phiR,phiI), objective=CiARphikalman, series=series, times=times,
                      series_esd=series_esd, zero_mean=zero_mean, standardized=standardized,
                      c=c,yest=0, lower=c(-1,-1), upper=c(1,1))
        value<-optim$objective
        par0<-optim$par
      }
      if(hessian==T)
      {
        out<-optim(par=c(phiR,phiI),fn=CiARphikalman,series=series, times=times,
                   series_esd=series_esd, zero_mean=zero_mean, standardized=standardized,
                   c=c,yest=0,method="L-BFGS-B",lower=c(-1,-1),upper=c(1,1),hessian=hessian)
        value<-out$value
        par0<-out$par
      }
    }
    if(aux>value)
    {
      par<-par0
      aux<-value
      if(hessian==T)
      {
        conv <- out$convergence
        hess=out$hessian
        stderr=as.numeric(sqrt(diag(solve(hess))))
        tvalue=par/stderr
        pvalue=2*pnorm(abs(tvalue),lower.tail=F)
        summary=list(hess=hess,stderr=stderr,tvalue=tvalue,pvalue=pvalue,convergence=conv)
      }
      br<-br+1
    }
    if(aux<=value & br>1 & i>trunc(niter/2))
      break;
  }
  if(aux==1e10)
    par<-c(0,0)
  return(list(phiR=par[1],phiI=par[2],ll=aux,summary=summary))
}

#' Interpolation for iAR, CiAR, and BiAR Classes
#'
#' Performs interpolation on time series with missing values. This method is implemented for:
#' 1. Irregular Autoregressive models (iAR)
#' 2. Complex Irregular Autoregressive models (CiAR)
#' 3. Bivariate Autoregressive models (BiAR)
#' 
#' @name interpolation
#' 
#' @param x An object of class \code{iAR}, \code{CiAR}, or \code{BiAR}, containing the model specification and parameters:
#'   \itemize{
#'     \item For \code{iAR}:
#'       \itemize{
#'         \item \code{family}: The distribution family of the iAR model (one of "norm", "t", or "gamma").
#'         \item \code{series}: A numeric vector representing the time series to interpolate.
#'         \item \code{coef}: The coefficients of the iAR model.
#'         \item \code{zero_mean}: Logical, whether the model assumes a zero-mean series.
#'         \item \code{standardized}: Logical, whether the model uses standardized data (only for "norm" family).
#'         \item \code{mean}: The mean parameter (only for "gamma" family).
#'       }
#'     \item For \code{CiAR}:
#'       \itemize{
#'         \item \code{coef}: The coefficients of the CiAR model.
#'         \item \code{series_esd}: The series of error standard deviations for the CiAR model.
#'         \item \code{zero_mean}: Logical, whether the model assumes a zero-mean series.
#'         \item \code{standardized}: Logical, whether the model uses standardized data.
#'         \item \code{seed}: Optional seed for random number generation.
#'       }
#'     \item For \code{BiAR}:
#'       \itemize{
#'         \item \code{coef}: The coefficients of the BiAR model.
#'         \item \code{series_esd}: The series of error standard deviations for the BiAR model.
#'         \item \code{zero_mean}: Logical, whether the model assumes a zero-mean series.
#'         \item \code{yini1}: Initial value for the first time series (for BiAR models).
#'         \item \code{yini2}: Initial value for the second time series (for BiAR models).
#'         \item \code{seed}: Optional seed for random number generation.
#'       }
#'   }
#'
#' @param ... Additional arguments (unused).
#'
#' @return An object of the same class as \code{x} with interpolated time series.
#'
#' @description
#' This method performs imputation of missing values in a time series using an autoregressive model.
#' The imputation is done iteratively for each missing value, utilizing available data and model coefficients.
#' Depending on the model family, the interpolation is performed differently:
#' - For \code{norm}: A standard autoregressive model for normally distributed data.
#' - For \code{t}: A model for time series with t-distributed errors.
#' - For \code{gamma}: A model for time series with gamma-distributed errors.
#' - For \code{CiAR}: A complex irregular autoregressive model.
#' - For \code{BiAR}: A bivariate autoregressive model.
#'
#' @details
#' The method handles missing values (NA) in the time series by imputing them iteratively. 
#' For each missing value, the available data is used to fit the autoregressive model, and the missing value is imputed based on the model's output.
#' For the \code{CiAR} and \code{BiAR} models, the error standard deviations and initial values are also considered during imputation.
#'
#' @seealso \code{\link{forecast}} for forecasting methods for these models.
#'
#' @references
#' \insertRef{Eyheramendy_2018}{iAR},\insertRef{Elorrieta_2019}{iAR},\insertRef{Elorrieta_2021}{iAR}
#'
#' @examples
#' # Interpolation for iAR model
#' library(iAR)
#' n=100
#' set.seed(6714)
#' times=gentime(n=n)@times
#' model_norm <- iAR(family = "norm", times = times, coef = 0.9)  
#' model_norm <- sim(model_norm)
#' y=model_norm@series
#' y1=y/sd(y)
#' model_norm@series=y1
#' model_norm@series_esd=rep(0,n)
#' model_norm <- kalman(model_norm) 
#' print(model_norm@coef)
#' napos=10
#' model_norm@series[napos]=NA
#' model_norm <- interpolation(model_norm)
#' interpolation=na.omit(model_norm@interpolated_values)
#' mse=as.numeric(y1[napos]-interpolation)^2
#' print(mse)
#' plot(times,y,type='l',xlim=c(times[napos-5],times[napos+5]))
#' points(times,y,pch=20)
#' points(times[napos],interpolation*sd(y),col="red",pch=20)
#' 
#' # Interpolation for CiAR model
#' model_CiAR <- CiAR(times = times,coef = c(0.9, 0))
#' model_CiAR <- sim(model_CiAR)
#' y=model_CiAR@series
#' y1=y/sd(y)
#' model_CiAR@series=y1
#' model_CiAR@series_esd=rep(0,n)
#' model_CiAR <- kalman(model_CiAR)
#' print(model_CiAR@coef)
#' napos=10
#' model_CiAR@series[napos]=NA
#' model_CiAR <- interpolation(model_CiAR)
#' interpolation=na.omit(model_CiAR@interpolated_values)
#' mse=as.numeric(y1[napos]-interpolation)^2
#' print(mse)
#' plot(times,y,type='l',xlim=c(times[napos-5],times[napos+5]))
#' points(times,y,pch=20)
#' points(times[napos],interpolation*sd(y),col="red",pch=20)

#' # Interpolation for BiAR model
#' model_BiAR <- BiAR(times = times,coef = c(0.9, 0.3), rho = 0.9)
#' model_BiAR <- sim(model_BiAR)
#' y=model_BiAR@series
#' y1=y/apply(y,2,sd)
#' model_BiAR@series=y1
#' model_BiAR@series_esd=matrix(0,n,2)
#' model_BiAR <- kalman(model_BiAR)
#' print(model_BiAR@coef) 
#' napos=10
#' model_BiAR@series[napos,1]=NA
#' model_BiAR@series_esd[napos,1]=NA
#' model_BiAR <- interpolation(model_BiAR)
#' interpolation=na.omit(model_BiAR@interpolated_values[,1])
#' mse=as.numeric(y1[napos,1]-interpolation)^2
#' print(mse)
#' par(mfrow=c(2,1))
#' plot(times,y[,1],type='l',xlim=c(times[napos-5],times[napos+5]))
#' points(times,y[,1],pch=20)
#' points(times[napos],interpolation*apply(y,1,sd)[1],col="red",pch=20)
#' plot(times,y[,2],type='l',xlim=c(times[napos-5],times[napos+5]))
#' points(times,y[,2],pch=20)
#' @export
interpolation <- S7::new_generic("interpolation", "x")
S7::method(generic = interpolation, signature = iAR) <- function(x, yini = 0) {
  if(length(x@series)==0) stop("The interpolation method needs a time series")
  if(x@family == "norm"){
    if(length(x@coef)==0) stop("The interpolation method needs the coefficient of the iAR model")
    # Original series
    vector <- x@series
    
    # Find positions of NA values in the vector
    nas <- which(is.na(vector))
    
    # If there are NA values, return the imputed vector
    if (length(nas) > 0) {
      for (i in seq_along(nas)) {
        # Current position of NA value
        na_pos <- nas[i]
        
        # Data available up to before the current NA value
        if((i+1) <= length(nas)){
          available_data <- vector[-nas[((i+1):length(nas))]]
          available_times <- x@times[-nas[((i+1):length(nas))]]
        }  else {
          available_data <- vector
          available_times <- x@times
        }
        if(length(x@series_esd) == 0){
          available_series_esd <- x@series_esd
        } else {
          if((i+1) <= length(nas)) available_series_esd <- x@series_esd[-nas[((i+1):length(nas))]] else available_series_esd <- x@series_esd
        }
        
        # Apply the imputation function to the available data
        imputed_value <- iARinterpolation(coef = x@coef,
                                          series = available_data,
                                          times = available_times,
                                          series_esd = available_series_esd,
                                          yini = yini,
                                          zero_mean = x@zero_mean,
                                          standardized = x@standardized)$fitted
        
        # Replace the NA value with the imputed value
        vector[na_pos] <- imputed_value
      }
      
      
      x@interpolated_values <- vector[nas]
      x@interpolated_times <- x@times[nas]
      x@interpolated_series <- vector
    }
    
    return(x)
  }
  if(x@family == "t"){
    if(length(x@coef)==0) stop("The interpolation method needs the coefficient of the iAR-T model")
    # Original series
    vector <- x@series
    
    # Find positions of NA values in the vector
    nas <- which(is.na(vector))
    
    # If there are NA values, return the imputed vector
    if (length(nas) > 0) {
      
      # Iterate over the positions of NA values
      for (i in seq_along(nas)) {
        # Current position of NA value
        na_pos <- nas[i]
        
        # Data available up to before the current NA value
        if((i+1) <= length(nas)){
          available_data <- vector[-nas[((i+1):length(nas))]]
          available_times <- x@times[-nas[((i+1):length(nas))]]
        }  else {
          available_data <- vector
          available_times <- x@times
        }
        if(length(x@series_esd) == 0){
          available_series_esd <- x@series_esd
        } else {
          if((i+1) <= length(nas)) available_series_esd <- x@series_esd[-nas[((i+1):length(nas))]] else available_series_esd <- x@series_esd
        }
        
        # Apply the imputation function to the available data
        imputed_value <- iARtinterpolation(coef = c(x@coef,x@sigma),
                                           series = available_data,
                                           times = available_times,
                                           df = x@df,
                                           # series_esd = available_series_esd, # es necesario aqui?
                                           # zero_mean = x@zero_mean, # no esta en la funcion original
                                           yini = yini)$fitted
        
        # Replace the NA value with the imputed value
        vector[na_pos] <- imputed_value
      }
      
      
      x@interpolated_values <- vector[nas]
      x@interpolated_times <- x@times[nas]
      x@interpolated_series <- vector
    }
    
    return(x)
  }
  
  if(x@family == "gamma"){
    if(length(x@coef)==0) stop("The interpolation method needs the coefficients of the iAR-Gamma model")
    # Original series
    vector <- x@series
    
    # Find positions of NA values in the vector
    nas <- which(is.na(vector))
    
    # If there are NA values, return the imputed vector
    if (length(nas) > 0) {
      
      # Iterate over the positions of NA values
      for (i in seq_along(nas)) {
        # Current position of NA value
        na_pos <- nas[i]
        
        # Data available up to before the current NA value
        if((i+1) <= length(nas)){
          available_data <- vector[-nas[((i+1):length(nas))]]
          available_times <- x@times[-nas[((i+1):length(nas))]]
        }  else {
          available_data <- vector
          available_times <- x@times
        }
        if(length(x@series_esd) == 0){
          available_series_esd <- x@series_esd
        } else {
          if((i+1) <= length(nas)) available_series_esd <- x@series_esd[-nas[((i+1):length(nas))]] else available_series_esd <- x@series_esd
        }
        
        # Apply the imputation function to the available data
        imputed_value <- iARginterpolation(coef = c(x@coef, x@mean, x@variance),
                                           series = available_data,
                                           times = available_times,
                                           # series_esd = available_series_esd, # preguntar?
                                           yini = yini)$fitted
        
        # Replace the NA value with the imputed value
        vector[na_pos] <- imputed_value
      }
      
      
      x@interpolated_values <- vector[nas]
      x@interpolated_times <- x@times[nas]
      x@interpolated_series <- vector
    }
    
    return(x)
  }
}
method(interpolation, CiAR) <- function(x, yini = 0, seed = NULL) {
  if(length(x@series)==0) stop("The interpolation method needs a time series")
  if(length(x@coef)==0) stop("The interpolation method needs the coefficients of the CiAR model")
  # Original series
  vector <- x@series
  
  # Find positions of NA values in the vector
  nas <- which(is.na(vector))
  
  # If there are NA values, return the imputed vector
  if (length(nas) > 0) {
    
    # Iterate over the positions of NA values
    for (i in seq_along(nas)) {
      # Current position of NA value
      na_pos <- nas[i]
      
      # Data available up to before the current NA value
      if((i+1) <= length(nas)){
        available_data <- vector[-nas[((i+1):length(nas))]]
        available_times <- x@times[-nas[((i+1):length(nas))]]
      }  else{
        available_data <- vector
        available_times <- x@times
      }
      if(length(x@series_esd) == 0){
        available_series_esd <- x@series_esd
      } else {
        if((i+1) <= length(nas)) available_series_esd <- x@series_esd[-nas[((i+1):length(nas))]] else available_series_esd <- x@series_esd
      }
      
      # Apply the imputation function to the available data
      imputed_value <- CiARinterpolation(coef = x@coef,
                                         series = available_data,
                                         times = available_times,
                                         series_esd = available_series_esd,
                                         yini = yini,
                                         zero_mean = x@zero_mean,
                                         standardized = x@standardized,
                                         seed = seed)$fitted
      
      # Replace the NA value with the imputed value
      vector[na_pos] <- imputed_value
    }
    
    
    x@interpolated_values <- vector[nas]
    x@interpolated_times <- x@times[nas]
    x@interpolated_series <- vector
  }
  
  return(x)
}
method(interpolation, BiAR) <- function(x, yini1 = 0, yini2 = 0, seed = NULL) {
  if(length(x@series)==0) stop("The interpolation method needs a bivariate time series")
  if(length(x@coef)==0) stop("The interpolation method needs the coefficients of the BiAR model")
  # Original series
  vector <- x@series
  # Find positions of NA values in the vector
  nas <- sort(unique(unlist(apply(is.na(vector),2,which))))
  auxtimes <- matrix(NA,dim(vector)[1],dim(vector)[2])
  auxseries <-  auxtimes
  # Local copy of series_esd to avoid mutating the S7 object inside the loop
  local_esd <- if(length(x@series_esd) == 0) {
    matrix(0, nrow = length(x@times), ncol = 2)
  } else {
    x@series_esd
  }
  # If there are NA values, return the imputed vector
  if (length(nas) > 0) {
    
    # Iterate over the positions of NA values
    for (i in seq_along(nas)) {
      # Current position of NA value
      na_pos <- nas[i]
      
      # Data available up to before the current NA value
      if((i+1) <= length(nas)){
        available_data <- vector[-nas[((i+1):length(nas))],]
        available_times <- x@times[-nas[((i+1):length(nas))]]
      }  else{
        available_data <- vector
        available_times <- x@times
      }
      na_serie=which(is.na(available_data[na_pos,]))
      nmiss=length(na_serie)
      local_esd[na_pos, na_serie] <- 0
      if((i+1) <= length(nas)) {
        available_series_esd <- local_esd[-nas[((i+1):length(nas))],]
      } else {
        available_series_esd <- local_esd
      }
      imputed_value <- iAR:::BiARinterpolation(coef = x@coef,
                                           series1 = available_data[,1],
                                           series2 = available_data[,2],
                                           times = available_times,
                                           series_esd1 = available_series_esd[,1],
                                           series_esd2 = available_series_esd[,2],
                                           yini1 = yini1,
                                           yini2 = yini2,
                                           zero_mean = x@zero_mean,nmiss=nmiss)$fitted
      # Replace the NA value with the imputed value
      vector[na_pos,na_serie] <- imputed_value
      auxtimes[na_pos,na_serie] <- x@times[na_pos]
      auxseries[na_pos,na_serie] <- vector[na_pos,na_serie]
    }
    x@interpolated_values <- auxseries
    x@interpolated_times <- auxtimes
    x@interpolated_series <- vector
  }
  return(x)
}

#' Multidata Class
#'
#' The `multidata` class is an S7 class designed to represent multidimensional time series models, including the main time series and additional series (e.g., error standard deviations or related variables).
#'
#' @param times A numeric vector representing the time points of the time series.
#' @param series A numeric vector or matrix representing the main time series.
#' @param series_esd A numeric vector or matrix representing the additional series, such as error standard deviations or other related data.
#' @param series_names An optional character vector representing the name of the series. 
#'
#' @section Validation Rules:
#' - `@times` must be a numeric vector and strictly increasing.
#' - `@series` must be a numeric matrix or empty. Its number of rows must match the length of `@times`.
#' - `@series_esd` must be a numeric matrix or empty. If provided and not empty, its dimensions must match those of `@series`.
#' - If `@series_names` is provided, it must be a character vector of the same length as the number of columns of `@series`, and contain unique values.
#'
#' @examples
#' # Create a multidata object
#' multidata_instance <- multidata(
#'   times = c(1, 2, 3, 4),
#'   series = matrix(c(10, 20, 15, 25,
#'                     12, 18, 17, 23), 
#'                   nrow = 4, ncol = 2),
#'   series_esd = matrix(c(1, 1.5, 1.2, 1.8,
#'                         0.9, 1.3, 1.1, 1.7),
#'                       nrow = 4, ncol = 2)
#' )
#' @seealso [BiAR]
#' @export
multidata <- S7::new_class(
  "multidata",
  package = "iAR",
  properties = list(
    times = S7::class_numeric,
    series = S7::class_numeric,
    series_esd = S7::class_numeric,
    series_names = S7::new_property(S7::class_character, default = character(0)),
    series_esd_names = S7::new_property(
      class = S7::class_character,
      getter = function(self) {
        if (length(self@series_names) > 0) {
          paste0(self@series_names, "_esd")
        } else {
          character(0)
        }
      }
    )
  ), validator = function(self) {
    if (!(is.atomic(self@times) && is.null(dim(self@times)))) {
      return("@times must be a numeric vector without dimensions")
    }
    if (!(is.matrix(self@series) && is.numeric(self@series)) && length(self@series) != 0) {
      return("@series must be a numeric matrix or be empty")
    }
    if (!(is.matrix(self@series_esd) && is.numeric(self@series_esd)) && length(self@series_esd) != 0) {
      return("@series_esd must be a numeric matrix or empty")
    }
    if (length(self@series) > 0) {
    if (nrow(self@series) != length(self@times)) {
      return("Number of rows in @series must match length of @times")
    }
    }
    if (length(self@series_esd) > 0 && length(self@series) > 0) {
      if (!all(dim(self@series) == dim(self@series_esd))) {
        return("Dimensions of @series_esd must match those of @series")
      }
    }
    # Validar series_names
    if (ncol(self@series) > 0 && length(self@series_names) > 0) {
      if (length(self@series_names) != ncol(self@series)) {
        return("Length of @series_names must match number of columns in @series")
      }
      if (anyDuplicated(self@series_names)) {
        return("@series_names must be unique")
      }
    }
    if (is.unsorted(self@times, strictly = TRUE)) {
      return("@times must be strictly increasing")
    }
    NULL
  }
)

#' Double Mode Cepheid.
#'
#' Time series of a double mode cepheid variable star obtained from OGLE.
#'
#' @format A data frame with 191 observations on the following 3 variables:
#' \describe{
#'   \item{t}{heliocentric Julian Day}
#'   \item{m}{magnitude}
#'   \item{merr}{measurement error of the magnitude (in mag).}
#' }
#' @details The dominant frequency computed by GLS for this light curve is 0.7410152.
#' The second frequency computed by GLS for this light curve is 0.5433353.
#' OGLE-ID:175210
#' @examples
#' data(dmcep)
#' f1=0.7410152
#' o1=phase(data=dmcep,f1=f1,twop=TRUE)
#' plot(o1@times_phased,o1@series_phased,pch=20)
#' #fit=harmonicfit(dmcep,f1)
#' #f2=0.5433353
#' #foldlc(cbind(dmcep$t,fit$res,dmcep$merr),f2)
"dmcep"

#' Simulate Time Series for Multiple iAR Models
#'
#' Simulates a time series for irregular autoregressive (iAR) models, including:
#' 1. Normal iAR model (`iAR`)
#' 2. T-distribution iAR model (`iAR-T`)
#' 3. Gamma-distribution iAR model (`iAR-Gamma`)
#'
#' @name sim
#'
#' @param x An object of class \code{iAR}, \code{CiAR}, or \code{BiAR}, containing the model specification and parameters:
#'   \itemize{
#'     \item For \code{iAR} (irregular AR models), the model family could be "norm", "t", or "gamma", where:
#'       \itemize{
#'         \item \code{family}: The distribution family of the iAR model (one of "norm", "t", or "gamma").
#'         \item \code{coef}: The coefficient(s) of the iAR model.
#'         \item \code{times}: A numeric vector specifying the time points of the series.
#'         \item \code{df}: Degrees of freedom for the t-distribution (only for \code{family = "t"}).
#'         \item \code{sigma}: The scale parameter for the t-distribution (only for \code{family = "t"}).
#'         \item \code{mean}: The mean parameter for the gamma distribution (only for \code{family = "gamma"}).
#'         \item \code{variance}: The variance parameter for the gamma distribution (only for \code{family = "gamma"}).
#'       }
#'     \item For \code{CiAR} (complex irregular autoregressive models):
#'       \itemize{
#'         \item \code{coef}: The real and imaginary parts of the CiAR model's coefficients.
#'         \item \code{times}: A numeric vector specifying the time points of the series.
#'         \item \code{rho}: The correlation parameter for the CiAR model.
#'         \item \code{c}: The scale parameter for the CiAR model.
#'       }
#'     \item For \code{BiAR} (BiAR models):
#'       \itemize{
#'         \item \code{coef}: The coefficients of the BiAR model (real and imaginary).
#'         \item \code{times}: A numeric vector specifying the time points of the series.
#'         \item \code{rho}: The correlation parameter for the BiAR model.
#'         \item \code{series_esd}: The series for the error structure (optional, used internally).
#'       }
#'   }
#' @param ... Additional arguments for generating irregular times. This is used only if no time points have been provided in the \code{times} argument:
#'   \describe{
#'   \item{n}{A positive integer. Length of observation times. Default is 100. This is used only if no time points are provided in \code{times}.}
#'   \item{distribution}{A character string specifying the distribution of the observation times. Default is `"expmixture"`. Available options are:
#'   \describe{
#'     \item{expmixture}{A mixture of two exponential distributions.}
#'     \item{uniform}{A uniform distribution.}
#'     \item{exponential}{A single exponential distribution.}
#'     \item{gamma}{A gamma distribution.}
#'     }
#'     }
#'     \item{lambda1}{Mean (1/rate) of the exponential distribution or the first exponential distribution in a mixture of exponential distributions. Default is `130`.}
#'     \item{lambda2}{Mean (1/rate) of the second exponential distribution in a mixture of exponential distributions. Default is `6.5`.}
#'     \item{p1}{Weight of the first exponential distribution in a mixture of exponential distributions. Default is `0.15`.}
#'     \item{p2}{Weight of the second exponential distribution in a mixture of exponential distributions. Default is `0.85`.}
#'     \item{a}{Shape parameter of a gamma distribution or lower limit of the uniform distribution. Default is `0`.}
#'     \item{b}{Scale parameter of a gamma distribution or upper limit of the uniform distribution. Default is `1`.}
#'   }
#'
#' 
#' @return An updated object of class \code{iAR}, \code{CiAR}, or \code{BiAR}, where the \code{series} property contains the simulated time series.
#'
#' @details
#' This function simulates time series based on the specified model and its parameters. Depending on the class of the input object:
#' \itemize{
#'   \item For \code{iAR} models, it supports three distribution families:
#'     \item "norm" for normal distribution.
#'     \item "t" for t-distribution.
#'     \item "gamma" for gamma distribution.
#'   \item For \code{CiAR} models, it uses complex autoregressive processes to generate the time series.
#'   \item For \code{BiAR} models, it simulates a BiAR process using specified coefficients and correlation.
#' }
#' The coefficients and any family-specific parameters must be set before calling this function.
#'
#' @references
#' \insertRef{Eyheramendy_2018}{iAR},\insertRef{Elorrieta_2019}{iAR},\insertRef{Elorrieta_2021}{iAR}
#'
#' @examples
#' # Example 1: Simulating a normal iAR model
#' library(iAR)
#' n=100
#' set.seed(6714)
#' times=gentime(n=n)@times
#' model_norm <- iAR(family = "norm", times = times, coef = 0.9,hessian=TRUE)
#' model_norm <- sim(model_norm)
#' plot(model_norm, type = "l", main = "Simulated iAR-Norm Series")
#'
#' # Example 2: Simulating a CiAR model
#' set.seed(6714)
#' model_CiAR <- CiAR(times = times,coef = c(0.9, 0))
#' model_CiAR <- sim(model_CiAR)
#' plot(model_CiAR , type = "l", main = "Simulated CiAR Series")
#'
#' # Example 3: Simulating a BiAR model
#' set.seed(6714)
#' model_BiAR <- BiAR(times = times,coef = c(0.9, 0.3), rho = 0.9)
#' model_BiAR <- sim(model_BiAR)
#' plot(times, model_BiAR@series[,1], type = "l", main = "Simulated BiAR Series")
#'
#' @export
sim <- S7::new_generic("sim", "x")
S7::method(generic = sim, signature = iAR) <- function(x, n = 100, distribution = "expmixture", lambda1 = 130, 
                                                       lambda2 = 6.5, p1 = 0.15, p2 = 0.85, a = 0, b = 1) {
    if (is.null(x@times) || length(x@times) == 0) {
      message("Generating irregular time points using the gentime method of the class utilities...")
      x@times <- gentime(NULL, n = n, distribution = distribution, lambda1 = lambda1,
                         lambda2 = lambda2, p1 = p1, p2 = p2, a = a, b = b)@times
    }
  if(x@family == "norm") {
    if(length(x@coef) == 0) stop("The sim method needs the coefficient of the iAR model")
    res <- iARsample(coef = x@coef, times = x@times)$series
    x@series <- res
    return(x)
  }
  if(x@family == "t") {
    if(length(x@coef) == 0) stop("The sim method needs the coefficient of the iAR-T model")
    res <- iARtsample(coef = x@coef, times = x@times, df = x@df, sigma = x@sigma)$series
    x@series <- res
    return(x)
  }
  if(x@family == "gamma") {
    if(length(x@coef) == 0) stop("The sim method needs the coefficient of the iAR-Gamma model")
    if(x@mean <= 0)
      stop(paste0(
        "For family = 'gamma', '@mean' must be strictly positive (got ", x@mean, "). ",
        "Set e.g. model@mean <- 1 before calling sim()."
      ))
    if(x@variance <= 0)
      stop(paste0(
        "For family = 'gamma', '@variance' must be strictly positive (got ", x@variance, ")."
      ))
    min_gap <- min(diff(x@times))
    if(min_gap < 1e-6)
      warning(paste0(
        "iAR-Gamma: minimum time gap is ", signif(min_gap, 3),
        ". With phi near 1, very small gaps cause gL -> 0 and numerical instability."
      ), call. = FALSE)
    res <- iARgsample(coef = x@coef, times = x@times, mean = x@mean, sigma = x@variance)$series
    x@series <- res
    return(x)
  }
}
S7::method(generic = sim, signature = CiAR) <- function(x, rho = 0, c = 1, n = 100, distribution = "expmixture", lambda1 = 130, 
                                                        lambda2 = 6.5, p1 = 0.15, p2 = 0.85, a = 0, b = 1) {
  if (is.null(x@times) || length(x@times) == 0) {
    message("Generating irregular time points using the gentime method of the class utilities...")
    x@times <- gentime(NULL, n = n, distribution = distribution, lambda1 = lambda1,
                       lambda2 = lambda2, p1 = p1, p2 = p2, a = a, b = b)@times
  }
  if(length(x@coef) == 0) stop("The sim method needs the coefficients of the CiAR model")
  res <- CiARsample(phiR = x@coef[1], phiI = x@coef[2], times = x@times, rho = rho, c = c)$series
  x@series <- res
  return(x)
}
S7::method(generic = sim, signature = BiAR) <- function(x, n = 100, distribution = "expmixture", lambda1 = 130, 
                                                        lambda2 = 6.5, p1 = 0.15, p2 = 0.85, a = 0, b = 1) {
  if (is.null(x@times) || length(x@times) == 0) {
    message("Generating irregular time points using the gentime method of the class utilities...")
    x@times <- gentime(NULL, n = n, distribution = distribution, lambda1 = lambda1,
                       lambda2 = lambda2, p1 = p1, p2 = p2, a = a, b = b)@times
  }
  if(length(x@coef) == 0) stop("The sim method needs the coefficients of the BiAR model")
  no_series_esd <- length(x@series_esd) == 0
  
  if(no_series_esd) x@series_esd <- matrix(0, ncol = 2)
  res <- BiARsample(times = x@times, phiR = x@coef[1], phiI = x@coef[2], series_esd1 = x@series_esd[,1], 
                    series_esd2 = x@series_esd[,2], rho = x@rho)$series
  if(no_series_esd) x@series_esd <- integer(0)
  x@series <- cbind(res[1,],res[2,])
  
  return(x)
}

#' Pairing two irregularly observed time series
#'
#' This method pairs the observational times of two irregularly observed time series.
#' @name pairingits
#' 
#' @usage pairingits(x, ...)
#' 
#' @param x An object of class `utilities`, or `NULL` (the default). When `x` is
#'   `NULL` or omitted, the `utilities` object is created internally, so
#'   `pairingits(data1 = ts1, data2 = ts2)` is the recommended usage.
#' @param ... Additional arguments for pairing time series: 
#'   \describe{
#'     \item{data1}{A data frame with three columns corresponding to the first irregularly observed time series.}
#'     \item{data2}{A data frame with three columns corresponding to the second irregularly observed time series.}
#'     \item{tol}{A numeric value indicating the tolerance parameter.}
#'   }
#'
#'
#' @return An object of class `utilities` with the slots:
#' \item{series}{A matrix containing the paired time series, where unmatched measurements are filled with `NA`.}
#' \item{series_esd}{A matrix containing the paired error standard deviations of the time series, where unmatched measurements are filled with `NA`.}
#' \item{times}{A numeric vector with the paired observational times.}
#'
#' @details The method checks the observational times in both input time series and pairs the measurements if they fall within the specified tolerance (`tol`). If a measurement in one series cannot be paired, it is filled with `NA` values for the corresponding columns of the other series.
#'
#' @export
#' @references
#' \insertRef{Elorrieta_2021}{iAR}
#'
#' @examples
#' data(cvnovag)
#' data(cvnovar)
#' o1=pairingits(data1=cvnovag, data2=cvnovar, tol=0.1)
#' pargr1=na.omit(o1@paired)
#' st=apply(pargr1[,c(1,4)],1,mean)
#' model_BiAR <- BiAR(times = st,series=pargr1[,c(2,5)],series_esd=pargr1[,c(3,6)])
#' model_BiAR <- kalman(model_BiAR)
#' 
pairingits <- S7::new_generic("pairingits", "x", function(x = NULL, ...) S7::S7_dispatch())

S7::method(pairingits, NULL) <- function(x, data1, data2, tol = 0.1) {
  pairingits(utilities(), data1 = data1, data2 = data2, tol = tol)
}

S7::method(pairingits, S7::class_any) <- function(x, data2, tol = 0.1) {
  if (!is.data.frame(x))
    stop("No method for pairingits() with x of class '", class(x)[1],
         "'. Supply a data.frame or omit x entirely.", call. = FALSE)
  pairingits(utilities(), data1 = x, data2 = data2, tol = tol)
}

S7::method(pairingits, utilities) <- function(x, data1,data2,tol=0.1)
{
  t1=data1[,1]
  t2=data2[,1]
  A=cbind(c(t1,t2),c(rep(1,length(t1)),rep(2,length(t2))),c(1:length(t1),1:length(t2)))
  A=A[order(A[,1]),]
  fin=NULL
  i=2
  while(i<=dim(A)[1])
  {
    if(A[i-1,2]!=A[i,2])
    {
      dt=diff(c(A[i,1],A[i-1,1]))
      if(abs(dt)<tol)
      {
        if(A[i,2]>A[i-1,2])
          par=c(data1[A[i-1,3],],data2[A[i,3],])
        if(A[i,2]<A[i-1,2])
          par=c(data1[A[i,3],],data2[A[i-1,3],])
        i=i+1
      }
      else
      {
        if(A[i-1,2]==1)
          par=c(data1[A[i-1,3],],rep(NA,3))
        if(A[i-1,2]==2)
          par=c(rep(NA,3),data2[A[i-1,3],])
      }
    }
    else
    {
      if(A[i-1,2]==1 & A[i,2]==1)
        par=c(data1[A[i-1,3],],rep(NA,3))
      if(A[i-1,2]==2)
        par=c(rep(NA,3),data2[A[i-1,3],])
    }
    fin=rbind(fin,c(unlist(par)))
    i=i+1
  }
  i=i-1
  if(i==dim(A)[1])
  {
    if(A[i-1,2]==A[i,2] | (A[i-1,2]!=A[i,2] & abs(dt)>=tol))
    {
      if(A[i,2]==1)
        par=c(data1[A[i,3],],rep(NA,3))
      if(A[i,2]==2)
        par=c(rep(NA,3),data2[A[i,3],])
      fin=rbind(fin,c(unlist(par)))
    }
  }
  prop=dim(na.omit(fin))[1]
  x@series <- fin[,c(2,5)]
  x@series_esd <- fin[,c(3,6)]
  x@times <- apply(fin[,c(1,4)],1,mean)
  x@paired <- fin
  return(x)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Fitted Values of BiAR model
#'
#' Fit a BiAR model to a bivariate irregularly observed time series.
#'
#' @param coef An array with the parameters of the BiAR model. The elements of the array are, in order, the autocorrelation and the cross correlation parameter of the BiAR model.
#' @param series1 Array with the observations of the first time series of the BiAR process.
#' @param series2 Array with the observations of the second time series of the BiAR process.
#' @param times Array with the irregular observational times.
#' @param series_esd1 Array with the measurements error standard deviations of the first time series of the BiAR process.
#' @param series_esd2 Array with the measurements error standard deviations of the second time series of the BiAR process.
#' @param zero_mean logical; if TRUE, the array y has zero mean; if FALSE, y has a mean different from zero.
#'
#' @return A list with the following components:
#' \itemize{
#' \item{rho}{ Estimated value of the contemporary correlation coefficient.}
#' \item{innov.var}{ Estimated value of the innovation variance.}
#' \item{fitted}{ Fitted values of the BiAR model.}
#' \item{fitted.state}{ Fitted state values of the BiAR model.}
#' \item{Lambda}{ Lambda value estimated by the BiAR model at the last time point.}
#' \item{Theta}{ Theta array estimated by the BiAR model at the last time point.}
#' \item{Sighat}{ Covariance matrix estimated by the BiAR model at the last time point.}
#' \item{Qt}{ Covariance matrix of the state equation estimated by the BiAR model at the last time point.}
#' }
#' @references
#' \insertRef{Elorrieta_2021}{iAR}
#' @seealso
#' \code{\link{gentime}}, \code{\link{BiARsample}}, \code{\link{BiARphikalman}}, \code{\link{BiARkalman}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' n=80
#' set.seed(6714)
#' st<-gentime(n)
#' x=BiARsample(phiR=0.9,phiI=0.3,times=st,rho=0.9)
#' y=x$series
#' y1=y/apply(y,1,sd)
#' yerr1=rep(0,n)
#' yerr2=rep(0,n)
#' biar=BiARkalman(series1=y1[1,],series2=y1[2,],times=st,series_esd1 = yerr1,series_esd2=yerr2)
#' biar
#' predbiar=BiARfit(coef=c(biar$phiR,biar$phiI),series1=y1[1,],series2=y1[2,],times=st,series_esd1 = 
#' rep(0,length(y[1,])),series_esd2=rep(0,length(y[1,])))
#' rho=predbiar$rho
#' print(rho)
#' yhat=predbiar$fitted
#' }
#' @noRd
BiARfit <- function(coef, series1, series2, times, series_esd1, series_esd2, zero_mean = TRUE) {
    .Call(`_iAR_BiARfit`, coef, series1, series2, times, series_esd1, series_esd2, zero_mean)
}

#' Forecast from BiAR model
#'
#' Forecast from models fitted by \code{\link{BiARkalman}}
#'
#' @param coef An array with the parameters of the BiAR model. The elements of the array are, in order, the autocorrelation and the cross-correlation coefficient of the BiAR model.
#' @param series1 Array with the observations of the first time series of the BiAR process.
#' @param series2 Array with the observations of the second time series of the BiAR process.
#' @param times Array with the observational times.
#' @param tAhead The time ahead for which the forecast is required.
#'
#' @return A list with the following components:
#' \itemize{
#' \item{forecast}{ Point forecast in the time ahead required.}
#' }
#' @references
#' \insertRef{Elorrieta_2021}{iAR}
#'
#' @seealso
#'
#' \code{\link{BiARsample}}, \code{\link{BiARkalman}}, \code{\link{BiARfit}}
#'
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' #Simulated Data
#' n=100
#' set.seed(6714)
#' st<-gentime(n)
#' x=BiARsample(phiR=0.9,phiI=0.3,times=st)
#' biar=iAR::BiARkalman(series1=x$series[1,],series2=x$series[2,],times=st)
#' forBiAR<-BiARforecast(coef=c(biar$phiR,biar$phiI),series1=x$series[1,],series2=x$series[2,],
#' times=st,tAhead=c(1.3))
#' }
#' @noRd
BiARforecast <- function(coef, series1, series2, times, tAhead) {
    .Call(`_iAR_BiARforecast`, coef, series1, series2, times, tAhead)
}

#' Minus Log Likelihood of the BiAR Model
#'
#' This function return the negative log likelihood of the BiAR process given specific values of phiR and phiI
#'
#'
#' @param coef An array with the parameters of the BiAR model. The elements of the array are, in order, the autocorrelation and the cross-correlation coefficient of the BiAR model.
#' @param series1 Array with the observations of the first time series of the BiAR process.
#' @param series2 Array with the observations of the second time series of the BiAR process.
#' @param times Array with the irregular observational times.
#' @param series_esd1 Array with the measurements error standard deviations of the first time series of the BiAR process.
#' @param series_esd2 Array with the measurements error standard deviations of the second time series of the BiAR process.
#' @param zero_mean logical; if TRUE, the array y has zero mean; if FALSE, y has a mean different from zero.
#' @param yest An array with the estimate of a missing value in one or both time series of the bivariate process. This function recognizes a missing value with a NA. If the bivariate time series does not have a missing value, this value does not affect the computation of the likelihood.
#'
#' @return Value of the negative log likelihood evaluated in phiR and phiI.
#'
#' @references
#' \insertRef{Elorrieta_2021}{iAR}
#' @seealso
#' \code{\link{gentime}}, \code{\link{BiARsample}}
#'
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' n=300
#' set.seed(6714)
#' st<-gentime(n)
#' x=BiARsample(phiR=0.9,phiI=0.3,times=st)
#' y=x$series
#' y1=y[1,]
#' y2=y[2,]
#' yerr1=rep(0,n)
#' yerr2=rep(0,n)
#' BiARphikalman(coef=c(0.8,0.2),series1=y1,series2=y2,times=st,
#' series_esd1=yerr1,series_esd2=yerr2,yest=c(0,0))
#' }
#' @noRd
BiARphikalman <- function(yest, coef, series1, series2, times, series_esd1, series_esd2, zero_mean = TRUE) {
    .Call(`_iAR_BiARphikalman`, yest, coef, series1, series2, times, series_esd1, series_esd2, zero_mean)
}

#' Fitted Values of CiAR model
#'
#' Fit a CiAR model to an irregularly observed time series.
#'
#' @param coef An array with the parameters of the CiAR model. The elements of the array are, in order, the real and the imaginary part of the phi parameter of the CiAR model.
#' @param series Array with the time series observations.
#' @param times Array with the irregular observational times.
#' @param standardized logical; if TRUE, the array series is standardized; if FALSE, series contains the raw time series
#' @param zero_mean logical; if TRUE, the array series has zero mean; if FALSE, series has a mean different from zero.
#' @param c Nuisance parameter corresponding to the variance of the imaginary part.
#'
#' @return A list with the following components:
#' \item{fitted}{ Fitted values of the observable part of CiAR model.}
#' \item{xhat}{ Fitted values of both observable part and imaginary part of CiAR model.}
#' \item{Lambda}{ Lambda value estimated by the CiAR model at the last time point.}
#' \item{Theta}{ Theta array estimated by the CiAR model at the last time point.}
#' \item{Sighat}{ Covariance matrix estimated by the CiAR model at the last time point.}
#' \item{Qt}{ Covariance matrix of the state equation estimated by the CiAR model at the last time point.}
#' @references
#' \insertRef{Elorrieta_2019}{iAR}
#' 
#' @keywords internal
#' @examples
#' \dontshow{
#' n=100
#' set.seed(6714)
#' }
#' @noRd
CiARfit <- function(coef, series, times, zero_mean = TRUE, standardized = TRUE, c = 1) {
    .Call(`_iAR_CiARfit`, coef, series, times, zero_mean, standardized, c)
}

#' Forecast from CiAR model
#'
#' Forecast from models fitted by \code{\link{CiARkalman}}
#'
#' @param coef An array with the parameters of the CiAR model. The elements of the array are, in order, the real and the imaginary part of the phi parameter of the CiAR model.
#' @param series Array with the time series observations.
#' @param times Array with the observational times.
#' @param tAhead The time ahead for which the forecast is required.
#' @param standardized logical; if TRUE, the array series is standardized; if FALSE, series contains the raw time series
#' @param zero_mean logical; if TRUE, the array series has zero mean; if FALSE, series has a mean different from zero.
#'
#' @return A list with the following components:
#' \itemize{
#' \item{forecast}{ Point forecast in the time ahead required.}
#' }
#' @references
#' \insertRef{Elorrieta_2019}{iAR}
#'
#' @seealso
#'
#' \code{\link{CiARsample}}, \code{\link{CiARkalman}}, \code{\link{CiARfit}}
#'
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' #Simulated Data
#' n=100
#' set.seed(6714)
#' st<-gentime(n)
#' x=CiARsample(phiR=0.9,phiI=0,times=st,c=1)
#' y=x$series
#' y1=y/sd(y)
#' n=length(y1)
#' p=trunc(n*0.99)
#' ytr=y1[1:p]
#' yte=y1[(p+1):n]
#' str=st[1:p]
#' ste=st[(p+1):n]
#' tahead=ste-str[p]
#'
#' ciar=CiARkalman(series=ytr,times=str)
#' forCIAR<-CiARforecast(coef=c(ciar$phiR,ciar$phiI),series=ytr,times=str,tAhead=tahead)
#' }
#' @noRd
CiARforecast <- function(coef, series, times, tAhead, zero_mean = TRUE, standardized = TRUE) {
    .Call(`_iAR_CiARforecast`, coef, series, times, tAhead, zero_mean, standardized)
}

#' Minus Log Likelihood of the CiAR Model
#'
#' This function return the negative log likelihood of the CiAR process given specific values of phiR and phiI
#'
#' @param coef An array with the parameters of the CiAR model. The elements of the array are, in order, the real (phiR) and the imaginary (phiI) part of the coefficient of CiAR model.
#' @param series Array with the time series observations.
#' @param times Array with the irregular observational times.
#' @param series_esd Array with the measurements error standard deviations.
#' @param zero_mean logical; if TRUE, the array series has zero mean; if FALSE, series has a mean different from zero.
#' @param standardized logical; if TRUE, the array series is standardized; if FALSE, series contains the raw time series.
#' @param c Nuisance parameter corresponding to the variance of the imaginary part.
#' @param yest The estimate of a missing value in the time series. This function recognizes a missing value with a NA. If the time series does not have a missing value, this value does not affect the computation of the likelihood.
#'
#' @return Value of the negative log likelihood evaluated in the coefficients phiR and phiI.
#' @references
#' \insertRef{Elorrieta_2019}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{CiARsample}}
#'
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' n=300
#' set.seed(6714)
#' st<-gentime(n)
#' x=CiARsample(phiR=0.9,phiI=0,times=st,c=1)
#' y=x$series
#' yerr=rep(0,n)
#' CiARphikalman(coef=c(0.8,0),series=y,times=st,series_esd=yerr,yest=0)
#' }
#' @noRd
CiARphikalman <- function(yest, coef, series, times, series_esd, zero_mean = TRUE, standardized = TRUE, c = 1.0) {
    .Call(`_iAR_CiARphikalman`, yest, coef, series, times, series_esd, zero_mean, standardized, c)
}

#' Simulate from a CiAR Model
#'
#' Simulates a CiAR Time Series Model
#'
#' @param times Array with observational times.
#' @param phiR Real part of the coefficient of CiAR model. A value between -1 and 1.
#' @param phiI Imaginary part of the coefficient of CiAR model. A value between -1 and 1.
#' @param rho Correlation between the real and the imaginary part of the process. A value between -1 and 1.
#' @param c Nuisance parameter corresponding to the variance of the imaginary part.
#'
#' @details The chosen phiR and phiI values must satisfy the condition $|phiR + i phiI| < 1$.
#'
#' @return A list with the following components:
#' \itemize{
#' \item{series}{ Array with the simulated real part of the CiAR process.}
#' }
#' @references
#' \insertRef{Elorrieta_2019}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' n=300
#' set.seed(6714)
#' st<-gentime(n)
#' x=CiARsample(phiR=0.9,phiI=0,times=st,c=1)
#' plot(st,x$series,type='l')
#' x=CiARsample(phiR=-0.9,phiI=0,times=st,c=1)
#' plot(st,x$series,type='l')
#' }
#' @noRd
CiARsample <- function(phiR, phiI, times, rho = 0L, c = 1L) {
    .Call(`_iAR_CiARsample`, phiR, phiI, times, rho, c)
}

#' Simulate from an iAR-Gamma Model
#'
#' Simulates an iAR-Gamma Time Series Model.
#'
#' @param coef A coefficient of iAR-Gamma model. A value between 0 and 1.
#' @param times Array with observational times.
#' @param sigma Scale parameter of the IAR-Gamma process. A positive value.
#' @param mean Mean parameter of the IAR-Gamma process. A positive value.
#'
#' @return  A list with the following components:
#' \itemize{
#' \item{series}{ Array with simulated IAR-Gamma process.}
#' \item{times}{ Array with observation times.}
#' }
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' n=100
#' set.seed(6714)
#' st<-gentime(n)
#' y<-iARgsample(coef=0.9,times=st,sigma=1,mean=1)
#' plot(st,y$series,type='l')
#' hist(y$series,breaks=20)
#' }
#' @noRd
iARgsample <- function(coef, times, sigma = 1.0, mean = 1.0) {
    .Call(`_iAR_iARgsample`, coef, times, sigma, mean)
}

#' Minus Log Likelihood iAR-Gamma Model
#'
#' This function return the negative log likelihood of the iAR-Gamma given specific values of phi, mean and sigma.
#'
#' @param coef An array with the parameters of the iAR-Gamma model. The first element of the array corresponding to the phi parameter, the second to the mean parameter, and the last one to the scale parameter sigma.
#' @param series Array with the time series observations.
#' @param times Array with the irregular observational times.
#' @param yest The estimate of a missing value in the time series. This function recognizes a missing value with a NA. If the time series does not have a missing value, this value does not affect the computation of the likelihood.
#'
#' @return Value of the negative log likelihood evaluated in phi, mean and sigma.
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{iARgsample}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' n=100
#' set.seed(6714)
#' st<-gentime(n)
#' y<-iARgsample(coef=0.9,times=st,sigma=1,mean=1)
#' iARphigamma(coef=c(0.9,1,1),series=y$series,times=st,yest=0)
#' }
#' @noRd
iARphigamma <- function(yest, coef, series, times) {
    .Call(`_iAR_iARphigamma`, yest, coef, series, times)
}

#' Minus Log Likelihood of the iAR Model estimated via Kalman Recursions
#'
#' This function return the negative log likelihood of the iAR process given a specific value of phi.
#'
#' @param coef A given phi coefficient of the iAR model.
#' @param series Array with the time series observations.
#' @param series_esd Array with the measurements error standard deviations.
#' @param times Array with the irregular observational times.
#' @param zero_mean logical; if TRUE, the array series has zero mean; if FALSE, series has a mean different from zero.
#' @param standardized logical; if TRUE, the array series is standardized; if FALSE, series contains the raw time series.
#' @param yest The estimate of a missing value in the time series. This function recognizes a missing value with a NA. If the time series does not have a missing value, this value does not affect the computation of the likelihood.
#'
#' @return Value of the negative log likelihood evaluated in phi.
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{iARsample}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' set.seed(6714)
#' st<-gentime(n=100)
#' y<-iARsample(coef=0.99,times=st)
#' y<-y$series
#' yerr=rep(0,100)
#' iARphikalman(coef=0.8,series=y,series_esd=yerr,times=st,yest=0)
#' }
#' @noRd
iARphikalman <- function(yest, coef, series, series_esd, times, zero_mean = TRUE, standardized = TRUE) {
    .Call(`_iAR_iARphikalman`, yest, coef, series, series_esd, times, zero_mean, standardized)
}

#' Minus Log Likelihood of the iAR Model
#'
#' This function return the negative log likelihood of the iAR Model for a specific value of phi.
#'
#' @param coef A given phi coefficient of the iAR model.
#' @param series Array with the time series observations.
#' @param times Array with the irregular observational times.
#' @param series_esd Array with the measurements error standard deviations.
#' @param zero_mean logical; if TRUE, the array series has zero mean; if FALSE, series has a mean different from zero.
#' @param standardized logical; if TRUE, the array series was standardized; if FALSE, series contains the raw data
#'
#' @return Value of the negative log likelihood evaluated in phi.
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{iARsample}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' set.seed(6714)
#' st<-gentime(n=100)
#' y<-iARsample(coef=0.99,times=st)
#' y<-y$series
#' iARphiloglik(coef=0.8,series=y,times=st,series_esd=0)
#' }
#' @noRd
iARphiloglik <- function(coef, series, times, series_esd, zero_mean = TRUE, standardized = TRUE) {
    .Call(`_iAR_iARphiloglik`, coef, series, times, series_esd, zero_mean, standardized)
}

#' Minus Log Likelihood iAR-T Model
#'
#' This function return the negative log likelihood of the iAR-T given specific values of phi and sigma.
#'
#' @param coef An array with the parameters of the iAR-T model. The first element of the array corresponding to the phi parameter and the second element to the scale parameter sigma
#' @param series Array with the time series observations
#' @param times Array with the irregular observational times
#' @param df degrees of freedom
#' @param yest The estimate of a missing value in the time series. This function recognizes a missing value with a NA. If the time series does not have a missing value, this value does not affect the computation of the likelihood.
#'
#' @return Value of the negative log likelihood evaluated in phi,sigma and df.
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{iARtsample}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' n=300
#' set.seed(6714)
#' st<-gentime(n)
#' y<-iARtsample(coef=0.9,times=st,sigma=1,df=3)
#' iARphit(coef=c(0.9,1),series=y$series,times=st,yest=0)
#' }
#' @noRd
iARphit <- function(yest, coef, series, times, df = 3) {
    .Call(`_iAR_iARphit`, yest, coef, series, times, df)
}

#' Simulate from an iAR Model
#'
#' Simulates an iAR Time Series Model.
#'
#' @param coef A coefficient of iAR model. A value between 0 and 1
#' @param times Array with observational times.
#'
#' @return A list with the following components:
#' \itemize{
#' \item{series}{ Array with simulated iAR data.}
#' }
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' set.seed(6714)
#' st<-gentime(n=100)
#' y<-iARsample(coef=0.99,times=st)
#' y<-y$series
#' plot(st,y,type='l')
#' }
#' @noRd
iARsample <- function(coef, times) {
    .Call(`_iAR_iARsample`, coef, times)
}

#' Forecast for iAR, CiAR, and BiAR Classes
#'
#' Generates forecasts for the specified time series model. This method is implemented for:
#' 1. Irregular Autoregressive models (`iAR`)
#' 2. Complex Irregular Autoregressive models (`CiAR`)
#' 3. Bivariate Autoregressive models (`BiAR`)
#'
#' @name forecast
#'
#' @param x An object of class \code{iAR}, \code{CiAR}, or \code{BiAR}, containing the model specification and parameters:
#'   \itemize{
#'     \item For \code{iAR}:
#'       \itemize{
#'         \item \code{family}: The distribution family of the iAR model (one of "norm", "t", or "gamma").
#'         \item \code{series}: A numeric vector representing the time series to forecast.
#'         \item \code{coef}: The coefficient(s) of the iAR model.
#'         \item \code{zero_mean}: Logical, whether the model assumes a zero-mean series.
#'         \item \code{standardized}: Logical, whether the model uses standardized data (only for "norm" family).
#'         \item \code{mean}: The mean parameter (only for "gamma" family).
#'         \item \code{tAhead}: Integer, the number of steps ahead to forecast.
#'       }
#'     \item For \code{CiAR}:
#'       \itemize{
#'         \item \code{coef}: The real and imaginary parts of the CiAR model's coefficients.
#'         \item \code{series}: A numeric vector representing the time series to forecast.
#'         \item \code{times}: A numeric vector specifying the time points of the series.
#'         \item \code{zero_mean}: Logical, whether the model assumes a zero-mean series.
#'         \item \code{standardized}: Logical, whether the model output should be standardized.
#'         \item \code{tAhead}: Integer, the number of steps ahead to forecast.
#'       }
#'     \item For \code{BiAR}:
#'       \itemize{
#'         \item \code{coef}: The coefficients of the BiAR model (real and imaginary parts).
#'         \item \code{series}: A numeric matrix with two columns representing the bivariate time series to forecast.
#'         \item \code{times}: A numeric vector specifying the time points of the series.
#'         \item \code{tAhead}: Integer, the number of steps ahead to forecast.
#'       }
#'   }
#'
#' @param ... Additional arguments.
#'
#' @return An updated object of class \code{iAR}, \code{CiAR}, or \code{BiAR}, where the \code{forecast} property contains the forecasted values.
#'
#' @details
#' This method generates forecasts for the specified time series model. Depending on the class of the input object:
#' \itemize{
#'   \item For \code{iAR}, the function supports three distribution families:
#'     \item "norm" for normal distribution.
#'     \item "t" for t-distribution.
#'     \item "gamma" for gamma distribution.
#'   \item For \code{CiAR}, the function uses complex autoregressive processes.
#'   \item For \code{BiAR}, the function generates forecasts for a bivariate autoregressive process.
#' }
#' All required parameters (e.g., coefficients, time points) must be set before calling this method.
#'
#' @references
#' \insertRef{Eyheramendy_2018}{iAR},\insertRef{Elorrieta_2019}{iAR},\insertRef{Elorrieta_2021}{iAR}
#'
#' @examples
#' # Example 1: Forecasting with a normal iAR model
#' library(iAR)
#' n=100
#' set.seed(6714)
#' times=gentime(n=n)@times
#' model_norm <- iAR(family = "norm", times = times, coef = 0.9)
#' model_norm <- sim(model_norm)
#' model_norm <- kalman(model_norm)
#' tAhead=1.3
#' model_norm <- forecast(model_norm,tAhead=tAhead)
#' plot(times, model_norm@series, type = "l", main = "Original Series with Forecast")
#' points(max(times)+ tAhead, model_norm@forecast, col = "blue", pch = 16)
#' plot_forecast(model_norm)
#'
#' # Example 2: Forecasting with a CiAR model
#' set.seed(6714)
#' model_CiAR <- CiAR(times = times,coef = c(0.9, 0))
#' model_CiAR <- sim(model_CiAR)
#' y=model_CiAR@series
#' y1=y/sd(y)
#' model_CiAR@series=y1
#' model_CiAR@series_esd=rep(0,n)
#' model_CiAR <- kalman(model_CiAR)
#' print(model_CiAR@coef)
#' model_CiAR@tAhead=1.3
#' model_CiAR <-forecast(model_CiAR)
#' model_CiAR@forecast
#'
#' # Example 3: Forecasting with a BiAR model
#' n=80
#' set.seed(6714)
#' times=gentime(n=n)@times
#' model_BiAR <- BiAR(times = times,coef = c(0.9, 0.3), rho = 0.9)
#' model_BiAR <- sim(model_BiAR)
#' y=model_BiAR@series
#' y1=y/apply(y,2,sd)
#' model_BiAR@series=y1
#' model_BiAR@series_esd=matrix(0,n,2)
#' model_BiAR <- kalman(model_BiAR)
#' print(model_BiAR@coef)
#' model_BiAR@tAhead=1.3
#' model_BiAR <-forecast(model_BiAR)
#' model_BiAR@forecast
#'
#' @export
forecast <- S7::new_generic("forecast", "x")
S7::method(forecast, iAR) <- function(x,tAhead=NULL) {
  if(is.null(tAhead))
    tAhead <- x@tAhead
  if (!(length(tAhead) == 1 && is.null(dim(tAhead)) && tAhead > 0))
    stop("'tAhead' must be a strictly positive numeric scalar")
  if(length(x@series)==0) stop("The forecast method needs a time series")
  if(x@family == "norm"){
    if(length(x@coef)==0) stop("The forecast method needs the coefficient of the iAR model")
    res <- iARforecast(coef = x@coef,
                       series = x@series,
                       zero_mean = x@zero_mean,
                       standardized = x@standardized,
                       tAhead = tAhead)
    x@forecast <- res
    x@tAhead <- tAhead
    return(x)
  }

  if(x@family == "t"){
    if(length(x@coef)==0) stop("The forecast method needs the coefficient of the iAR-T model")
    res <- iARforecast(coef = x@coef,
                       series = x@series,
                       zero_mean = x@zero_mean,
                       standardized = FALSE,
                       tAhead = tAhead)
    x@forecast <- res
    x@tAhead <- tAhead
    x@standardized <- FALSE
    return(x)
  }

  if(x@family == "gamma"){
    if(length(x@coef)==0) stop("The forecast method needs the coefficients of the iAR-Gamma model")
    res <- iARgforecast(coef = x@coef,
                        series = x@series,
                        mean = x@mean,
                        tAhead = tAhead)
    x@forecast <- res
    x@tAhead <- tAhead
    return(x)
  }
}
S7::method(forecast, CiAR) <- function(x,tAhead=NULL) {
  if(is.null(tAhead))
    tAhead <- x@tAhead
  if (!(length(tAhead) == 1 && is.null(dim(tAhead)) && tAhead > 0))
    stop("'tAhead' must be a strictly positive numeric scalar")
  if(length(x@series)==0) stop("The forecast method needs a time series")
  if(length(x@coef)==0) stop("The forecast method needs the coefficients of the CiAR model")
  res <- CiARforecast(coef = x@coef,
                      series = x@series,
                      times = x@times,
                      zero_mean = x@zero_mean,
                      standardized = x@standardized,
                      tAhead = tAhead)$forecast
  x@forecast <- res
  x@tAhead <- tAhead
  return(x)
}
S7::method(forecast, BiAR) <- function(x,tAhead=NULL) {
  if(is.null(tAhead))
    tAhead <- x@tAhead
  if (!(length(tAhead) == 1 && is.null(dim(tAhead)) && tAhead > 0))
    stop("'tAhead' must be a strictly positive numeric scalar")
  if(length(x@series)==0) stop("The forecast method needs a bivariate time series")
  if(length(x@coef)==0) stop("The forecast method needs the coefficients of the BiAR model")
  res <- t(BiARforecast(coef = x@coef,
                        series1 = x@series[,1],
                        series2 = x@series[,2],
                        times = x@times,
                        # zero_mean = x@zero_mean,
                        # standardized = x@standardized,
                        tAhead = tAhead)$forecast)
  x@forecast <- res
  x@tAhead <- tAhead
  return(x)
}

#' `CiAR` Class
#'
#' Represents a complex irregular autoregressive (CiAR) time series model. 
#' This class extends the `unidata` class and provides additional properties 
#' for modeling, forecasting, and interpolating irregularly observed time 
#' series data with both negative and positive autocorrelation.
#'
#' @param times A numeric vector representing the time points.
#' @param series A complex vector representing the values of the time series.
#' @param series_esd A numeric vector representing the error standard deviations of the time series.
#' @param series_names An optional character vector of length 1 representing the name of the series. 
#' @param fitted_values A numeric vector containing the fitted values from the model.
#' @param kalmanlik A numeric value representing the Kalman likelihood of the model.
#' @param coef A numeric vector of length 2 containing the real (phiR) and imaginary (phiI)
#'   parts of the autoregressive coefficient. Each value must be in [-1, 1] and the modulus
#'   `sqrt(phiR^2 + phiI^2)` must be strictly less than 1 (stationarity condition).
#'   Defaults to `c(0.9, 0)`. Unlike [iAR], which uses a scalar, both `CiAR` and [BiAR]
#'   use a length-2 vector.
#' @param tAhead A numeric value specifying the forecast horizon (default: 1).
#' @param forecast A numeric vector containing the forecasted values.
#' @param interpolated_values A numeric vector containing the interpolated values.
#' @param interpolated_times A numeric vector containing the times of the interpolated data points.
#' @param interpolated_series A numeric vector containing the interpolated series.
#' @param zero_mean A logical value indicating if the model assumes a zero-mean process (default: TRUE).
#' @param standardized A logical value indicating if the model assumes a standardized process (default: TRUE).
#' @param hessian A logical value indicating whether the Hessian matrix is computed during estimation (default: FALSE).
#' @param summary A list containing the summary of the model fit, including diagnostics and statistical results.
#'
#' @section Validation:
#' - Inherits all validation rules from the `unidata` class:
#'  - `@times`, `@series`, and `@series_esd` must be numeric vectors.
#'  - `@times` must not contain `NA` values and must be strictly increasing.
#'  - The length of `@series` must match the length of `@times`.
#'  - The length of `@series_esd` must be 0, 1, or equal to the length of `@series`.
#'  - `NA` values in `@series` must correspond exactly (positionally) to `NA` values in `@series_esd`.
#'  - `@series_names`, if provided, must be a character vector of length 1.
#' 
#' - `@coef` must be a numeric vector of length 2 with no dimensions.
#' - Each value in `@coef` must be in the interval [-1, 1].
#' - The modulus `sqrt(phiR^2 + phiI^2)` must be strictly less than 1 (stationarity condition).
#' - `@tAhead` must be a strictly positive numeric scalar.
#' 
#' @details
#' The `CiAR` class is designed to handle irregularly observed time series 
#' data with either negative or positive autocorrelation using an autoregressive 
#' approach. It extends the `unidata` class to include functionalities 
#' specific to the `CiAR` model.
#'
#' Key features of the `CiAR` class include:
#' - Support for irregularly observed time series data with negative 
#' or positive autocorrelation.
#' - Forecasting and interpolation functionalities for irregular time points.
#' - Configurable assumptions of zero-mean and standardized processes.
#'
#' @references
#' \insertRef{Elorrieta_2019}{iAR}
#'
#'
#' @examples
#' times=gentime(n=200, distribution = "expmixture",
#'              lambda1 = 130, lambda2 = 6.5, p1 = 0.15, p2 = 0.85)@times
#' my_CiAR <- CiAR(times = times,coef = c(0.9, 0))
#'
#' # Access properties
#' my_CiAR@coef
#'
#' @export
CiAR <- S7::new_class(
  "CiAR",
  parent = unidata,
  package = "iAR",
  properties = list(fitted_values = S7::class_numeric,
                    kalmanlik = S7::class_numeric,
                    coef = S7::new_property(
                      S7::class_numeric,
                      default = c(0.9, 0), # restringir cada parametro a -1,1
                      validator = function(value) {
                        if (!(is.numeric(value) &&
                              length(value) == 2 &&
                              is.null(dim(value)))) {
                          return("'coef' must be a numeric vector of length 2")
                        }
                        if (any(value < -1 | value > 1)) {
                          return("Each value in 'coef' must be between -1 and 1")
                        }
                        if (sqrt(sum(value^2)) >= 1) {
                          return(paste0(
                            "The modulus sqrt(phiR^2 + phiI^2) must be strictly less than 1 ",
                            "(got ", round(sqrt(sum(value^2)), 4), "). ",
                            "See the stationarity condition for CiAR models."
                          ))
                        }
                        NULL
                      }
                    ),
                    tAhead = S7::new_property(
                      S7::class_numeric,
                      default = 1,
                      validator = function(value) {
                        if (!(length(value) == 1 &&
                              is.null(dim(value)) &&
                              value > 0)) {
                          return("'tAhead' must be a strictly positive numeric scalar")
                        }
                        NULL
                      }
                    ),
                    forecast = S7::class_numeric,
                    interpolated_values = S7::class_numeric,
                    interpolated_times = S7::class_numeric,
                    interpolated_series = S7::class_numeric,
                    zero_mean = S7::new_property(class_logical, default = TRUE),
                    standardized = S7::new_property(class_logical, default = TRUE),
                    hessian = S7::new_property(class_logical, default = FALSE),
                    summary = S7::class_list))

#' Interpolation from iAR-Gamma model
#'
#' Interpolation of missing values from models fitted by \code{\link{iARgamma}}
#'
#' @param coef A given array with the parameters of the iAR-Gamma model. The first element of the array corresponding to the phi parameter, the second to the mean parameter, and the last one to the scale parameter sigma.
#' @param series Array with the time series observations.
#' @param times Array with the irregular observational times.
#' @param yini a single value, initial value for the estimation of the missing value of the time series.
#'
#'
#' @return A list with the following components:
#' \itemize{
#' \item{fitted}{ Estimation of a missing value of the iAR-Gamma process.}
#' \item{ll}{ Value of the negative log likelihood evaluated in the fitted missing values.}
#' }
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{iARgsample}}, \code{\link{iARgamma}}
#'
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' set.seed(6714)
#' n<-100
#' st<-gentime(n)
#' y<-iARgsample(coef=0.9,times=st,sigma=1,mean=1)
#' model<-iARgamma(series=y$series, times=st)
#' y<-y$series
#' napos=10
#' y0=y
#' y[napos]=NA
#' xest=c(model$coef,model$mean,model$sigma)
#' yest=iARginterpolation(coef=xest,series=y,times=st)
#' yest$fitted
#' mse=(y0[napos]-yest$fitted)^2
#' print(mse)
#' plot(st,y,type='l',xlim=c(st[napos-5],st[napos+5]))
#' points(st,y,pch=20)
#' points(st[napos],yest$fitted,col="red",pch=20)
#' }
#' @noRd
iARginterpolation<-function(coef,series,times,yini=1)
{
  aux<-1e10
  value<-1e10
  br<-0
  countNA = length(which(is.na(series)))
  if(countNA==0){
    stop("This function requires a NA value at the point to be interpolated.")}
  if(countNA > 1){
    stop("This function requires only one NA value.")}
  if (yini==1)
    yini = rgamma(1,1,1)
  out = optim(yini, iARphigamma, lower = 0.0001, upper = Inf,
              coef=coef,series = series, times = times, method="L-BFGS-B")
  par = out$par
  aux = out$value
  return(list(fitted = par, ll = aux))
}

#' Transit of an extrasolar planet
#'
#' Time series corresponding to the residuals of the parametric model fitted by Jordan et al (2013) for a transit of an extrasolar planet.
#'
#' @format A data frame with 91 observations on the following 2 variables:
#' \describe{
#'   \item{t}{Time from mid-transit (hours).}
#'   \item{r}{Residuals of the parametric model fitted by Jordan et al (2013).}
#' }
#' @references
#' \insertRef{Jordan_etal13}{iAR}
#' @examples
#' data(Planets)
#' #plot(Planets[,1],Planets[,2],xlab='Time from mid-transit (hours)',ylab='Noise',pch=20)
"Planets"

#' Classical Cepheid
#'
#' Time series of a classical cepheid variable star obtained from HIPPARCOS.
#'
#' @format A data frame with 109 observations on the following 3 variables:
#' \describe{
#'   \item{t}{heliocentric Julian Day}
#'   \item{m}{magnitude}
#'   \item{merr}{measurement error of the magnitude (in mag).}
#' }
#' @details The frequency computed by GLS for this light curve is 0.060033386.
#' Catalogs and designations of this star:
#' HD 1989: HD 305996
#' TYCHO-2 2000:TYC 8958-2333-1
#' USNO-A2.0:USNO-A2 0225-10347916
#' HIP: HIP-54101
#' @examples
#' data(clcep)
#' f1=0.060033386
#' o1=phase(data=clcep,f1=f1,twop=TRUE)
#' plot(o1@times_phased,o1@series_phased,pch=20)
"clcep"

#' Simulate from a BiAR Model
#'
#' Simulates a BiAR Time Series Model
#'
#' @param times Array with observational times.
#' @param phiR Autocorrelation coefficient of BiAR model. A value between -1 and 1.
#' @param phiI Crosscorrelation coefficient of BiAR model. A value between -1 and 1.
#' @param series_esd1 Array with the measurements error standard deviations of the first time series of the bivariate process.
#' @param series_esd2 Array with the measurements error standard deviations of the second time series of the bivariate process.
#' @param rho Contemporary correlation coefficient of BiAR model. A value between -1 and 1.
#'
#' @details The chosen phiR and phiI values must satisfy the condition $|phiR + i phiI| < 1$.
#'
#' @return A list with the following components:
#' \itemize{
#' \item{series}{ Matrix with the simulated BiAR process.}
#' }
#'
#'
#' @references
#' \insertRef{Elorrieta_2021}{iAR}
#' @seealso
#' \code{\link{gentime}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' n=300
#' set.seed(6714)
#' st<-gentime(n)
#' x=BiARsample(phiR=0.9,phiI=0.3,times=st)
#' plot(st,x$series[1,],type='l')
#' plot(st,x$series[2,],type='l')
#' x=BiARsample(phiR=-0.9,phiI=-0.3,times=st)
#' plot(st,x$series[1,],type='l')
#' plot(st,x$series[2,],type='l')
#' }
#' @noRd
BiARsample<-function (times, phiR, phiI, series_esd1=0,series_esd2=0,rho = 0)
{
  n <- length(times)
  delta <- diff(times)
  x = matrix(0, nrow = 2, ncol = n)
  F = matrix(0, nrow = 2, ncol = 2)
  phi = complex(1, real = phiR, imaginary = phiI)
  if (Mod(phi) >= 1)
    stop("Mod of Phi must be less than one")
  Phi = Mod(phi)
  psi <- acos(phiR/Phi)
  if (phiI < 0)
    psi = -acos(phiR/Phi)
  #psi2 <- asin(phiI/Phi)
  e.R = rnorm(n)
  e.I = rnorm(n)
  state.error = rbind(e.R, e.I)
  Sigma = matrix(1, nrow = 2, ncol = 2)
  Sigma[1, 1] = 1
  Sigma[2, 2] = 1
  Sigma[1, 2] = rho * sqrt(Sigma[1, 1]) * sqrt(Sigma[2, 2])
  Sigma[2, 1] = Sigma[1, 2]
  B = svd(Sigma)
  A = matrix(0, nrow = 2, ncol = 2)
  diag(A) = sqrt(B$d)
  Sigma.root = (B$u) %*% A %*% B$u
  state.error = Sigma.root %*% state.error
  #measurement error
  w.R = rnorm(n)
  w.I = rnorm(n)
  observation.error = rbind(w.R, w.I)
  SigmaO = matrix(0, nrow = 2, ncol = 2)
  SigmaO[1, 1] = series_esd1**2
  SigmaO[2, 2] = series_esd2**2
  BO = svd(SigmaO)
  AO = matrix(0, nrow = 2, ncol = 2)
  diag(AO) = sqrt(BO$d)
  SigmaO.root = (BO$u) %*% AO %*% BO$u
  observation.error = SigmaO.root %*% observation.error
  G = diag(2)
  y = observation.error
  x[, 1] = state.error[, 1]
  for (i in 1:(n - 1)) {
    phi2.R <- (Phi^delta[i]) * cos(delta[i] * psi)
    phi2.I <- (Phi^delta[i]) * sin(delta[i] * psi)
    phi2 <- 1 - Mod(phi^delta[i])^2
    F[1, 1] = phi2.R
    F[1, 2] = -phi2.I
    F[2, 1] = phi2.I
    F[2, 2] = phi2.R
    x[, i + 1] = F %*% x[, i] + sqrt(phi2) * state.error[,i]
    y[, i ] = G %*% x[, i] + observation.error[,i]
  }
  y[, n ] = G %*% x[, n] + observation.error[,n]
  return(list(series = y))
}

#' Interpolation from BiAR model
#'
#' Interpolation of missing values from models fitted by \code{\link{BiARkalman}}
#'
#' @param coef An array with the parameters of the BiAR model. The elements of the array are, in order, the real (phiR) and the imaginary (phiI) part of the coefficient of BiAR model.
#' @param series1 Array with the observations of the first time series of the BiAR process.
#' @param series2 Array with the observations of the second time series of the BiAR process.
#' @param times Array with the irregular observational times.
#' @param series_esd1 Array with the measurements error standard deviations of the first time series of the BiAR process.
#' @param series_esd2 Array with the measurements error standard deviations of the second time series of the BiAR process.
#' @param yini1 a single value, initial value of the estimation of the missing value of the first time series of the BiAR process.
#' @param yini2 a single value, initial value of the estimation of the missing value of the second time series of the BiAR process.
#' @param zero_mean logical; if TRUE, the array y has zero mean; if FALSE, y has a mean different from zero.
#' @param niter Number of iterations in which the function nlminb will be repeated.
#' @param seed a single value, interpreted as the seed of the random process.
#' @param nmiss a single value; If 1, only one time series of the BiAR process has a missing value. If 2, both time series of the BiAR process have a missing value.
#'
#' @return A list with the following components:
#' \itemize{
#' \item{fitted}{ Estimation of the missing values of the BiAR process.}
#' \item{ll}{ Value of the negative log likelihood evaluated in the fitted missing values.}
#' }
#' @references
#' \insertRef{Elorrieta_2021}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{BiARsample}}, \code{\link{BiARphikalman}}
#' @keywords internal
#' @examples
#' \dontshow{
#' set.seed(6713)
#' n=100
#' st<-gentime(n)
#' x=BiARsample(phiR=0.9,phiI=0.3,times=st,rho=0.9)
#' y=x$series
#' y1=y/apply(y,1,sd)
#' yerr1=rep(0,n)
#' yerr2=rep(0,n)
#' biar=BiARkalman(series1=y1[1,],series2=y1[2,],times=st,series_esd1 = yerr1,series_esd2=yerr2)
#' biar
#' napos=10
#' y0=y1
#' y1[1,napos]=NA
#' xest=c(biar$phiR,biar$phiI)
#' yest=BiARinterpolation(coef=xest,series1=y1[1,],series2=y1[2,],times=st,series_esd1=yerr1,
#' series_esd2=yerr2,nmiss=1)
#' yest$fitted
#' mse=(y0[1,napos]-yest$fitted)^2
#' print(mse)
#' par(mfrow=c(2,1))
#' plot(st,x$series[1,],type='l',xlim=c(st[napos-5],st[napos+5]))
#' points(st,x$series[1,],pch=20)
#' points(st[napos],yest$fitted*apply(y,1,sd)[1],col="red",pch=20)
#' plot(st,x$series[2,],type='l',xlim=c(st[napos-5],st[napos+5]))
#' points(st,x$series[2,],pch=20)
#' }
#' @noRd
BiARinterpolation <- function (coef,series1, series2, times, series_esd1 = 0, series_esd2 = 0, yini1=0,yini2=0,zero_mean = TRUE, niter = 10, seed = 1234,nmiss=1) {
  set.seed(seed)
  aux <- 1e+10
  value <- 1e+10
  br <- 0

  if (sum(series_esd1) == 0) {
    series_esd1 = rep(0, length(series1))
  }

  if (sum(series_esd2) == 0) {
    series_esd2 = rep(0, length(series2))
  }

  if (yini1==0)
    yini1 = rnorm(1)

  if (yini2==0)
    yini2 = rnorm(1)

  if (nmiss==2){
    for (i in 1:niter) {
      optim <- nlminb(start = c(yini1, yini2), objective = BiARphikalman,
                      coef=coef,series1 = series1,series2 = series2, times = times, series_esd1 = series_esd1,
                      series_esd2=series_esd2, zero_mean = zero_mean,
                      lower = c(-Inf, -Inf), upper = c(Inf, Inf))

      value <- optim$objective

      if (aux > value) {
        par <- optim$par
        aux <- value
        br <- br + 1
      }

      if (aux <= value & br > 1 & i > trunc(niter/2))
        break
    }
  }

  if (aux == 1e+10)
    par <- c(0, 0)

  if (nmiss==1){
    out = optim(c(yini1), BiARphikalman, lower = -Inf, upper = Inf,
                coef=coef, series1=series1, series2=series2, times=times, series_esd1 = series_esd1,
                series_esd2=series_esd2, zero_mean = zero_mean, method="BFGS")
    par = out$par
    aux = out$value
  }

  return(list(fitted = par, ll = aux))
}

#' Generating Irregularly spaced times
#'
#' A method for generating time points based on a statistical distribution.
#' The results are stored in the `times` slot of the `utilities` object.
#'
#' @param x An object of class `utilities`, or `NULL` (the default). When `x` is
#'   `NULL` or omitted, the `utilities` object is created internally, so
#'   `gentime(n = 200)` is equivalent to the three-step pattern
#'   `o <- utilities(); o <- gentime(o, n = 200); o`.
#' @param ... Additional arguments for generating time points: 
#'   \describe{
#'      \item{n}{A positive integer. Length of observation times.}
#'      \item{distribution}{A character string specifying the distribution of the observation times. Default is `"expmixture"`.
#'   Available options are:
#'   - `"expmixture"`: A mixture of two exponential distributions.
#'   - `"uniform"`: A uniform distribution.
#'   - `"exponential"`: A single exponential distribution.
#'   - `"gamma"`: A gamma distribution.}
#'      \item{lambda1}{Mean (1/rate) of the exponential distribution or the first exponential distribution in a mixture of exponential distributions. Default is `130`.}
#'      \item{lambda2}{Mean (1/rate) of the second exponential distribution in a mixture of exponential distributions. Default is `6.5`.}
#'      \item{p1}{Weight of the first exponential distribution in a mixture of exponential distributions. Default is `0.15`.}
#'      \item{p2}{Weight of the second exponential distribution in a mixture of exponential distributions. Default is `0.85`.}
#'      \item{a}{Shape parameter of a gamma distribution or lower limit of the uniform distribution. Default is `0`.}
#'      \item{b}{Scale parameter of a gamma distribution or upper limit of the uniform distribution. Default is `1`.}
#'   }
#'
#' @return An updated `utilities` object with the generated observation times stored in the `times` slot.
#'
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @examples
#' set.seed(12917)
#' st=gentime(n=200, distribution = "expmixture", lambda1 = 130,
#' lambda2 = 6.5, p1 = 0.15, p2 = 0.85)@times
#' mean(diff(st))
#'
#' st=gentime(n=200, distribution = "expmixture", lambda1 = 15,
#' lambda2 = 2.5, p1 = 0.15, p2 = 0.85)@times
#' mean(diff(st))
#'
#' @export
gentime <- S7::new_generic("gentime", "x", function(x = NULL, ...) S7::S7_dispatch())
S7::method(gentime, NULL) <- function(x, n, distribution = "expmixture", lambda1 = 130,
                                      lambda2 = 6.5, p1 = 0.15, p2 = 0.85, a = 0, b = 1) {
  gentime(utilities(), n = n, distribution = distribution, lambda1 = lambda1,
          lambda2 = lambda2, p1 = p1, p2 = p2, a = a, b = b)
}

S7::method(gentime, utilities) <- function(x, n, distribution = "expmixture", lambda1 = 130, 
                                           lambda2 = 6.5, p1 = 0.15, p2 = 0.85, a = 0, b = 1) {
  if (!is.numeric(n) || length(n) != 1 || n <= 0) {
    stop("'n' must be a positive integer.")
  }
  valid_distributions <- c("expmixture", "uniform", "exponential", "gamma")
  if (!distribution %in% valid_distributions) {
    stop(paste0(
      "'distribution' must be one of: ",
      paste(paste0('"', valid_distributions, '"'), collapse = ", "),
      ". Got: \"", distribution, "\""
    ))
  }
  if(distribution=="expmixture")
  {
    dT <- rep(0, n)
    a <- sample(c(lambda1, lambda2), size = n, prob = c(p1, p2),
                replace = TRUE)
    dT[which(a == lambda1)] = rexp(length(which(a == lambda1)),
                                   rate = 1/lambda1)
    dT[which(a == lambda2)] = rexp(length(which(a == lambda2)),
                                   rate = 1/lambda2)
    sT = cumsum(dT)
  }
  if(distribution=="uniform")
  {
    sT=cumsum(runif(n,a,b))
  }
  if(distribution=="exponential")
  {
    sT=cumsum(rexp(n,rate=1/lambda1))
  }
  if(distribution=="gamma")
  {
    sT=cumsum(rgamma(n,shape=a,scale=b))
  }
  x@times <- sT
  return(x)
}  

#' `iAR` Class
#'
#' Represents a univariate irregular autoregressive (iAR) time series model.
#' This class extends the "unidata" class and includes additional properties
#' for modeling, forecasting, and interpolation.
#'
#' @param times A numeric vector representing the time points.
#' @param series A numeric vector representing the values of the time series.
#' @param series_esd A numeric vector representing the error standard deviations of the time series.
#' @param series_names An optional character vector of length 1 representing the name of the series.
#' @param family A character string indicating the distribution family of the model. Must be one of `"norm"`, `"t"`, or `"gamma"`. Defaults to `"norm"`.
#' @param fitted_values A numeric vector containing the fitted values from the model.
#' @param loglik A numeric value representing the log-likelihood of the model.
#' @param kalmanlik A numeric value representing the Kalman likelihood of the model.
#' @param coef A single numeric value (scalar) representing the autoregressive coefficient,
#'   strictly between 0 and 1 (default: 0.9). Note that `iAR` only supports positive
#'   autocorrelation. Unlike [CiAR] and [BiAR], which use a numeric vector of length 2,
#'   this argument must be a scalar; passing a vector will raise a validation error.
#' @param df A numeric value representing the degrees of freedom (`t` distribution) (default: 3).
#' @param sigma A numeric value representing the scale parameter (`t` distribution) (default: 1).
#' @param mean A numeric value representing the estimated mean of the model (`gamma` parameter).
#' @param variance A numeric value representing the estimated variance of the model (`gamma` parameter).
#' @param forecast A numeric vector containing the forecasted values.
#' @param tAhead A numeric value specifying the forecast horizon (default: 1).
#' @param interpolated_values A numeric vector containing the interpolated values.
#' @param interpolated_times A numeric vector containing the times of the interpolated data points.
#' @param interpolated_series A numeric vector containing the interpolated series.
#' @param zero_mean A logical value indicating if the model assumes a zero-mean process (default: TRUE).
#' @param standardized A logical value indicating if the model assumes a standardized process (default: TRUE).
#' @param hessian A logical value indicating whether the Hessian matrix is computed during estimation (default: FALSE).
#' @param summary A list containing the summary of the model fit, including diagnostics and statistical results.
#'
#' @section Validation Rules:
#' - Inherits all validation rules from the `unidata` class:
#'  - `@times`, `@series`, and `@series_esd` must be numeric vectors.
#'  - `@times` must not contain `NA` values and must be strictly increasing.
#'  - The length of `@series` must match the length of `@times`.
#'  - The length of `@series_esd` must be 0, 1, or equal to the length of `@series`.
#'  - `NA` values in `@series` must correspond exactly (positionally) to `NA` values in `@series_esd`.
#'  - `@series_names`, if provided, must be a character vector of length 1.
#'
#' - `@family` must be one of `"norm"`, `"t"`, or `"gamma"`.
#' - `@coef` must be a numeric scalar strictly between 0 and 1.
#' - `@df` must be a positive integer scalar **only if** `family = "t"`; otherwise, it should not be specified.
#' - `@sigma` must be a strictly positive numeric scalar (used in `"t"` family).
#' - `@mean` and `@variance` must be strictly positive numeric scalars **only if** `family = "gamma"`.
#' - `@tAhead` must be a strictly positive numeric scalar.


#'
#' @details
#' The `iAR` class is designed to handle irregularly observed time series data using an
#' autoregressive approach. It extends the "unidata" class to include additional
#' modeling and diagnostic capabilities. Key functionalities include forecasting,
#' interpolation, and model fitting.
#'
#' The class also supports advanced modeling features, such as:
#' - Different distribution families for the data (e.g., Gaussian, `t`-distribution).
#' - Optional computation of the Hessian matrix for parameter estimation.
#' - Standardized or zero-mean process assumptions.
#'
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @examples
#' # Create an `iAR` object
#' times=gentime(n=200, distribution = "expmixture",
#'              lambda1 = 130, lambda2 = 6.5, p1 = 0.15, p2 = 0.85)@times
#' my_iAR <- iAR(family = "norm", times = times, coef = 0.9,hessian=TRUE)
#'
#' my_iAR@family
#' my_iAR@coef
#'
#' @export
iAR <- S7::new_class(
  "iAR",
  parent = unidata,
  package = "iAR",
  properties = list(family = S7::new_property(S7::class_character,
                                              validator = function(value) {
                                                if (!value %in% c("norm", "t", "gamma")) {
                                                  return("family must be one of 'norm', 't', or 'gamma'")
                                                }
                                                NULL
                                              },
                                              default = "norm"),
                    fitted_values = S7::class_numeric,
                    loglik = S7::class_numeric,
                    kalmanlik = S7::class_numeric,
                    coef = S7::new_property(
                      S7::class_numeric,
                      default = 0.9,
                      validator = function(value) {
                        if (!(length(value) == 1 && is.null(dim(value)))) {
                          return(paste0(
                            "'coef' for iAR must be a single numeric scalar, not a vector. ",
                            "For a length-2 coefficient use CiAR or BiAR instead."
                          ))
                        }
                        if (value <= 0) {
                          return(paste0(
                            "'coef' must be strictly between 0 and 1. ",
                            "iAR only supports positive autocorrelation; ",
                            "for negative autocorrelation use CiAR instead."
                          ))
                        }
                        if (value >= 1) {
                          return("'coef' must be strictly between 0 and 1 (stationarity requires phi < 1).")
                        }
                        NULL
                      }
                    ),
                    df = S7::new_property(
                      S7::class_numeric,
                      default = 3,
                      validator = function(value) {
                        if (!(length(value) == 1 &&
                              is.null(dim(value)) &&
                              value > 0 &&
                              value %% 1 == 0)) {
                          return("'df' must be a positive integer value")
                        }
                        NULL
                      }
                    )
                    , # t #colocar df por defecto = 3
                    sigma = S7::new_property(
                      S7::class_numeric,
                      default = 1,
                      validator = function(value) {
                        if (!(length(value) == 1 &&
                              is.null(dim(value)) &&
                              value > 0)) {
                          return("'sigma' must be a positive numeric value")
                        }
                        NULL
                      }
                    ), # t
                    mean = S7::new_property(
                      S7::class_numeric,
                      default = 0), # gamma
                    variance = S7::new_property(
                      S7::class_numeric,
                      default = 1,
                      validator = function(value) {
                        if (!(length(value) == 1 &&
                              is.null(dim(value)) &&
                              value > 0)) {
                          return("'variance' must be a strictly positive numeric scalar")
                        }
                        NULL
                      }
                    ),
                    tAhead = S7::new_property(
                      S7::class_numeric,
                      default = 1,
                      validator = function(value) {
                        if (!(length(value) == 1 &&
                              is.null(dim(value)) &&
                              value > 0)) {
                          return("'tAhead' must be a strictly positive numeric scalar")
                        }
                        NULL
                      }
                    ),
                    forecast = S7::class_numeric,
                    interpolated_values = S7::class_numeric,
                    interpolated_times = S7::class_numeric,
                    interpolated_series = S7::class_numeric,
                    zero_mean = S7::new_property(class_logical, default = TRUE), # no gamma
                    standardized = S7::new_property(class_logical, default = TRUE), # no gamma # no t
                    hessian = S7::new_property(class_logical, default = FALSE),
                    summary = S7::class_list),
  validator = function(self) {
    fam <- self@family
    if (fam == "t") {
      if (is.na(self@df) || self@df <= 0 || self@df %% 1 != 0) {
        return("For family = 't', df must be a positive integer scalar")
      }
    }
    if (fam == "gamma") {
      if (is.na(self@mean) || is.na(self@variance)) {
        return("For family = 'gamma', both mean and variance must be specified")
      }
    }
    NULL
  }
)

#' Unidata Class
#'
#' The `unidata` class is an S7 class designed to represent univariate irregularly observed time series models
#' with associated times, values, and optional error standard deviations.
#'
#' @param times A numeric vector representing the time points.
#' @param series A numeric vector representing the values of the time series.
#' @param series_esd A numeric vector representing the error standard deviations of the time series.
#' @param series_names An optional character vector of length 1 representing the name of the series. 
#'
#' @section Validation Rules:
#' - `@times`, `@series`, and `@series_esd` must be numeric vectors.
#' - `@times` must not contain `NA` values and must be strictly increasing.
#' - The length of `@series` must match the length of `@times`.
#' - The length of `@series_esd` must be 0, 1, or equal to the length of `@series`.
#' - `NA` values in `@series` must correspond exactly (positionally) to `NA` values in `@series_esd`.
#' - `@series_names`, if provided, must be a character vector of length 1.
#'
#' @examples
#' # Create a unidata object
#' unidata_instance <- unidata(
#'   times = c(1, 2, 3, 4),
#'   series = c(10, 20, 15, 25),
#'   series_esd = c(1, 1.5, 1.2, 1.8),
#'   series_names = "my_series")
#'
#' @seealso [iAR], [CiAR]
#' @export
unidata <- S7::new_class(
    "unidata",
    package = "iAR",
    properties = list(
      times = S7::class_numeric,
      series = S7::class_numeric,
      series_esd = S7::class_numeric,
      series_names = S7::new_property(S7::class_character, default = character(0)),
      series_esd_names = S7::new_property(
        class = S7::class_character,
        getter = function(self) {
          if (length(self@series_names) == 1) {
            return(paste0(self@series_names, "_esd"))
          } else {
            return(character(0))
          }
        }
      )
    ),
    validator = function(self) {
      if (!(is.atomic(self@times) && is.null(dim(self@times))) ||
          !is.vector(self@series) || !is.vector(self@series_esd)) {
        return("@times, @series, and @series_esd must be numeric vectors without dimensions")
      }
      if (anyNA(self@times)) {
        return("NA values are not allowed in @times")
      }
      if (length(self@series_esd) == length(self@series) && anyNA(self@series_esd)) {
        mismatch <- is.na(self@series) != is.na(self@series_esd)
        if (any(mismatch)) {
          return("NA values in @series must correspond to NA values in @series_esd")
        }
      }
      if (length(self@series) > 0 && length(self@times) != length(self@series)) {
        return("The length of @times must match the length of @series")
      }
      if (!(length(self@series_esd) %in% c(0, 1, length(self@series)))) {
        return("The length of @series_esd must be 0, 1, or match @series")
      }
      if (is.unsorted(self@times, strictly = TRUE)) {
        return("@times must be strictly increasing")
      }
      # Validar series_names
      if (length(self@series_names) > 0) {
        if (length(self@series_names) != 1) return("Length of @series_names must be 1")
        # if (anyDuplicated(self@series_names)) return("@series_names must be unique")
      }
      NULL
    }
)

#' Maximum Likelihood Estimation of the iAR Model via Kalman Recursions
#'
#' Maximum Likelihood Estimation of the iAR model parameter phi. The estimation procedure uses the Kalman Filter to find the maximum of the likelihood.
#'
#' @param series Array with the time series observations.
#' @param times Array with the irregular observational times.
#' @param series_esd Array with the measurements error standard deviations.
#' @param zero_mean logical; if TRUE, the array series has zero mean; if FALSE, series has a mean different from zero.
#' @param standardized logical; if TRUE, the array series is standardized; if FALSE, series contains the raw time series.
#' @param hessian logical. Indicates whether to compute the Hessian matrix.; if TRUE, the Hessian matrix will be calculated; if FALSE, the Hessian will not be computed.
#'
#' @return A list with the following components:
#' \itemize{
#' \item{coef}{ MLE of the phi parameter of the iAR model.}
#' \item{ll}{ Value of the negative log likelihood evaluated in phi.}
#' \item{summary}{ If hessian=T, returns the standard error and p-value of the estimated parameter phi.}
#' }
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#'
#' \code{\link{gentime}}, \code{\link{iARsample}}, \code{\link{arima}},\code{\link{iARphikalman}}
#'
#' @keywords internal
#' @examples
#' \dontshow{
#' set.seed(6714)
#' st<-gentime(n=100)
#' y<-iARsample(coef=0.99,times=st)
#' y<-y$series
#' phi=iARkalman(series=y,times=st)$coef
#' print(phi)
#' }
#' @noRd
iARkalman<-function (series, times, series_esd=0,zero_mean=TRUE,standardized=TRUE,hessian=FALSE)
{
  if(sum(series_esd)==0){
    series_esd=rep(0,length(series))}
  summary=NULL
  if(hessian==F)
  {
    out = optimize(iARphikalman, interval = c(0, 1), series = series, times = times, series_esd=series_esd,zero_mean = zero_mean,standardized=standardized,yest=0)
    phi = out$minimum
    ll = out$objective
    summary=list()
  }
  if(hessian==T)
  {
    out=optim(par=c(x=0),fn=iARphikalman, series=series, times=times, series_esd=series_esd, zero_mean = zero_mean,yest=0, standardized = standardized,method="L-BFGS-B",lower=0,upper=0.9999999,hessian=hessian)
    phi=as.numeric(out$par)
    conv <- out$convergence
    ll=out$value
    hess=as.numeric(out$hessian)
    stderr=sqrt(diag(solve(hess)))
    tvalue=phi/stderr
    pvalue=2*pnorm(abs(tvalue),lower.tail=F)
    summary=list(hess=hess,stderr=stderr,tvalue=tvalue,pvalue=pvalue,convergence=conv)
  }
  return(list(coef=phi,kalman=ll,summary=summary))
}

#' Summary of iAR package models
#'
#' This method provides a summary for objects created by the iAR package,
#' including `iAR`, `CiAR`, `BiAR`, and `utilities` objects. For fitted model
#' objects, the summary reports model-specific information such as the model
#' family, coefficients, standard errors, t-values, and p-values.
#' The output is formatted to display the relevant statistics based on the model class.
#'
#' @name summary
#'
#' @param object An object of class `iAR`, `CiAR`, `BiAR`, or `utilities`.
#'   For fitted model objects, this contains model-specific parameters such as
#'   coefficients (`coef`), standard errors (`stderr`), t-values (`tvalue`),
#'   p-values (`pvalue`), and other class-specific components.
#' @param ... Additional arguments (unused).
#'
#'
#' @export
S7::method(summary, iAR) <- function(object,diagnostic=TRUE,...) {
  x   <- object
  if(length(x@loglik) == 0 && length(x@kalmanlik) == 0) {
    stop("The model has not been fitted. Use loglik() or kalman() before summary().",call. = FALSE)
  }
  n   <- length(x@series)
  k   <- 1
  has_hess <- length(x@summary) > 0 && !is.null(x@summary$stderr)
  if (length(x@loglik)    > 0) nll <- x@loglik
  if (length(x@kalmanlik) > 0) nll <- 0.5 * n * (log(2*pi) + x@kalmanlik)
  AIC <- 2*nll + 2*k
  BIC <- 2*nll + log(n)*k
  cat("iAR model\n\n")
  if (x@family == "norm") {
    cat("Family:\n ", x@family, "\n\n")
    cat("Coefficients:\n")
    if (has_hess) {
      cat(paste("    ", "Estimate St. Error t value Pr(>|t|)\n"))
      cat("phi", "     ", format(round(x@coef,2),nsmall=2), "      ",
          .fmt_se(x@summary$stderr), "   ",
          .fmt_se(x@summary$tvalue), "     ",
          .fmt_se(x@summary$pvalue), "\n", sep = "")
    } else {
      cat("    Estimate\nphi     ", format(round(x@coef,2),nsmall=2), "\n", sep = "")
      cat("(Set hessian=TRUE before loglik()/kalman() to obtain standard errors)\n")
    }
  }
  if (x@family == "t") {
    k <- 2; AIC <- 2*nll + 2*k; BIC <- 2*nll + log(n)*k
    cat("Family:\n ", x@family, "\n\nd.f.:\n ", x@df, "\n\nCoefficients:\n")
    if (has_hess) {
      cat(paste("      ", "Estimate St. Error t value Pr(>|t|)\n"))
      cat("phi", "       ", format(round(x@coef,2),nsmall=2), "      ",
          .fmt_se(x@summary$stderr[1]), "   ", .fmt_se(x@summary$tvalue[1]), "     ",
          .fmt_se(x@summary$pvalue[1]), "\n", sep = "")
      cat("sigma", "     ", format(round(x@sigma,2),nsmall=2), "      ",
          .fmt_se(x@summary$stderr[2]), "    ", .fmt_se(x@summary$tvalue[2]), "     ",
          .fmt_se(x@summary$pvalue[2]), "\n", sep = "")
    } else {
      cat("    Estimate\nphi     ", format(round(x@coef,2),nsmall=2),
          "\nsigma   ", format(round(x@sigma,2),nsmall=2), "\n", sep = "")
      cat("(Set hessian=TRUE before loglik() to obtain standard errors)\n")
    }
  }
  if (x@family == "gamma") {
    k <- 3; AIC <- 2*nll + 2*k; BIC <- 2*nll + log(n)*k
    cat("Family:\n ", x@family, "\n\nCoefficients:\n")
    if (has_hess) {
      cat(paste("         ", "Estimate St. Error t value Pr(>|t|)\n"))
      cat("phi", "          ", format(round(x@coef,2),nsmall=2), "      ",
          .fmt_se(x@summary$stderr[1]), "   ", .fmt_se(x@summary$tvalue[1]), "     ",
          .fmt_se(x@summary$pvalue[1]), "\n", sep = "")
      cat("mean", "         ", format(round(x@mean,2),nsmall=2), "      ",
          .fmt_se(x@summary$stderr[2]), "   ", .fmt_se(x@summary$tvalue[2]), "     ",
          .fmt_se(x@summary$pvalue[2]), "\n", sep = "")
      cat("variance", "     ", format(round(x@variance,2),nsmall=2), "      ",
          .fmt_se(x@summary$stderr[3]), "    ", .fmt_se(x@summary$tvalue[3]), "     ",
          .fmt_se(x@summary$pvalue[3]), "\n", sep = "")
    } else {
      cat("    Estimate\nphi      ", format(round(x@coef,2),nsmall=2),
          "\nmean     ", format(round(x@mean,2),nsmall=2),
          "\nvariance ", format(round(x@variance,2),nsmall=2), "\n", sep = "")
      cat("(Set hessian=TRUE before loglik() to obtain standard errors)\n")
    }
  }
  cat("\nInformation criteria:\n")
  cat("AIC:", format(round(AIC,2), nsmall=2), "\n")
  cat("BIC:", format(round(BIC,2), nsmall=2), "\n")
  if(diagnostic)
  {
    x <- fit(x)
    residuals <- x@series - x@fitted_values
    cat("\n", sep = "")
    cat("Residual diagnostics:", "\n", sep = "")
    cat("ACF lag 1:",
      format(round(acf(residuals, plot = FALSE)$acf[2], 2), nsmall = 2),
      "\n", sep = " ")
    cat("Ljung-Box test p-value (lag=1):",
      format(round(Box.test(residuals, lag = 1, type = "Ljung-Box")$p.value, 2), nsmall = 2),
      "\n", sep = " ")
    cat("Ljung-Box test p-value (lag=10):",
      format(round(Box.test(residuals, lag = 10, type = "Ljung-Box")$p.value, 2), nsmall = 2),
      "\n", sep = " ")
  }
  if (has_hess) {
    if (!isTRUE(tryCatch({ chol(x@summary$hess); TRUE }, error = function(e) FALSE)))
      warning("Hessian matrix is not positive definite or not invertible.", call. = FALSE)
    if (any(!is.finite(x@summary$stderr)))
      warning("Some standard errors are non-finite.", call. = FALSE)
    if (!is.null(x@summary$convergence) && x@summary$convergence != 0)
      warning(paste0("Optimization may not have converged (code = ", x@summary$convergence, ")."), call. = FALSE)
  }
}

S7::method(summary, CiAR) <- function(object,diagnostic=TRUE,...) {
  x <- object
  if(length(x@kalmanlik) == 0) {
    stop("The model has not been fitted. Use kalman() before summary().",call. = FALSE)}
  n <- length(x@series)
  k <- 2
  has_hess <- length(x@summary) > 0 && !is.null(x@summary$stderr)
  nll <- 0.5 * n * (log(2*pi) + x@kalmanlik)
  AIC <- 2*nll + 2*k
  BIC <- 2*nll + log(n)*k
  cat("CiAR model\n\nCoefficients:\n")
  if (has_hess) {
    cat(paste("    ", "Estimate St. Error t value Pr(>|t|)\n"))
    cat("phiR", "     ", format(round(x@coef[1],2),nsmall=2), "      ",
        .fmt_se(x@summary$stderr[1]), "   ", .fmt_se(x@summary$tvalue[1]), "     ",
        .fmt_se(x@summary$pvalue[1]), "\n", sep = "")
    cat("phiI", "     ", format(round(x@coef[2],2),nsmall=2), "      ",
        .fmt_se(x@summary$stderr[2]), "    ", .fmt_se(x@summary$tvalue[2]), "     ",
        .fmt_se(x@summary$pvalue[2]), "\n", sep = "")
  } else {
    cat("     Estimate\nphiR     ", format(round(x@coef[1],2),nsmall=2),
        "\nphiI     ", format(round(x@coef[2],2),nsmall=2), "\n", sep = "")
    cat("(Set hessian=TRUE before kalman() to obtain standard errors)\n")
  }
  cat("\nInformation criteria:\n")
  cat("AIC:", format(round(AIC,2), nsmall=2), "\n")
  cat("BIC:", format(round(BIC,2), nsmall=2), "\n")
  cat("\n", sep = "")
  if(diagnostic)
  {
    x <- fit(x)
    residuals <- x@series - x@fitted_values
    cat("Residual diagnostics:", "\n", sep = "")
    cat("ACF lag 1:",
      format(round(acf(residuals, plot = FALSE)$acf[2], 2), nsmall = 2),
      "\n", sep = " ")
    cat("Ljung-Box test p-value (lag=1):",
      format(round(Box.test(residuals, lag = 1, type = "Ljung-Box")$p.value, 2), nsmall = 2),
      "\n", sep = " ")
    cat("Ljung-Box test p-value (lag=10):",
      format(round(Box.test(residuals, lag = 10, type = "Ljung-Box")$p.value, 2), nsmall = 2),
      "\n", sep = " ")
  }
  if (has_hess) {
    if (!isTRUE(tryCatch({ chol(x@summary$hess); TRUE }, error = function(e) FALSE)))
      warning("Hessian matrix is not positive definite or not invertible.", call. = FALSE)
    if (any(!is.finite(x@summary$stderr)))
      warning("Some standard errors are non-finite.", call. = FALSE)
    if (!is.null(x@summary$convergence) && x@summary$convergence != 0)
      warning(paste0("Optimization may not have converged (code = ", x@summary$convergence, ")."), call. = FALSE)
  }
}

S7::method(summary, BiAR) <- function(object,diagnostic = TRUE,...) {
  x <- object
  if(length(x@kalmanlik) == 0) {
    stop("The model has not been fitted. Use kalman() before summary().",call. = FALSE)}
  n <- dim(x@series)[1]
  k <- 2
  has_hess <- length(x@summary) > 0 && !is.null(x@summary$stderr)
  nll <- 0.5 * n * (log(2*pi) + x@kalmanlik)
  AIC <- 2*nll + 2*k
  BIC <- 2*nll + log(n)*k
  cat("BiAR model\n\nCoefficients:\n")
  if (has_hess) {
    cat(paste("    ", "Estimate St. Error t value Pr(>|t|)\n"))
    cat("phiR", "     ", format(round(x@coef[1],2),nsmall=2), "      ",
        .fmt_se(x@summary$stderr[1]), "   ", .fmt_se(x@summary$tvalue[1]), "     ",
        .fmt_se(x@summary$pvalue[1]), "\n", sep = "")
    cat("phiI", "     ", format(round(x@coef[2],2),nsmall=2), "      ",
        .fmt_se(x@summary$stderr[2]), "    ", .fmt_se(x@summary$tvalue[2]), "     ",
        .fmt_se(x@summary$pvalue[2]), "\n", sep = "")
  } else {
    cat("     Estimate\nphiR     ", format(round(x@coef[1],2),nsmall=2),
        "\nphiI     ", format(round(x@coef[2],2),nsmall=2), "\n", sep = "")
    cat("(Set hessian=TRUE before kalman() to obtain standard errors)\n")
  }
  cat("\nInformation criteria:\n")
  cat("AIC:", format(round(AIC,2), nsmall=2), "\n")
  cat("BIC:", format(round(BIC,2), nsmall=2), "\n\n")
  cat("\n", sep = "")
  if(diagnostic)
  {
    x <- fit(x)
    residuals <- x@series - x@fitted_values
    cat("Residual diagnostics:", "\n", sep = "")
    cat("ACF lag 1 residual 1:",
      format(round(acf(residuals[,1], plot = FALSE)$acf[2], 2), nsmall = 2),
      "\n", sep = " ")
    cat("ACF lag 1 residual 2:",
      format(round(acf(residuals[,2], plot = FALSE)$acf[2], 2), nsmall = 2),
      "\n", sep = " ")
    ccf_res <- ccf(residuals[,1], residuals[,2], plot = FALSE, lag.max = 1)
    cat("CCF between residual 1(t) and residual 2(t-1):",
      format(round(ccf_res$acf[1], 2), nsmall = 2), "\n", sep = " ")
    cat("CCF between residual 1(t-1) and residual 2(t):",
      format(round(ccf_res$acf[3], 2), nsmall = 2), "\n", sep = " ")
    cat("Ljung-Box test p-value (lag=1) residual 1:",
      format(round(Box.test(residuals[,1], lag = 1, type = "Ljung-Box")$p.value, 2), nsmall = 2),
      "\n", sep = " ")
    cat("Ljung-Box test p-value (lag=1) residual 2:",
      format(round(Box.test(residuals[,2], lag = 1, type = "Ljung-Box")$p.value, 2), nsmall = 2),
      "\n", sep = " ")
    cat("Ljung-Box test p-value (lag=10) residual 1:",
      format(round(Box.test(residuals[,1], lag = 10, type = "Ljung-Box")$p.value, 2), nsmall = 2),
      "\n", sep = " ")
    cat("Ljung-Box test p-value (lag=10) residual 2:",
      format(round(Box.test(residuals[,2], lag = 10, type = "Ljung-Box")$p.value, 2), nsmall = 2),
      "\n", sep = " ")
  }
  if (has_hess) {
    if (!isTRUE(tryCatch({ chol(x@summary$hess); TRUE }, error = function(e) FALSE)))
      warning("Hessian matrix is not positive definite or not invertible.", call. = FALSE)
    if (any(!is.finite(x@summary$stderr)))
      warning("Some standard errors are non-finite.", call. = FALSE)
    if (!is.null(x@summary$convergence) && x@summary$convergence != 0)
      warning(paste0("Optimization may not have converged (code = ", x@summary$convergence, ")."), call. = FALSE)
  }
}

S7::method(summary, utilities) <- function(object,...) {
  print(object@summary)}

#' Fitted Values of iAR model
#'
#' Fit an iAR model to an irregularly observed time series.
#'
#' @param coef Estimated phi parameter by the iAR model.
#' @param series Array with the time series observations.
#' @param times Array with the irregular observational times.
#' @param standardized logical; if TRUE, the array series is standardized; if FALSE, series contains the raw time series
#' @param zero_mean logical; if TRUE, the array series has zero mean; if FALSE, series has a mean different from zero.
#'
#' @return Fitted values of the iAR model
#' @references
#' \insertRef{Eyheramendy_2018}{iAR}
#'
#' @seealso
#' \code{\link{gentime}}, \code{\link{iARsample}}, \code{\link{iARloglik}}, \code{\link{iARkalman}}
#' @keywords internal
#' @examples
#' \dontshow{
#' set.seed(6714)
#' st<-gentime(n=100)
#' y<-iARsample(coef=0.99,times=st)
#' y<-y$series
#' phi=iARloglik(series=y,times=st)$coef
#' fit=iARfit(coef=phi,series=y,times=st)
#' }
#' @noRd
iARfit<-function(coef,series,times,standardized=TRUE,zero_mean=TRUE)
{
  sigma = 1
  mu = 0
  if (standardized == FALSE)
    sigma = var(series)
  if (zero_mean == TRUE)
    mu = mean(series)
  if (zero_mean == TRUE)
    series = series-mu
  if (standardized == FALSE)
    series=series/sqrt(sigma)
  delta=diff(times)
  y1=series[-c(length(series))]
  fit=c(0,(coef**delta)*y1)
  fit=(fit*sqrt(sigma)+mu)
  return(fitted=fit)
}

library(iAR)

# ── iAR validators ────────────────────────────────────────────────────────────

test_that("iAR rejects non-scalar coef", {
  expect_error(iAR(times = 1:5, coef = c(0.9, 0)), "scalar")
})

test_that("iAR rejects coef <= 0 and suggests CiAR", {
  expect_error(iAR(times = 1:5, coef = -0.5), "CiAR")
})

test_that("iAR rejects coef >= 1", {
  expect_error(iAR(times = 1:5, coef = 1.0), "stationarity")
})

# ── CiAR / BiAR validators ────────────────────────────────────────────────────

test_that("CiAR rejects scalar coef", {
  expect_error(CiAR(times = 1:5, coef = 0.9), "length 2")
})

test_that("CiAR rejects coef with Mod >= 1", {
  expect_error(CiAR(times = 1:5, coef = c(0.8, 0.7)), "modulus")
})

test_that("BiAR rejects rho outside [-1, 1]", {
  expect_error(BiAR(times = 1:5, rho = 1.5), "\\[-1, 1\\]")
})

test_that("BiAR rejects coef with Mod >= 1", {
  expect_error(BiAR(times = 1:5, coef = c(0.9, 0.5)), "modulus")
})

# ── gentime ───────────────────────────────────────────────────────────────────

test_that("gentime rejects unknown distribution", {
  expect_error(gentime(NULL, n = 10, distribution = "poisson"), "distribution")
})

test_that("gentime NULL dispatch returns utilities object", {
  set.seed(1)
  o <- gentime(NULL, n = 10, distribution = "uniform")
  expect_s3_class(o, "iAR::utilities")
  expect_length(o@times, 10)
  expect_true(all(diff(o@times) > 0))
})

# ── kalman / loglik edge cases ─────────────────────────────────────────────────

test_that("kalman stops on series shorter than 3", {
  m <- iAR(family = "norm", times = 1:2, series = c(1, -1),
           series_esd = c(0, 0))
  expect_error(kalman(m), "at least 3")
})

test_that("kalman stops on constant series", {
  m <- iAR(family = "norm", times = 1:5, series = rep(1, 5),
           series_esd = rep(0, 5))
  expect_error(kalman(m), "constant")
})

# ── iAR-Gamma sim ─────────────────────────────────────────────────────────────

test_that("gamma sim stops when mean <= 0", {
  set.seed(1)
  o <- gentime(NULL, n = 20)
  m <- iAR(family = "gamma", times = o@times, coef = 0.8)
  m@mean     <- 0
  m@variance <- 1
  expect_error(sim(m), "strictly positive")
})

test_that("gamma sim works with non-integer mean and variance", {
  set.seed(42)
  o <- gentime(NULL, n = 50)
  m <- iAR(family = "gamma", times = o@times, coef = 0.8)
  m@mean     <- 1.5
  m@variance <- 0.5
  m <- sim(m)
  expect_length(m@series, 50)
  expect_true(all(m@series > 0))
})

# ── multidata dimension check ─────────────────────────────────────────────────

test_that("multidata rejects mismatched series_esd dimensions", {
  expect_error(
    multidata(times = 1:3,
              series     = matrix(1:6, nrow = 3, ncol = 2),
              series_esd = matrix(0,   nrow = 2, ncol = 3)),
    "Dimensions"
  )
})

---
title: "Getting Started with iAR"
author: "Felipe Elorrieta, Cesar Ojeda, Susana Eyheramendy, Wilfredo Palma"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Getting Started with iAR}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  fig.width = 7,
  fig.height = 4
)
```

## Overview

The **iAR** package provides autoregressive models for time series observed at
*irregularly spaced* time points, a common situation in astronomy, finance, and
environmental science. It implements three complementary model classes:

| Class | Coefficient | Autocorrelation | Reference |
|-------|-------------|-----------------|-----------|
| `iAR` | Scalar $\phi \in (0,1)$ | Positive only | Eyheramendy et al. (2018) |
| `CiAR` | Vector $(\phi_R, \phi_I)$, $\|\phi\| < 1$ | Positive *and* negative | Elorrieta et al. (2019) |
| `BiAR` | Vector $(\phi_R, \phi_I)$, $\|\phi\| < 1$ | Bivariate, cross-correlated | Elorrieta et al. (2021) |

All three share the same workflow: **construct** → **simulate** → **estimate**
→ **forecast / interpolate**.

```{r load-pkg}
library(iAR)
```

---

## Generating Irregular Observation Times

Real astronomical or environmental data is rarely sampled on a regular grid.
The `gentime()` function generates realistic irregular time grids from several
distributions. The first argument `x` defaults to `NULL`, so the `utilities`
object is created internally and you can call `gentime()` directly without any
setup step.

```{r gentime}
set.seed(2847)

# x defaults to NULL — no setup needed
times <- gentime(n = 100)@times

cat("Number of observations:", length(times), "\n")
cat("Time span: [", round(min(times), 1), ",", round(max(times), 1), "]\n")
cat("Mean gap between observations:", round(mean(diff(times)), 2), "\n")
```

The default distribution (`"expmixture"`) mixes two exponential distributions,
mimicking data sets where most observations are closely spaced but occasional
long gaps occur. Alternative distributions include `"uniform"`, `"exponential"`,
and `"gamma"`.

```{r gentime-plot, fig.cap = "Distribution of inter-observation gaps (log scale) for 100 irregular times drawn from an exponential mixture."}
gaps <- diff(times)
hist(log1p(gaps),
     main = "Inter-observation gaps (log1p scale)",
     xlab = "log(1 + gap)", col = "steelblue", border = "white",
     breaks = 20)
```

---

## iAR: Univariate Irregular Autoregressive Model

### Simulation

Construct an `iAR` object with the desired coefficient, then call `sim()`.

```{r iar-sim}
set.seed(2847)
times <- gentime(n = 100)@times

# Build the model: family = "norm", phi = 0.9
m_iar <- iAR(family = "norm", times = times, coef = 0.9)
m_iar <- sim(m_iar)

head(m_iar@series)
```

```{r iar-plot, fig.cap = "Simulated iAR series (phi = 0.9). The irregular spacing is visible in the unequal distances along the x-axis."}
plot(m_iar, type = "o",main = "Simulated iAR series (phi = 0.9)",ylab="Values",xlab="Time",pch=20)
```

### Parameter Estimation

Two estimators are available:

* **`loglik()`** — direct MLE; supports all three families (`"norm"`, `"t"`,
  `"gamma"`).
* **`kalman()`** — Kalman-filter MLE; available for all three model classes
  (`iAR`, `CiAR`, `BiAR`).

Before estimation, standardise the series (zero mean, unit variance) and supply
a vector of error standard deviations. For simulated data with no measurement
noise, zeros suffice.

```{r iar-prep}
y       <- m_iar@series
y_std   <- y / sd(y)           # unit variance
m_iar@series     <- y_std
m_iar@series_esd <- rep(0, length(times))
```

#### Using `loglik()`

```{r iar-loglik}
m_loglik <- loglik(m_iar)
cat("loglik estimate: phi =", round(m_loglik@coef, 4), "\n")
cat("Log-likelihood: ", round(m_loglik@loglik, 4), "\n")
```

#### Using `kalman()`

```{r iar-kalman}
m_kalman <- kalman(m_iar)
cat("kalman estimate: phi =", round(m_kalman@coef, 4), "\n")
cat("Kalman log-lik: ", round(m_kalman@kalmanlik, 4), "\n")
```

Both estimators should recover a value close to the true $\phi = 0.9$.

#### Standard Errors via the Hessian

Set `hessian = TRUE` to obtain standard errors and p-values.

```{r iar-hessian}
m_h <- iAR(family = "norm", times = times, coef = 0.9, hessian = TRUE)
m_h@series     <- y_std
m_h@series_esd <- rep(0, length(times))
m_h <- loglik(m_h)

# Summary contains the coefficient, SE, p-value, information criteria, and residual diagnostics
summary(m_h)
```

### Forecasting

After estimation, use `forecast()` to predict `tAhead` time units into the
future.

```{r iar-forecast}
m_fc <- forecast(m_kalman, tAhead = 10)
cat("Forecast (10 time units ahead):", round(m_fc@forecast, 4), "\n")
```

```{r iar-plot-forecast, fig.cap = "iAR series with one-step-ahead forecast (blue dot)."}
plot_forecast(m_fc,ylab="Values",xlab="Time",type="o",pch=20)
```

### Interpolation (Imputing Missing Values)

`interpolation()` imputes `NA` values in the series using the fitted model.

```{r iar-interp}
# Introduce a missing value at position 10
m_interp           <- m_kalman
m_interp@series[10] <- NA

m_interp <- interpolation(m_interp)
cat("Interpolated value at position 10:",
    round(m_interp@interpolated_values, 4), "\n")
cat("Original value at position 10    :",
    round(y_std[10], 4), "\n")
```

---

## CiAR: Complex Irregular Autoregressive Model

`CiAR` extends `iAR` to allow **negative autocorrelation** via a complex
coefficient $\phi = \phi_R + i\,\phi_I$. The stationarity condition is
$|\phi| = \sqrt{\phi_R^2 + \phi_I^2} < 1$.

### Simulation and Estimation

```{r ciar-sim}
set.seed(2847)
times <- gentime(n = 100)@times

# phi = (0.9, 0): purely real CiAR — same as iAR but more general
m_ciar <- CiAR(times = times, coef = c(0.9, 0))
m_ciar <- sim(m_ciar)

# Standardise
y_c           <- m_ciar@series
y_c_std       <- y_c / sd(y_c)
m_ciar@series     <- y_c_std
m_ciar@series_esd <- rep(0, length(times))
```

```{r ciar-kalman}
m_ciar <- kalman(m_ciar)
cat("CiAR estimate: phiR =", round(m_ciar@coef[1], 4),
    " phiI =", round(m_ciar@coef[2], 4), "\n")
cat("Modulus |phi| =", round(sqrt(sum(m_ciar@coef^2)), 4), "\n")
```

### Forecasting

```{r ciar-forecast}
m_ciar <- forecast(m_ciar, tAhead = 10)
cat("CiAR forecast:", round(m_ciar@forecast, 4), "\n")
```

---

## BiAR: Bivariate Irregular Autoregressive Model

`BiAR` models two time series observed at the **same** irregular time points
with a shared autoregressive structure and cross-correlation $\rho$.

### Simulation

```{r biar-sim}
set.seed(2847)
times <- gentime(n = 100)@times

# coef = c(phiR, phiI), rho = cross-series correlation
m_biar <- BiAR(times = times, coef = c(0.9, 0.3), rho = 0.9)
m_biar <- sim(m_biar)

head(m_biar@series)   # matrix: column 1 = series 1, column 2 = series 2
```

### Estimation

```{r biar-kalman}
n       <- length(times)
y_b     <- m_biar@series
y_b_std <- y_b / matrix(apply(y_b, 2, sd), nrow = n, ncol = 2, byrow = TRUE)

m_biar@series     <- y_b_std
m_biar@series_esd <- matrix(0, n, 2)

m_biar <- kalman(m_biar)
cat("BiAR estimate: phiR =", round(m_biar@coef[1], 4),
    " phiI =", round(m_biar@coef[2], 4), "\n")
```

### Forecasting

```{r biar-forecast}
m_biar <- forecast(m_biar, tAhead = 10)
cat("BiAR forecast (series 1):", round(m_biar@forecast[1], 4), "\n")
cat("BiAR forecast (series 2):", round(m_biar@forecast[2], 4), "\n")
```

---

## Real Data Example: AGN Light Curve

The package includes the `agn` dataset — 237 K-band flux measurements of the
active galactic nucleus MCG-6-30-15, taken between 2006 and 2011.

```{r agn-load}
data(agn)
head(agn)
```

```{r agn-plot, fig.cap = "AGN MCG-6-30-15 light curve. Gaps in the observation schedule produce the irregular spacing typical of ground-based astronomical surveys."}
plot(agn$t, agn$m, xlab = "Heliocentric JD - 2450000",
     ylab = "Flux [10^-15 erg/s/cm^2/A]",
     main = "AGN MCG-6-30-15 (K band)",type="o",pch=20)
```

### Fitting an iAR Model

```{r agn-fit}
# Standardise: centre and scale by SD (measurement errors available)
y_agn   <- agn$m
y_agn_c <- (y_agn - mean(y_agn)) / sd(y_agn)
esd_agn <- agn$merr / sd(y_agn)

m_agn <- iAR(
  family     = "norm",
  times      = agn$t,
  series     = y_agn_c,
  series_esd = esd_agn
)

m_agn <- kalman(m_agn)
cat("Estimated phi:", round(m_agn@coef, 4), "\n")
cat("Kalman log-lik:", round(m_agn@kalmanlik, 4), "\n")
```

### Fitted Values and Forecast

```{r agn-fit-plot, fig.cap = "AGN light curve (standardised) with fitted iAR values overlaid."}
m_agn <- fit(m_agn)
plot_fit(m_agn, xlab = "Heliocentric JD - 2450000",
     ylab = "Flux [10^-15 erg/s/cm^2/A]",type="o",pch=20)
```

```{r agn-forecast}
m_agn <- forecast(m_agn, tAhead = 1)
cat("One-step-ahead forecast:", round(m_agn@forecast, 4), "\n")
```

---

## Utility: Pairing Two Irregular Time Series

When two series are observed at different (but overlapping) time points,
`pairingits()` matches them within a tolerance window before fitting a `BiAR`
model.

```{r pair}
data(cvnovag)
data(cvnovar)

# Pair the G-band and R-band CV Nova light curves
paired <- pairingits(data1 = cvnovag, data2 = cvnovar, tol = 0.01)

cat("Paired observations:", nrow(paired@paired), "\n")
head(paired@paired)
```

---

## Summary

The table below collects the key functions for each step of the modelling
workflow.

| Step | iAR | CiAR | BiAR |
|------|-----|------|------|
| Construct | `iAR(family, times, coef)` | `CiAR(times, coef)` | `BiAR(times, series, coef, rho)` |
| Simulate | `sim(m)` | `sim(m)` | `sim(m)` |
| Estimate (direct MLE) | `loglik(m)` | — | — |
| Estimate (Kalman) | `kalman(m)` | `kalman(m)` | `kalman(m)` |
| Forecast | `forecast(m, tAhead)` | `forecast(m, tAhead)` | `forecast(m, tAhead)` |
| Interpolate | `interpolation(m)` | `interpolation(m)` | `interpolation(m)` |
| Plot fitted | `plot_fit(m)` | `plot_fit(m)` | `plot_fit(m)` |
| Plot forecast | `plot_forecast(m)` | `plot_forecast(m)` | `plot_forecast(m)` |
| Generate times | `gentime(n = ...)` | — | — |
| Pair two series | `pairingits(data1, data2)` | — | — |

## References

Eyheramendy, S., Elorrieta, F., & Palma, W. (2018). An irregular discrete time
series model to identify residuals with autocorrelation in astronomical light
curves. *Monthly Notices of the Royal Astronomical Society*, **481**(4),
4990–5003. <doi:10.1093/mnras/sty2487>

Elorrieta, F., Eyheramendy, S., & Palma, W. (2019). Discrete semi-parametric
model for the investigation of variable stars. *Astronomy & Astrophysics*,
**627**, A120. <doi:10.1051/0004-6361/201935560>

Elorrieta, F., Eyheramendy, S., Palma, W., & Ojeda, C. (2021). A bivariate
irregular autoregressive model. *Monthly Notices of the Royal Astronomical
Society*, **505**(1), 1105–1116. <doi:10.1093/mnras/stab1216>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dmcep.R
\docType{data}
\name{dmcep}
\alias{dmcep}
\title{Double Mode Cepheid.}
\format{
A data frame with 191 observations on the following 3 variables:
\describe{
  \item{t}{heliocentric Julian Day}
  \item{m}{magnitude}
  \item{merr}{measurement error of the magnitude (in mag).}
}
}
\usage{
dmcep
}
\description{
Time series of a double mode cepheid variable star obtained from OGLE.
}
\details{
The dominant frequency computed by GLS for this light curve is 0.7410152.
The second frequency computed by GLS for this light curve is 0.5433353.
OGLE-ID:175210
}
\examples{
data(dmcep)
f1=0.7410152
o1=phase(data=dmcep,f1=f1,twop=TRUE)
plot(o1@times_phased,o1@series_phased,pch=20)
#fit=harmonicfit(dmcep,f1)
#f2=0.5433353
#foldlc(cbind(dmcep$t,fit$res,dmcep$merr),f2)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Planets.R
\docType{data}
\name{Planets}
\alias{Planets}
\title{Transit of an extrasolar planet}
\format{
A data frame with 91 observations on the following 2 variables:
\describe{
  \item{t}{Time from mid-transit (hours).}
  \item{r}{Residuals of the parametric model fitted by Jordan et al (2013).}
}
}
\usage{
Planets
}
\description{
Time series corresponding to the residuals of the parametric model fitted by Jordan et al (2013) for a transit of an extrasolar planet.
}
\examples{
data(Planets)
#plot(Planets[,1],Planets[,2],xlab='Time from mid-transit (hours)',ylab='Noise',pch=20)
}
\references{
\insertRef{Jordan_etal13}{iAR}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/08_plot.R
\name{plot}
\alias{plot}
\title{Plot Method for unidata Objects}
\usage{
plot(x, y, ...)
}
\arguments{
\item{x}{An object of class `unidata` or `multidata`. This object must contain the time 
series in the `series` slot and the associated time points in the 
`times` slot.}

\item{y}{Ignored. Present for compatibility with the base \code{plot} generic.}

\item{...}{Additional arguments passed to the `plot.zoo` function for 
customizing the plot (e.g., titles, colors, etc.).}
}
\description{
This method allows visualizing `unidata` and `multidata` objects using the `plot.zoo` 
function from the `zoo` package. It converts the `unidata` (`multidata`) object into 
a `zoo` object and then applies the plotting function to the time series 
contained in the object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/04_kalman.R
\name{kalman}
\alias{kalman}
\title{Maximum Likelihood Estimation of Parameters for iAR, CiAR, and BiAR Models using the Kalman Filter}
\usage{
kalman(x, ...)
}
\arguments{
\item{x}{An object of class \code{iAR}, \code{CiAR}, or \code{BiAR} containing the model specification and parameters:
\itemize{
  \item For \code{iAR} models, the object should contain:
    \itemize{
      \item \code{series}: A numeric vector of the time series data.
      \item \code{times}: A numeric vector specifying the time points of the series.
      \item \code{series_esd}: A numeric vector specifying the error structure.
      \item \code{zero_mean}: A logical value indicating if the series should be zero-centered.
      \item \code{standardized}: A logical value indicating if the series should be standardized.
    }
  \item For \code{CiAR} models, the object should contain:
    \itemize{
      \item \code{series}: A numeric vector of the time series data.
      \item \code{times}: A numeric vector specifying the time points of the series.
      \item \code{series_esd}: A numeric vector specifying the error structure.
      \item \code{zero_mean}: A logical value indicating if the series should be zero-centered.
      \item \code{standardized}: A logical value indicating if the series should be standardized.
      \item \code{c}: A numeric value specifying the scale parameter for the CiAR model.
      \item \code{niter}: An integer specifying the number of iterations for the Kalman filter.
      \item \code{seed}: An integer seed for random number generation (optional).
    }
  \item For \code{BiAR} models, the object should contain:
    \itemize{
      \item \code{series}: A numeric matrix containing two columns for bivariate time series data.
      \item \code{times}: A numeric vector specifying the time points of the series.
      \item \code{series_esd}: A numeric matrix specifying the error structure for both series.
      \item \code{zero_mean}: A logical value indicating if the series should be zero-centered.
      \item \code{niter}: An integer specifying the number of iterations for the Kalman filter.
      \item \code{seed}: An integer seed for random number generation (optional).
    }
}}

\item{...}{Additional arguments (unused).}
}
\value{
An updated object of class \code{iAR}, \code{CiAR}, or \code{BiAR}, where the \code{coef} property is updated with the estimated model parameters (using MLE) and the \code{kalmanlik} property contains the log-likelihood value of the model.
}
\description{
Performs Maximum Likelihood Estimation (MLE) of the parameters of the iAR, CiAR, and BiAR models by maximizing the likelihood function using the Kalman filter.
This method applies the Kalman filter to compute the likelihood and estimate the model parameters that maximize the likelihood for each model type.
}
\details{
This function applies the Kalman filter to perform Maximum Likelihood Estimation (MLE) of the parameters of the autoregressive models (iAR, CiAR, BiAR). The Kalman filter is used to maximize the likelihood function based on the given time series data, and the parameters that maximize the likelihood are estimated.

- For \code{iAR}, the Kalman filter is applied to estimate the model parameters by maximizing the likelihood.
- For \code{CiAR}, the Kalman filter is applied to estimate the parameters of the complex autoregressive model by maximizing the likelihood.
- For \code{BiAR}, the Kalman filter is applied to estimate the parameters of the bivariate autoregressive model by maximizing the likelihood.

The method returns the updated model object, including the estimated parameters and the log-likelihood value.
}
\examples{
# Example 1: Applying Kalman filter for MLE of iAR model parameters
library(iAR)
n=100
set.seed(6714)
times=gentime(n=n)@times
model_norm <- iAR(family = "norm", times = times, coef = 0.9,hessian=TRUE)
model_norm <- sim(model_norm)
model_norm <- kalman(model_norm)
print(model_norm@coef)  # Access the estimated coefficients
print(model_norm@kalmanlik)  # Access the Kalman likelihood value

# Example 2: Applying Kalman filter for MLE of CiAR model parameters
set.seed(6714)
model_CiAR <- CiAR(times = times,coef = c(0.9, 0))
model_CiAR <- sim(model_CiAR)
y=model_CiAR@series
y1=y/sd(y)
model_CiAR@series=y1
model_CiAR@series_esd=rep(0,n)
model_CiAR <- kalman(model_CiAR)
print(model_CiAR@coef)
print(model_CiAR@kalmanlik)

# Example 3: Applying Kalman filter for MLE of BiAR model parameters
set.seed(6714)
model_BiAR <- BiAR(times = times,coef = c(0.9, 0.3), rho = 0.9)
model_BiAR <- sim(model_BiAR)
y=model_BiAR@series
y1=y/apply(y,2,sd)
model_BiAR@series=y1
model_BiAR@series_esd=matrix(0,n,2)
model_BiAR <- kalman(model_BiAR)
print(model_BiAR@coef)
print(model_BiAR@kalmanlik)

}
\references{
\insertRef{Eyheramendy_2018}{iAR},\insertRef{Elorrieta_2019}{iAR},\insertRef{Elorrieta_2021}{iAR}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_unidata.R
\name{unidata}
\alias{unidata}
\title{Unidata Class}
\usage{
unidata(
  times = integer(0),
  series = integer(0),
  series_esd = integer(0),
  series_names = character(0)
)
}
\arguments{
\item{times}{A numeric vector representing the time points.}

\item{series}{A numeric vector representing the values of the time series.}

\item{series_esd}{A numeric vector representing the error standard deviations of the time series.}

\item{series_names}{An optional character vector of length 1 representing the name of the series.}
}
\description{
The `unidata` class is an S7 class designed to represent univariate irregularly observed time series models
with associated times, values, and optional error standard deviations.
}
\section{Validation Rules}{

- `@times`, `@series`, and `@series_esd` must be numeric vectors.
- `@times` must not contain `NA` values and must be strictly increasing.
- The length of `@series` must match the length of `@times`.
- The length of `@series_esd` must be 0, 1, or equal to the length of `@series`.
- `NA` values in `@series` must correspond exactly (positionally) to `NA` values in `@series_esd`.
- `@series_names`, if provided, must be a character vector of length 1.
}

\examples{
# Create a unidata object
unidata_instance <- unidata(
  times = c(1, 2, 3, 4),
  series = c(10, 20, 15, 25),
  series_esd = c(1, 1.5, 1.2, 1.8),
  series_names = "my_series")

}
\seealso{
[iAR], [CiAR]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/09_harmonicfit.R
\name{harmonicfit}
\alias{harmonicfit}
\title{Harmonic Fit to Time Series}
\usage{
harmonicfit(x, ...)
}
\arguments{
\item{x}{An object of class `utilities`, or `NULL` (the default). When `x` is
`NULL` or omitted, the `utilities` object is created internally, so
`harmonicfit(data = clcep, f1 = f1)` is the recommended usage.}

\item{...}{Additional arguments for pairing time series:
\describe{
  \item{data}{A data frame with three columns corresponding to the time, values, and standard errors of the irregularly observed time series.}
  \item{f1}{frequency (1 / period) of the time series.}
  \item{nham}{Number of harmonic components in the model.}
  \item{weighted}{logical; if true, a weighted least squares (WLS) estimation is performed using weights based on the standard deviations of the errors. Default is 'FALSE'.}
  \item{remove_trend}{logical; if true, the linear trend of time series will be removed before the the harmonic model is fitted.}
 }}
}
\value{
An object of class `utilities` with the slots:
\item{fitted_values}{Fitted values from the harmonic model.}
\item{residuals}{Residuals from the harmonic model.} 
\item{coef}{Estimated coefficients of the harmonic model.}
\item{summary}{A summary object containing detailed model information.}
}
\description{
This function fit an k-harmonic function to time series data.
}
\details{
The function fits a harmonic regression model with 'nham' components to the input time series, optionally removing a linear trend and allowing for weighted estimation when standard errors are available.
}
\examples{
data(clcep)
f1=0.060033386
o1=harmonicfit(data=clcep,f1=f1)
head(o1@fitted_values)
head(o1@residuals)
}
\references{
\insertRef{Elorrieta_2021}{iAR}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_sim.R
\name{sim}
\alias{sim}
\title{Simulate Time Series for Multiple iAR Models}
\usage{
sim(x, ...)
}
\arguments{
\item{x}{An object of class \code{iAR}, \code{CiAR}, or \code{BiAR}, containing the model specification and parameters:
\itemize{
  \item For \code{iAR} (irregular AR models), the model family could be "norm", "t", or "gamma", where:
    \itemize{
      \item \code{family}: The distribution family of the iAR model (one of "norm", "t", or "gamma").
      \item \code{coef}: The coefficient(s) of the iAR model.
      \item \code{times}: A numeric vector specifying the time points of the series.
      \item \code{df}: Degrees of freedom for the t-distribution (only for \code{family = "t"}).
      \item \code{sigma}: The scale parameter for the t-distribution (only for \code{family = "t"}).
      \item \code{mean}: The mean parameter for the gamma distribution (only for \code{family = "gamma"}).
      \item \code{variance}: The variance parameter for the gamma distribution (only for \code{family = "gamma"}).
    }
  \item For \code{CiAR} (complex irregular autoregressive models):
    \itemize{
      \item \code{coef}: The real and imaginary parts of the CiAR model's coefficients.
      \item \code{times}: A numeric vector specifying the time points of the series.
      \item \code{rho}: The correlation parameter for the CiAR model.
      \item \code{c}: The scale parameter for the CiAR model.
    }
  \item For \code{BiAR} (BiAR models):
    \itemize{
      \item \code{coef}: The coefficients of the BiAR model (real and imaginary).
      \item \code{times}: A numeric vector specifying the time points of the series.
      \item \code{rho}: The correlation parameter for the BiAR model.
      \item \code{series_esd}: The series for the error structure (optional, used internally).
    }
}}

\item{...}{Additional arguments for generating irregular times. This is used only if no time points have been provided in the \code{times} argument:
\describe{
\item{n}{A positive integer. Length of observation times. Default is 100. This is used only if no time points are provided in \code{times}.}
\item{distribution}{A character string specifying the distribution of the observation times. Default is `"expmixture"`. Available options are:
\describe{
  \item{expmixture}{A mixture of two exponential distributions.}
  \item{uniform}{A uniform distribution.}
  \item{exponential}{A single exponential distribution.}
  \item{gamma}{A gamma distribution.}
  }
  }
  \item{lambda1}{Mean (1/rate) of the exponential distribution or the first exponential distribution in a mixture of exponential distributions. Default is `130`.}
  \item{lambda2}{Mean (1/rate) of the second exponential distribution in a mixture of exponential distributions. Default is `6.5`.}
  \item{p1}{Weight of the first exponential distribution in a mixture of exponential distributions. Default is `0.15`.}
  \item{p2}{Weight of the second exponential distribution in a mixture of exponential distributions. Default is `0.85`.}
  \item{a}{Shape parameter of a gamma distribution or lower limit of the uniform distribution. Default is `0`.}
  \item{b}{Scale parameter of a gamma distribution or upper limit of the uniform distribution. Default is `1`.}
}}
}
\value{
An updated object of class \code{iAR}, \code{CiAR}, or \code{BiAR}, where the \code{series} property contains the simulated time series.
}
\description{
Simulates a time series for irregular autoregressive (iAR) models, including:
1. Normal iAR model (`iAR`)
2. T-distribution iAR model (`iAR-T`)
3. Gamma-distribution iAR model (`iAR-Gamma`)
}
\details{
This function simulates time series based on the specified model and its parameters. Depending on the class of the input object:
\itemize{
  \item For \code{iAR} models, it supports three distribution families:
    \item "norm" for normal distribution.
    \item "t" for t-distribution.
    \item "gamma" for gamma distribution.
  \item For \code{CiAR} models, it uses complex autoregressive processes to generate the time series.
  \item For \code{BiAR} models, it simulates a BiAR process using specified coefficients and correlation.
}
The coefficients and any family-specific parameters must be set before calling this function.
}
\examples{
# Example 1: Simulating a normal iAR model
library(iAR)
n=100
set.seed(6714)
times=gentime(n=n)@times
model_norm <- iAR(family = "norm", times = times, coef = 0.9,hessian=TRUE)
model_norm <- sim(model_norm)
plot(model_norm, type = "l", main = "Simulated iAR-Norm Series")

# Example 2: Simulating a CiAR model
set.seed(6714)
model_CiAR <- CiAR(times = times,coef = c(0.9, 0))
model_CiAR <- sim(model_CiAR)
plot(model_CiAR , type = "l", main = "Simulated CiAR Series")

# Example 3: Simulating a BiAR model
set.seed(6714)
model_BiAR <- BiAR(times = times,coef = c(0.9, 0.3), rho = 0.9)
model_BiAR <- sim(model_BiAR)
plot(times, model_BiAR@series[,1], type = "l", main = "Simulated BiAR Series")

}
\references{
\insertRef{Eyheramendy_2018}{iAR},\insertRef{Elorrieta_2019}{iAR},\insertRef{Elorrieta_2021}{iAR}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/08_summary.R
\name{summary}
\alias{summary}
\title{Summary of iAR package models}
\arguments{
\item{object}{An object of class `iAR`, `CiAR`, `BiAR`, or `utilities`.
For fitted model objects, this contains model-specific parameters such as
coefficients (`coef`), standard errors (`stderr`), t-values (`tvalue`),
p-values (`pvalue`), and other class-specific components.}

\item{...}{Additional arguments (unused).}
}
\description{
This method provides a summary for objects created by the iAR package,
including `iAR`, `CiAR`, `BiAR`, and `utilities` objects. For fitted model
objects, the summary reports model-specific information such as the model
family, coefficients, standard errors, t-values, and p-values.
The output is formatted to display the relevant statistics based on the model class.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/iAR.R
\name{iAR-package}
\alias{iAR-package}
\title{iAR: Irregularly Observed Autoregressive Models}
\description{
Description: Data sets, functions and scripts with examples to implement autoregressive
models for irregularly observed time series. The models available in this package
are the irregular autoregressive model (Eyheramendy et al.(2018) <doi:10.1093/mnras/sty2487>),
the complex irregular autoregressive model (Elorrieta et al.(2019) <doi:10.1051/0004-6361/201935560>)
and the bivariate irregular autoregressive model (Elorrieta et al.(2021) <doi:10.1093/mnras/stab1216>)
}
\details{
"_PACKAGE"
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_multidata.R
\name{multidata}
\alias{multidata}
\title{Multidata Class}
\usage{
multidata(
  times = integer(0),
  series = integer(0),
  series_esd = integer(0),
  series_names = character(0)
)
}
\arguments{
\item{times}{A numeric vector representing the time points of the time series.}

\item{series}{A numeric vector or matrix representing the main time series.}

\item{series_esd}{A numeric vector or matrix representing the additional series, such as error standard deviations or other related data.}

\item{series_names}{An optional character vector representing the name of the series.}
}
\description{
The `multidata` class is an S7 class designed to represent multidimensional time series models, including the main time series and additional series (e.g., error standard deviations or related variables).
}
\section{Validation Rules}{

- `@times` must be a numeric vector and strictly increasing.
- `@series` must be a numeric matrix or empty. Its number of rows must match the length of `@times`.
- `@series_esd` must be a numeric matrix or empty. If provided and not empty, its dimensions must match those of `@series`.
- If `@series_names` is provided, it must be a character vector of the same length as the number of columns of `@series`, and contain unique values.
}

\examples{
# Create a multidata object
multidata_instance <- multidata(
  times = c(1, 2, 3, 4),
  series = matrix(c(10, 20, 15, 25,
                    12, 18, 17, 23), 
                  nrow = 4, ncol = 2),
  series_esd = matrix(c(1, 1.5, 1.2, 1.8,
                        0.9, 1.3, 1.1, 1.7),
                      nrow = 4, ncol = 2)
)
}
\seealso{
[BiAR]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/09_pairingits.R
\name{pairingits}
\alias{pairingits}
\title{Pairing two irregularly observed time series}
\usage{
pairingits(x, ...)
}
\arguments{
\item{x}{An object of class `utilities`, or `NULL` (the default). When `x` is
`NULL` or omitted, the `utilities` object is created internally, so
`pairingits(data1 = ts1, data2 = ts2)` is the recommended usage.}

\item{...}{Additional arguments for pairing time series: 
\describe{
  \item{data1}{A data frame with three columns corresponding to the first irregularly observed time series.}
  \item{data2}{A data frame with three columns corresponding to the second irregularly observed time series.}
  \item{tol}{A numeric value indicating the tolerance parameter.}
}}
}
\value{
An object of class `utilities` with the slots:
\item{series}{A matrix containing the paired time series, where unmatched measurements are filled with `NA`.}
\item{series_esd}{A matrix containing the paired error standard deviations of the time series, where unmatched measurements are filled with `NA`.}
\item{times}{A numeric vector with the paired observational times.}
}
\description{
This method pairs the observational times of two irregularly observed time series.
}
\details{
The method checks the observational times in both input time series and pairs the measurements if they fall within the specified tolerance (`tol`). If a measurement in one series cannot be paired, it is filled with `NA` values for the corresponding columns of the other series.
}
\examples{
data(cvnovag)
data(cvnovar)
o1=pairingits(data1=cvnovag, data2=cvnovar, tol=0.1)
pargr1=na.omit(o1@paired)
st=apply(pargr1[,c(1,4)],1,mean)
model_BiAR <- BiAR(times = st,series=pargr1[,c(2,5)],series_esd=pargr1[,c(3,6)])
model_BiAR <- kalman(model_BiAR)

}
\references{
\insertRef{Elorrieta_2021}{iAR}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_BiARclass.R
\name{BiAR}
\alias{BiAR}
\title{`BiAR` Class}
\usage{
BiAR(
  times = integer(0),
  series = integer(0),
  series_esd = integer(0),
  series_names = character(0),
  fitted_values = integer(0),
  loglik = integer(0),
  kalmanlik = integer(0),
  coef = c(0.8, 0),
  tAhead = 1,
  rho = 0,
  forecast = integer(0),
  interpolated_values = integer(0),
  interpolated_times = integer(0),
  interpolated_series = integer(0),
  zero_mean = TRUE,
  standardized = TRUE,
  hessian = FALSE,
  summary = list()
)
}
\arguments{
\item{times}{A numeric vector representing the time points.}

\item{series}{A numeric matrix or vector representing the values of the time series.}

\item{series_esd}{A numeric matrix or vector representing the error standard deviations of the time series.}

\item{series_names}{An optional character vector representing the name of the series.}

\item{fitted_values}{A numeric vector containing the fitted values from the model.}

\item{loglik}{A numeric value representing the log-likelihood of the model.}

\item{kalmanlik}{A numeric value representing the Kalman likelihood of the model.}

\item{coef}{A numeric vector of length 2 containing the autoregressive (phiR) and
cross-correlation (phiI) coefficients of the model. Each value must be in [-1, 1]
and the modulus `sqrt(phiR^2 + phiI^2)` must be less than 1. Defaults to `c(0.8, 0)`.
Unlike [iAR], which uses a scalar, both [CiAR] and `BiAR` use a length-2 vector.}

\item{tAhead}{A numeric value specifying the forecast horizon (default: 1).}

\item{rho}{A numeric vector containing the estimated coefficients of the model.}

\item{forecast}{A numeric vector containing the forecasted values.}

\item{interpolated_values}{A numeric vector containing the interpolated values.}

\item{interpolated_times}{A numeric vector containing the times of the interpolated data points.}

\item{interpolated_series}{A numeric vector containing the interpolated series.}

\item{zero_mean}{A logical value indicating if the model assumes a zero-mean process (default: TRUE).}

\item{standardized}{A logical value indicating if the model assumes a standardized process (default: TRUE).}

\item{hessian}{A logical value indicating whether the Hessian matrix is computed during estimation (default: FALSE).}

\item{summary}{A list containing the summary of the model fit, including diagnostics and statistical results.}
}
\description{
Represents a bivariate irregular autoregressive (BiAR) time series model.
This class extends the `multidata` class and provides additional properties
for modeling, forecasting, and interpolation of bivariate time series data.
}
\details{
The `BiAR` class is designed to handle bivariate irregularly observed time series data
using an autoregressive approach. It extends the `multidata` class to include 
additional properties for modeling bivariate time series.

Key features of the `BiAR` class include:
- Support for bivariate time series data.
- Forecasting and interpolation functionalities for irregular time points.
- Assumptions of zero-mean and standardized processes, configurable by the user.
- Estimation of model parameters and likelihoods, including Kalman likelihood.
}
\section{Validation Rules}{

- `@times` must be a numeric vector without dimensions and strictly increasing.
- `@series` must be a numeric matrix with two columns (bivariate) or be empty.
- The number of rows in `@series` must match the length of `@times`.
- `@series_esd`, if provided, must be a numeric matrix. Its dimensions must match those of `@series`, or it must have one row and the same number of columns.
- If `@series_esd` contains NA values, they must correspond positionally to NA values in `@series`.
- `@series_names`, if provided, must be a character vector with length equal to the number of columns in `@series`, and all names must be unique.
- `@coef` must be a numeric vector of length 2, with each element strictly between -1 and 1.
- The modulus `sqrt(phiR^2 + phiI^2)` must be strictly less than 1 (stationarity condition).
- `@tAhead` must be a strictly positive numeric scalar.
}

\examples{
times=gentime(n=200, distribution = "expmixture",
             lambda1 = 130, lambda2 = 6.5, p1 = 0.15, p2 = 0.85)@times
my_BiAR <- BiAR(times = times,coef = c(0.9, 0.3), rho = 0.9)

# Access properties
my_BiAR@coef

}
\references{
\insertRef{Elorrieta_2021}{iAR}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_CiARclass.R
\name{CiAR}
\alias{CiAR}
\title{`CiAR` Class}
\usage{
CiAR(
  times = integer(0),
  series = integer(0),
  series_esd = integer(0),
  series_names = character(0),
  fitted_values = integer(0),
  kalmanlik = integer(0),
  coef = c(0.9, 0),
  tAhead = 1,
  forecast = integer(0),
  interpolated_values = integer(0),
  interpolated_times = integer(0),
  interpolated_series = integer(0),
  zero_mean = TRUE,
  standardized = TRUE,
  hessian = FALSE,
  summary = list()
)
}
\arguments{
\item{times}{A numeric vector representing the time points.}

\item{series}{A complex vector representing the values of the time series.}

\item{series_esd}{A numeric vector representing the error standard deviations of the time series.}

\item{series_names}{An optional character vector of length 1 representing the name of the series.}

\item{fitted_values}{A numeric vector containing the fitted values from the model.}

\item{kalmanlik}{A numeric value representing the Kalman likelihood of the model.}

\item{coef}{A numeric vector of length 2 containing the real (phiR) and imaginary (phiI)
parts of the autoregressive coefficient. Each value must be in [-1, 1] and the modulus
`sqrt(phiR^2 + phiI^2)` must be strictly less than 1 (stationarity condition).
Defaults to `c(0.9, 0)`. Unlike [iAR], which uses a scalar, both `CiAR` and [BiAR]
use a length-2 vector.}

\item{tAhead}{A numeric value specifying the forecast horizon (default: 1).}

\item{forecast}{A numeric vector containing the forecasted values.}

\item{interpolated_values}{A numeric vector containing the interpolated values.}

\item{interpolated_times}{A numeric vector containing the times of the interpolated data points.}

\item{interpolated_series}{A numeric vector containing the interpolated series.}

\item{zero_mean}{A logical value indicating if the model assumes a zero-mean process (default: TRUE).}

\item{standardized}{A logical value indicating if the model assumes a standardized process (default: TRUE).}

\item{hessian}{A logical value indicating whether the Hessian matrix is computed during estimation (default: FALSE).}

\item{summary}{A list containing the summary of the model fit, including diagnostics and statistical results.}
}
\description{
Represents a complex irregular autoregressive (CiAR) time series model. 
This class extends the `unidata` class and provides additional properties 
for modeling, forecasting, and interpolating irregularly observed time 
series data with both negative and positive autocorrelation.
}
\details{
The `CiAR` class is designed to handle irregularly observed time series 
data with either negative or positive autocorrelation using an autoregressive 
approach. It extends the `unidata` class to include functionalities 
specific to the `CiAR` model.

Key features of the `CiAR` class include:
- Support for irregularly observed time series data with negative 
or positive autocorrelation.
- Forecasting and interpolation functionalities for irregular time points.
- Configurable assumptions of zero-mean and standardized processes.
}
\section{Validation}{

- Inherits all validation rules from the `unidata` class:
 - `@times`, `@series`, and `@series_esd` must be numeric vectors.
 - `@times` must not contain `NA` values and must be strictly increasing.
 - The length of `@series` must match the length of `@times`.
 - The length of `@series_esd` must be 0, 1, or equal to the length of `@series`.
 - `NA` values in `@series` must correspond exactly (positionally) to `NA` values in `@series_esd`.
 - `@series_names`, if provided, must be a character vector of length 1.

- `@coef` must be a numeric vector of length 2 with no dimensions.
- Each value in `@coef` must be in the interval [-1, 1].
- The modulus `sqrt(phiR^2 + phiI^2)` must be strictly less than 1 (stationarity condition).
- `@tAhead` must be a strictly positive numeric scalar.
}

\examples{
times=gentime(n=200, distribution = "expmixture",
             lambda1 = 130, lambda2 = 6.5, p1 = 0.15, p2 = 0.85)@times
my_CiAR <- CiAR(times = times,coef = c(0.9, 0))

# Access properties
my_CiAR@coef

}
\references{
\insertRef{Elorrieta_2019}{iAR}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clcep.R
\docType{data}
\name{clcep}
\alias{clcep}
\title{Classical Cepheid}
\format{
A data frame with 109 observations on the following 3 variables:
\describe{
  \item{t}{heliocentric Julian Day}
  \item{m}{magnitude}
  \item{merr}{measurement error of the magnitude (in mag).}
}
}
\usage{
clcep
}
\description{
Time series of a classical cepheid variable star obtained from HIPPARCOS.
}
\details{
The frequency computed by GLS for this light curve is 0.060033386.
Catalogs and designations of this star:
HD 1989: HD 305996
TYCHO-2 2000:TYC 8958-2333-1
USNO-A2.0:USNO-A2 0225-10347916
HIP: HIP-54101
}
\examples{
data(clcep)
f1=0.060033386
o1=phase(data=clcep,f1=f1,twop=TRUE)
plot(o1@times_phased,o1@series_phased,pch=20)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_iARclass.R
\name{iAR}
\alias{iAR}
\title{`iAR` Class}
\usage{
iAR(
  times = integer(0),
  series = integer(0),
  series_esd = integer(0),
  series_names = character(0),
  family = "norm",
  fitted_values = integer(0),
  loglik = integer(0),
  kalmanlik = integer(0),
  coef = 0.9,
  df = 3,
  sigma = 1,
  mean = 0,
  variance = 1,
  tAhead = 1,
  forecast = integer(0),
  interpolated_values = integer(0),
  interpolated_times = integer(0),
  interpolated_series = integer(0),
  zero_mean = TRUE,
  standardized = TRUE,
  hessian = FALSE,
  summary = list()
)
}
\arguments{
\item{times}{A numeric vector representing the time points.}

\item{series}{A numeric vector representing the values of the time series.}

\item{series_esd}{A numeric vector representing the error standard deviations of the time series.}

\item{series_names}{An optional character vector of length 1 representing the name of the series.}

\item{family}{A character string indicating the distribution family of the model. Must be one of `"norm"`, `"t"`, or `"gamma"`. Defaults to `"norm"`.}

\item{fitted_values}{A numeric vector containing the fitted values from the model.}

\item{loglik}{A numeric value representing the log-likelihood of the model.}

\item{kalmanlik}{A numeric value representing the Kalman likelihood of the model.}

\item{coef}{A single numeric value (scalar) representing the autoregressive coefficient,
strictly between 0 and 1 (default: 0.9). Note that `iAR` only supports positive
autocorrelation. Unlike [CiAR] and [BiAR], which use a numeric vector of length 2,
this argument must be a scalar; passing a vector will raise a validation error.}

\item{df}{A numeric value representing the degrees of freedom (`t` distribution) (default: 3).}

\item{sigma}{A numeric value representing the scale parameter (`t` distribution) (default: 1).}

\item{mean}{A numeric value representing the estimated mean of the model (`gamma` parameter).}

\item{variance}{A numeric value representing the estimated variance of the model (`gamma` parameter).}

\item{tAhead}{A numeric value specifying the forecast horizon (default: 1).}

\item{forecast}{A numeric vector containing the forecasted values.}

\item{interpolated_values}{A numeric vector containing the interpolated values.}

\item{interpolated_times}{A numeric vector containing the times of the interpolated data points.}

\item{interpolated_series}{A numeric vector containing the interpolated series.}

\item{zero_mean}{A logical value indicating if the model assumes a zero-mean process (default: TRUE).}

\item{standardized}{A logical value indicating if the model assumes a standardized process (default: TRUE).}

\item{hessian}{A logical value indicating whether the Hessian matrix is computed during estimation (default: FALSE).}

\item{summary}{A list containing the summary of the model fit, including diagnostics and statistical results.}
}
\description{
Represents a univariate irregular autoregressive (iAR) time series model.
This class extends the "unidata" class and includes additional properties
for modeling, forecasting, and interpolation.
}
\details{
The `iAR` class is designed to handle irregularly observed time series data using an
autoregressive approach. It extends the "unidata" class to include additional
modeling and diagnostic capabilities. Key functionalities include forecasting,
interpolation, and model fitting.

The class also supports advanced modeling features, such as:
- Different distribution families for the data (e.g., Gaussian, `t`-distribution).
- Optional computation of the Hessian matrix for parameter estimation.
- Standardized or zero-mean process assumptions.
}
\section{Validation Rules}{

- Inherits all validation rules from the `unidata` class:
 - `@times`, `@series`, and `@series_esd` must be numeric vectors.
 - `@times` must not contain `NA` values and must be strictly increasing.
 - The length of `@series` must match the length of `@times`.
 - The length of `@series_esd` must be 0, 1, or equal to the length of `@series`.
 - `NA` values in `@series` must correspond exactly (positionally) to `NA` values in `@series_esd`.
 - `@series_names`, if provided, must be a character vector of length 1.

- `@family` must be one of `"norm"`, `"t"`, or `"gamma"`.
- `@coef` must be a numeric scalar strictly between 0 and 1.
- `@df` must be a positive integer scalar **only if** `family = "t"`; otherwise, it should not be specified.
- `@sigma` must be a strictly positive numeric scalar (used in `"t"` family).
- `@mean` and `@variance` must be strictly positive numeric scalars **only if** `family = "gamma"`.
- `@tAhead` must be a strictly positive numeric scalar.
}

\examples{
# Create an `iAR` object
times=gentime(n=200, distribution = "expmixture",
             lambda1 = 130, lambda2 = 6.5, p1 = 0.15, p2 = 0.85)@times
my_iAR <- iAR(family = "norm", times = times, coef = 0.9,hessian=TRUE)

my_iAR@family
my_iAR@coef

}
\references{
\insertRef{Eyheramendy_2018}{iAR}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/06_forecast.R
\name{forecast}
\alias{forecast}
\title{Forecast for iAR, CiAR, and BiAR Classes}
\usage{
forecast(x, ...)
}
\arguments{
\item{x}{An object of class \code{iAR}, \code{CiAR}, or \code{BiAR}, containing the model specification and parameters:
\itemize{
  \item For \code{iAR}:
    \itemize{
      \item \code{family}: The distribution family of the iAR model (one of "norm", "t", or "gamma").
      \item \code{series}: A numeric vector representing the time series to forecast.
      \item \code{coef}: The coefficient(s) of the iAR model.
      \item \code{zero_mean}: Logical, whether the model assumes a zero-mean series.
      \item \code{standardized}: Logical, whether the model uses standardized data (only for "norm" family).
      \item \code{mean}: The mean parameter (only for "gamma" family).
      \item \code{tAhead}: Integer, the number of steps ahead to forecast.
    }
  \item For \code{CiAR}:
    \itemize{
      \item \code{coef}: The real and imaginary parts of the CiAR model's coefficients.
      \item \code{series}: A numeric vector representing the time series to forecast.
      \item \code{times}: A numeric vector specifying the time points of the series.
      \item \code{zero_mean}: Logical, whether the model assumes a zero-mean series.
      \item \code{standardized}: Logical, whether the model output should be standardized.
      \item \code{tAhead}: Integer, the number of steps ahead to forecast.
    }
  \item For \code{BiAR}:
    \itemize{
      \item \code{coef}: The coefficients of the BiAR model (real and imaginary parts).
      \item \code{series}: A numeric matrix with two columns representing the bivariate time series to forecast.
      \item \code{times}: A numeric vector specifying the time points of the series.
      \item \code{tAhead}: Integer, the number of steps ahead to forecast.
    }
}}

\item{...}{Additional arguments.}
}
\value{
An updated object of class \code{iAR}, \code{CiAR}, or \code{BiAR}, where the \code{forecast} property contains the forecasted values.
}
\description{
Generates forecasts for the specified time series model. This method is implemented for:
1. Irregular Autoregressive models (`iAR`)
2. Complex Irregular Autoregressive models (`CiAR`)
3. Bivariate Autoregressive models (`BiAR`)
}
\details{
This method generates forecasts for the specified time series model. Depending on the class of the input object:
\itemize{
  \item For \code{iAR}, the function supports three distribution families:
    \item "norm" for normal distribution.
    \item "t" for t-distribution.
    \item "gamma" for gamma distribution.
  \item For \code{CiAR}, the function uses complex autoregressive processes.
  \item For \code{BiAR}, the function generates forecasts for a bivariate autoregressive process.
}
All required parameters (e.g., coefficients, time points) must be set before calling this method.
}
\examples{
# Example 1: Forecasting with a normal iAR model
library(iAR)
n=100
set.seed(6714)
times=gentime(n=n)@times
model_norm <- iAR(family = "norm", times = times, coef = 0.9)
model_norm <- sim(model_norm)
model_norm <- kalman(model_norm)
tAhead=1.3
model_norm <- forecast(model_norm,tAhead=tAhead)
plot(times, model_norm@series, type = "l", main = "Original Series with Forecast")
points(max(times)+ tAhead, model_norm@forecast, col = "blue", pch = 16)
plot_forecast(model_norm)

# Example 2: Forecasting with a CiAR model
set.seed(6714)
model_CiAR <- CiAR(times = times,coef = c(0.9, 0))
model_CiAR <- sim(model_CiAR)
y=model_CiAR@series
y1=y/sd(y)
model_CiAR@series=y1
model_CiAR@series_esd=rep(0,n)
model_CiAR <- kalman(model_CiAR)
print(model_CiAR@coef)
model_CiAR@tAhead=1.3
model_CiAR <-forecast(model_CiAR)
model_CiAR@forecast

# Example 3: Forecasting with a BiAR model
n=80
set.seed(6714)
times=gentime(n=n)@times
model_BiAR <- BiAR(times = times,coef = c(0.9, 0.3), rho = 0.9)
model_BiAR <- sim(model_BiAR)
y=model_BiAR@series
y1=y/apply(y,2,sd)
model_BiAR@series=y1
model_BiAR@series_esd=matrix(0,n,2)
model_BiAR <- kalman(model_BiAR)
print(model_BiAR@coef)
model_BiAR@tAhead=1.3
model_BiAR <-forecast(model_BiAR)
model_BiAR@forecast

}
\references{
\insertRef{Eyheramendy_2018}{iAR},\insertRef{Elorrieta_2019}{iAR},\insertRef{Elorrieta_2021}{iAR}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/07_interpolation.R
\name{interpolation}
\alias{interpolation}
\title{Interpolation for iAR, CiAR, and BiAR Classes}
\usage{
interpolation(x, ...)
}
\arguments{
\item{x}{An object of class \code{iAR}, \code{CiAR}, or \code{BiAR}, containing the model specification and parameters:
\itemize{
  \item For \code{iAR}:
    \itemize{
      \item \code{family}: The distribution family of the iAR model (one of "norm", "t", or "gamma").
      \item \code{series}: A numeric vector representing the time series to interpolate.
      \item \code{coef}: The coefficients of the iAR model.
      \item \code{zero_mean}: Logical, whether the model assumes a zero-mean series.
      \item \code{standardized}: Logical, whether the model uses standardized data (only for "norm" family).
      \item \code{mean}: The mean parameter (only for "gamma" family).
    }
  \item For \code{CiAR}:
    \itemize{
      \item \code{coef}: The coefficients of the CiAR model.
      \item \code{series_esd}: The series of error standard deviations for the CiAR model.
      \item \code{zero_mean}: Logical, whether the model assumes a zero-mean series.
      \item \code{standardized}: Logical, whether the model uses standardized data.
      \item \code{seed}: Optional seed for random number generation.
    }
  \item For \code{BiAR}:
    \itemize{
      \item \code{coef}: The coefficients of the BiAR model.
      \item \code{series_esd}: The series of error standard deviations for the BiAR model.
      \item \code{zero_mean}: Logical, whether the model assumes a zero-mean series.
      \item \code{yini1}: Initial value for the first time series (for BiAR models).
      \item \code{yini2}: Initial value for the second time series (for BiAR models).
      \item \code{seed}: Optional seed for random number generation.
    }
}}

\item{...}{Additional arguments (unused).}
}
\value{
An object of the same class as \code{x} with interpolated time series.
}
\description{
This method performs imputation of missing values in a time series using an autoregressive model.
The imputation is done iteratively for each missing value, utilizing available data and model coefficients.
Depending on the model family, the interpolation is performed differently:
- For \code{norm}: A standard autoregressive model for normally distributed data.
- For \code{t}: A model for time series with t-distributed errors.
- For \code{gamma}: A model for time series with gamma-distributed errors.
- For \code{CiAR}: A complex irregular autoregressive model.
- For \code{BiAR}: A bivariate autoregressive model.
}
\details{
Performs interpolation on time series with missing values. This method is implemented for:
1. Irregular Autoregressive models (iAR)
2. Complex Irregular Autoregressive models (CiAR)
3. Bivariate Autoregressive models (BiAR)


The method handles missing values (NA) in the time series by imputing them iteratively. 
For each missing value, the available data is used to fit the autoregressive model, and the missing value is imputed based on the model's output.
For the \code{CiAR} and \code{BiAR} models, the error standard deviations and initial values are also considered during imputation.
}
\examples{
# Interpolation for iAR model
library(iAR)
n=100
set.seed(6714)
times=gentime(n=n)@times
model_norm <- iAR(family = "norm", times = times, coef = 0.9)  
model_norm <- sim(model_norm)
y=model_norm@series
y1=y/sd(y)
model_norm@series=y1
model_norm@series_esd=rep(0,n)
model_norm <- kalman(model_norm) 
print(model_norm@coef)
napos=10
model_norm@series[napos]=NA
model_norm <- interpolation(model_norm)
interpolation=na.omit(model_norm@interpolated_values)
mse=as.numeric(y1[napos]-interpolation)^2
print(mse)
plot(times,y,type='l',xlim=c(times[napos-5],times[napos+5]))
points(times,y,pch=20)
points(times[napos],interpolation*sd(y),col="red",pch=20)

# Interpolation for CiAR model
model_CiAR <- CiAR(times = times,coef = c(0.9, 0))
model_CiAR <- sim(model_CiAR)
y=model_CiAR@series
y1=y/sd(y)
model_CiAR@series=y1
model_CiAR@series_esd=rep(0,n)
model_CiAR <- kalman(model_CiAR)
print(model_CiAR@coef)
napos=10
model_CiAR@series[napos]=NA
model_CiAR <- interpolation(model_CiAR)
interpolation=na.omit(model_CiAR@interpolated_values)
mse=as.numeric(y1[napos]-interpolation)^2
print(mse)
plot(times,y,type='l',xlim=c(times[napos-5],times[napos+5]))
points(times,y,pch=20)
points(times[napos],interpolation*sd(y),col="red",pch=20)
# Interpolation for BiAR model
model_BiAR <- BiAR(times = times,coef = c(0.9, 0.3), rho = 0.9)
model_BiAR <- sim(model_BiAR)
y=model_BiAR@series
y1=y/apply(y,2,sd)
model_BiAR@series=y1
model_BiAR@series_esd=matrix(0,n,2)
model_BiAR <- kalman(model_BiAR)
print(model_BiAR@coef) 
napos=10
model_BiAR@series[napos,1]=NA
model_BiAR@series_esd[napos,1]=NA
model_BiAR <- interpolation(model_BiAR)
interpolation=na.omit(model_BiAR@interpolated_values[,1])
mse=as.numeric(y1[napos,1]-interpolation)^2
print(mse)
par(mfrow=c(2,1))
plot(times,y[,1],type='l',xlim=c(times[napos-5],times[napos+5]))
points(times,y[,1],pch=20)
points(times[napos],interpolation*apply(y,1,sd)[1],col="red",pch=20)
plot(times,y[,2],type='l',xlim=c(times[napos-5],times[napos+5]))
points(times,y[,2],pch=20)
}
\references{
\insertRef{Eyheramendy_2018}{iAR},\insertRef{Elorrieta_2019}{iAR},\insertRef{Elorrieta_2021}{iAR}
}
\seealso{
\code{\link{forecast}} for forecasting methods for these models.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/09_phase.R
\name{phase}
\alias{phase}
\title{Computing phased time series}
\usage{
phase(x, ...)
}
\arguments{
\item{x}{An object of class `utilities`, or `NULL` (the default). When `x` is
`NULL` or omitted, the `utilities` object is created internally, so
`phase(data = clcep, f1 = f1)` is the recommended usage.}

\item{...}{Additional arguments for pairing time series:
\describe{
  \item{data}{A data frame with three columns corresponding to the time, values, and standard errors of the irregularly observed time series.}
  \item{f1}{frequency (1 / period) of the time series.}
  \item{twop}{logical; if TRUE, the phased series will be duplicated over two cycles (0–2).}
 }}
}
\value{
An object of class `utilities` with the slots:
\item{series_phased}{A numeric vector containing the time series values ordered by phase.}
\item{series_esd_phased}{A numeric vector containing the error standard deviations of the time series ordered by phase.}
\item{times_phased}{A numeric vector of phased times (values between 0 and 1, or 0 and 2 if 'two.cycles = TRUE').}
}
\description{
This function computes a time series folded on its period.
}
\details{
The phase \eqn{\phi} of an observation is computed as
\deqn{\phi = \frac{t - t_0}{p} - \mathrm{E}(t),}
where \eqn{t_0} is the reference time (by default the first observation),
\eqn{p = 1/f_1} is the period, and \eqn{\mathrm{E}(t)} is the integer part of
\eqn{(t - t_0)/p}.
}
\examples{
data(clcep)
f1=0.060033386
o1=phase(data=clcep,f1=f1,twop=TRUE)
plot(o1@times_phased,o1@series_phased,pch=20)
}
\references{
\insertRef{Elorrieta_2021}{iAR}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/09_gentime.R
\name{gentime}
\alias{gentime}
\title{Generating Irregularly spaced times}
\usage{
gentime(x = NULL, ...)
}
\arguments{
\item{x}{An object of class `utilities`, or `NULL` (the default). When `x` is
`NULL` or omitted, the `utilities` object is created internally, so
`gentime(n = 200)` is equivalent to the three-step pattern
`o <- utilities(); o <- gentime(o, n = 200); o`.}

\item{...}{Additional arguments for generating time points: 
\describe{
   \item{n}{A positive integer. Length of observation times.}
   \item{distribution}{A character string specifying the distribution of the observation times. Default is `"expmixture"`.
Available options are:
- `"expmixture"`: A mixture of two exponential distributions.
- `"uniform"`: A uniform distribution.
- `"exponential"`: A single exponential distribution.
- `"gamma"`: A gamma distribution.}
   \item{lambda1}{Mean (1/rate) of the exponential distribution or the first exponential distribution in a mixture of exponential distributions. Default is `130`.}
   \item{lambda2}{Mean (1/rate) of the second exponential distribution in a mixture of exponential distributions. Default is `6.5`.}
   \item{p1}{Weight of the first exponential distribution in a mixture of exponential distributions. Default is `0.15`.}
   \item{p2}{Weight of the second exponential distribution in a mixture of exponential distributions. Default is `0.85`.}
   \item{a}{Shape parameter of a gamma distribution or lower limit of the uniform distribution. Default is `0`.}
   \item{b}{Scale parameter of a gamma distribution or upper limit of the uniform distribution. Default is `1`.}
}}
}
\value{
An updated `utilities` object with the generated observation times stored in the `times` slot.
}
\description{
A method for generating time points based on a statistical distribution.
The results are stored in the `times` slot of the `utilities` object.
}
\examples{
set.seed(12917)
st=gentime(n=200, distribution = "expmixture", lambda1 = 130,
lambda2 = 6.5, p1 = 0.15, p2 = 0.85)@times
mean(diff(st))

st=gentime(n=200, distribution = "expmixture", lambda1 = 15,
lambda2 = 2.5, p1 = 0.15, p2 = 0.85)@times
mean(diff(st))

}
\references{
\insertRef{Eyheramendy_2018}{iAR}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/agn.R
\docType{data}
\name{agn}
\alias{agn}
\title{Active Galactic Nuclei}
\format{
A data frame with 237 observations on the following 3 variables:
\describe{
  \item{t}{heliocentric Julian Day - 2450000}
  \item{m}{Flux $(10^(-15) ergs/s/cm^2 /A)$}
  \item{merr}{measurement error standard deviations.}
}
}
\usage{
agn
}
\description{
Time series of the AGN MCG-6-30-15 measured in the K-band between 2006 August and 2011 July with the ANDICAM camera mounted on the 1.3 m telescope at Cerro Tololo Inter-American Observatory (CTIO)
}
\examples{
data(agn)
plot(agn$t,agn$m,type="l",ylab="",xlab="")
}
\references{
\insertRef{Lira_2015}{iAR}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/03_iARloglik.R
\name{loglik}
\alias{loglik}
\title{Maximum Likelihood Estimation for iAR Models}
\usage{
loglik(x, ...)
}
\arguments{
\item{x}{An object of class \code{iAR}, containing the model specification, parameters, and the time series to be evaluated:
\itemize{
  \item \code{series}: The observed time series.
  \item \code{times}: A numeric vector specifying the time points of the series.
  \item \code{series_esd}: (Optional) A standardized version of the series.
  \item \code{zero_mean}: Logical. Indicates whether the series should be mean-centered.
  \item \code{standardized}: Logical. Indicates whether the series is standardized.
  \item \code{hessian}: Logical. If \code{TRUE}, the function computes the Hessian matrix for parameter estimation.
  \item \code{family}: The distribution family of the iAR model (one of "norm", "t", or "gamma").
  \item \code{df}: Degrees of freedom for the t-distribution (only for \code{family = "t"}).
  \item \code{sigma}: The scale parameter for the t-distribution (only for \code{family = "t"}).
  \item \code{mean}: The mean parameter for the gamma distribution (only for \code{family = "gamma"}).
  \item \code{variance}: The variance parameter for the gamma distribution (only for \code{family = "gamma"}).
}}

\item{...}{Additional arguments (unused).}
}
\value{
An updated \code{iAR} object with the following additional attributes:
  \itemize{
    \item \code{coef}: Estimated model coefficients.
    \item \code{loglik}: Log-likelihood value of the model.
    \item \code{summary}: A summary table containing parameter estimates, standard errors, and p-values.
    \item \code{sigma}: For t and gamma families, the estimated scale parameter.
    \item \code{mean}: For the gamma family, the estimated mean parameter.
    \item \code{variance}: For the gamma family, the estimated variance parameter.
  }
}
\description{
Maximum Likelihood Estimation for irregular autoregressive (iAR) models, supporting different distribution families:
normal (`iAR`), t (`iAR-T`), and gamma (`iAR-Gamma`).
}
\details{
This method estimates the parameters of an iAR model using the Maximum Likelihood Estimation (MLE) approach.
Depending on the chosen distribution family, the corresponding likelihood function is maximized:
\itemize{
  \item "norm" maximizes the likelihood for a normally-distributed series.
  \item "t" maximizes the likelihood for a t-distributed series.
  \item "gamma" maximizes the likelihood for a gamma-distributed series.
}
The function updates the \code{iAR} object with the estimated parameters, the log-likelihood value, and a summary
table that includes standard errors and p-values.
}
\examples{
# Example: Estimating parameters for a normal iAR model
library(iAR)
times <- 1:100
model <- iAR(family = "norm", times = times, coef = 0.9, hessian = TRUE)
model <- sim(model)  # Simulate the series
model <- loglik(model)  # Estimate parameters using MLE
print(model@coef)  # Access the estimated coefficients
print(model@loglik)  # Access the computed log-likelihood

}
\references{
\insertRef{Eyheramendy_2018}{iAR}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eb.R
\docType{data}
\name{eb}
\alias{eb}
\title{Eclipsing Binaries (Beta Lyrae)}
\format{
A data frame with 470 observations on the following 3 variables:
\describe{
  \item{t}{heliocentric Julian Day}
  \item{m}{magnitude}
  \item{merr}{measurement error of the magnitude (in mag).}
}
}
\usage{
eb
}
\description{
Time series of a Beta Lyrae variable star obtained from OGLE.
}
\details{
The frequency computed by GLS for this light curve is 1.510571586.
Catalogs and designations of this star:OGLE051951.22-694002.7
}
\examples{
data(eb)
f1=1.510571586
o1=phase(data=eb,f1=f1,twop=TRUE)
plot(o1@times_phased,o1@series_phased,pch=20)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/00_utilities.R
\name{utilities}
\alias{utilities}
\title{Utilities Class}
\usage{
utilities(
  times = integer(0),
  series = integer(0),
  series_esd = integer(0),
  times_phased = integer(0),
  series_phased = integer(0),
  series_esd_phased = integer(0),
  fitted_values = integer(0),
  residuals = integer(0),
  coef = integer(0),
  summary = list(),
  paired = integer(0)
)
}
\arguments{
\item{times}{A numeric vector storing the time points.}

\item{series}{A numeric vector representing the values of the time series.}

\item{series_esd}{A numeric vector representing the error standard deviations of the time series.}

\item{times_phased}{A numeric vector of phased times.}

\item{series_phased}{A numeric vector containing the time series values ordered by phase.}

\item{series_esd_phased}{A numeric vector containing the error standard deviations of the time series ordered by phase.}

\item{fitted_values}{A numeric vector containing the fitted values from the harmonic model.}

\item{residuals}{A numeric vector containing the residuals from the harmonic model.}

\item{coef}{A numeric vector containing the estimated coefficients of the harmonic model.}

\item{summary}{A summary object containing the harmonic model information.}

\item{paired}{Data Frame with the paired datasets.}
}
\description{
The `utilities` class is an S7 class designed to group utility functions 
that are not directly tied to other specific objects in the package. 
These include the functions `gentime`, `paringits`, `harmonicfit`, and `foldlc`.
}
\section{Description}{

This class acts as a container for standalone methods that perform independent 
operations within the package. By grouping them under a single class, the package 
achieves better modularity and organization, facilitating maintenance and extensibility.
}

\section{Available Methods}{

- `gentime`: Generates time points based on a specified statistical distribution.
- `paringits`: A method for pairing irregular time series.
- `harmonicfit`: A method for fitting harmonic models to data.
- `foldlc`: A method for folding light curves in time series analysis.
}

\examples{
# Create a utilities object

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/08_plot_fit.R
\name{plot_fit}
\alias{plot_fit}
\title{Plot Fit Method for unidata Objects}
\usage{
plot_fit(x,...)
}
\arguments{
\item{x}{An object of class `unidata` or `multidata`. This object must contain the 
time series in the `series` slot, the associated time points in the 
`times` slot, and the fitted values in the `fitted_values` slot.}

\item{...}{Additional arguments passed to the `plot.zoo` function for 
customizing the plot (e.g., titles, colors, etc.).}
}
\description{
This method visualizes the `unidata` and `multidata` objects by plotting both the 
original time series and the fitted values. The `unidata` (`multidata`) object is 
converted into a `zoo` object for plotting, and the fitted values are 
added as a secondary line with a different style and color.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/08_plot_forecast.R
\name{plot_forecast}
\alias{plot_forecast}
\title{Plot Forecast Method for unidata Objects}
\usage{
plot_forecast(x,...)
}
\arguments{
\item{x}{An object of class `unidata` or `multidata`. This object must contain the 
time series in the `series` slot, the associated time points in the 
`times` slot, the forecasted values in the `forecast` slot, and the 
forecast horizon in the `tAhead` slot.}

\item{...}{Additional arguments passed to the `plot.zoo` function for 
customizing the plot (e.g., titles, colors, etc.).}
}
\description{
This method visualizes the `unidata` and `multidata` objects by plotting both the 
original time series and the forecasted values. The `unidata` (`multidata`) object 
is converted into a `zoo` object for plotting, and the forecasted 
values are added as points with a different color.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dscut.R
\docType{data}
\name{dscut}
\alias{dscut}
\title{Delta Scuti}
\format{
A data frame with 116 observations on the following 3 variables:
\describe{
  \item{t}{heliocentric Julian Day}
  \item{m}{magnitude}
  \item{merr}{measurement error of the magnitude (in mag).}
}
}
\usage{
dscut
}
\description{
Time series of a Delta Scuti variable star obtained from HIPPARCOS.
}
\details{
The frequency computed by GLS for this light curve is 14.88558646.
Catalogs and designations of this star:
HD 1989: HD 199757
TYCHO-2 2000: TYC 7973-401-1
USNO-A2.0: USNO-A2 0450-39390397
HIP: HIP 103684
}
\examples{
data(dscut)
f1=14.88558646
o1=phase(data=dscut,f1=f1,twop=TRUE)
plot(o1@times_phased,o1@series_phased,pch=20)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cvnovar.R
\docType{data}
\name{cvnovar}
\alias{cvnovar}
\title{ZTF r-band Cataclysmic Variable/Nova}
\format{
A data frame with 65 observations on the following 3 variables:
\describe{
  \item{t}{heliocentric Julian Day - 2400000}
  \item{m}{magnitude}
  \item{merr}{measurement error standard deviations.}
}
}
\usage{
cvnovar
}
\description{
Time series of a cataclysmic variable/nova object observed in the r-band of the ZTF survey and processed by the ALeRCE broker.ZTF Object code: ZTF18aayzpbr
}
\examples{
data(cvnovar)
plot(cvnovar$t,cvnovar$m,type="l",ylab="",xlab="",col="red")
}
\references{
\insertRef{Forster_2021}{iAR}
}
\keyword{datasets}
